package com.ims.common.utils;

import com.ims.common.function.IFSFunction;
import lombok.extern.slf4j.Slf4j;
import org.apache.poi.hssf.usermodel.*;
import org.apache.poi.ss.formula.FormulaParseException;
import org.apache.poi.ss.formula.functions.FreeRefFunction;
import org.apache.poi.ss.formula.udf.AggregatingUDFFinder;
import org.apache.poi.ss.formula.udf.DefaultUDFFinder;
import org.apache.poi.ss.formula.udf.UDFFinder;
import org.apache.poi.ss.usermodel.*;

import java.io.IOException;
import java.math.BigDecimal;

@Slf4j
public class FormulaUtils {

	private static Workbook workbook = null;

	static {
		if (workbook == null) {
			workbook = new HSSFWorkbook();
		}

		String[] functionNames = { IFSFunction.FUNCTION_NAME } ;
		FreeRefFunction[] functionImpls = { new IFSFunction() } ;

		UDFFinder udfs = new DefaultUDFFinder( functionNames, functionImpls ) ;
		UDFFinder udfToolpack = new AggregatingUDFFinder( udfs ) ;

		workbook.addToolPack(udfToolpack);
	}

	/**
	 * 计算值
	 *
	 * @param formula Excel 中的公式，例如：MAX(56-FLOOR(20/6,1),2)
	 * @return
	 */
	public static double calculateFormula1(String formula) {
		if(formula.startsWith("=")){
			formula =formula.replace("=","");
		}
		HSSFWorkbook workbook = new HSSFWorkbook();
		workbook.addToolPack(new AggregatingUDFFinder(new DefaultUDFFinder(new String[]{IFSFunction.FUNCTION_NAME}, new FreeRefFunction[]{new IFSFunction()})));

		HSSFSheet sheet = workbook.createSheet();

		HSSFRow row = sheet.createRow(0);
		FormulaEvaluator formulaEvaluator = new HSSFFormulaEvaluator(workbook);

		// 这里必须新建一个对象，否则只有第一个 formula 才有效。查看 formulaEvaluator.evaluate 的源码。
		HSSFCell cell = row.createCell(0);
		cell.setCellType(CellType.FORMULA);
		try {
			cell.setCellFormula(formula);
			double value = formulaEvaluator.evaluate(cell).getNumberValue();
			return new BigDecimal(value).setScale(4, BigDecimal.ROUND_CEILING).doubleValue();
		} catch (FormulaParseException e) {
			System.out.print(e.getMessage());
			return 0;
		} finally {
			try {
				workbook.close();
			} catch (IOException e) {
				e.printStackTrace();
			}
		}
	}


	/**
	 * 计算结果返回double类型
	 * @param formula
	 * @return
	 */
	public static double calculateFormula(String formula){
		try {
			FormulaEvaluator evaluator = workbook.getCreationHelper().createFormulaEvaluator();

			Sheet sheet = workbook.createSheet();
			Row row = sheet.createRow(0);
			Cell cell = row.createCell(0);
			cell.setCellFormula(formula);
			cell.setCellType(CellType.FORMULA);

			double value = evaluator.evaluate(cell).getNumberValue() ;
			return new BigDecimal(value).setScale(4, BigDecimal.ROUND_HALF_UP).doubleValue();
		} catch (Exception e) {
			e.printStackTrace();
			log.error("FormulaUtils==excel公式解析异常： ", e.getMessage());

		}

		return 0.0;
	}


	/**
	 * 计算结果返回String类型
	 * @param formula
	 * @return
	 */
	public static String calculateFormulaString(String formula){
		try {
			FormulaEvaluator evaluator = workbook.getCreationHelper().createFormulaEvaluator();

			Sheet sheet = workbook.createSheet();
			Row row = sheet.createRow(0);
			Cell cell = row.createCell(0);
			cell.setCellFormula(formula);
			cell.setCellType(CellType.FORMULA);

			String  value = evaluator.evaluate(cell).getStringValue();
			log.info("计算结果："+value+" = "+formula);
			return value;
		} catch (Exception e) {
			e.printStackTrace();
			log.error("FormulaUtils==excel公式解析异常： ", e.getMessage());
		}

		return "";
	}


	public static void main(String[] args) {
		Double va = calculateFormula("IF(0.0>0,-0.0*2/0.0+10,0.0*2/0.0+10)");
		System.out.println(va);
	}



}
