package com.ims.common.utils;

import javax.crypto.Cipher;
import java.security.*;


public class RSAUtils {
	private static final String ALGORITHM = "RSA";

	private static KeyPair keyPair ;
	// 生成密钥对
	public static KeyPair generateKeyPair(int keySize) throws Exception {
		if(null == keyPair){
			KeyPairGenerator keyPairGenerator = KeyPairGenerator.getInstance(ALGORITHM);
			SecureRandom secureRandom = new SecureRandom();
			keyPairGenerator.initialize(keySize, secureRandom);
			keyPair = keyPairGenerator.generateKeyPair();
		}
		return keyPair;
	}



	// 公钥加密
	public static byte[] encrypt(byte[] data, PublicKey publicKey) throws Exception {
		Cipher cipher = Cipher.getInstance(ALGORITHM);
		cipher.init(Cipher.ENCRYPT_MODE, publicKey);
		return cipher.doFinal(data);
	}

	// 私钥解密
	public static byte[] decrypt(byte[] encryptedData, PrivateKey privateKey) throws Exception {
		Cipher cipher = Cipher.getInstance(ALGORITHM);
		cipher.init(Cipher.DECRYPT_MODE, privateKey);
		return cipher.doFinal(encryptedData);
	}

	// 数字签名
	public static byte[] sign(byte[] data, PrivateKey privateKey) throws Exception {
		Signature signature = Signature.getInstance("SHA256withRSA");
		signature.initSign(privateKey);
		signature.update(data);
		return signature.sign();
	}

	// 验证数字签名
	public static boolean verify(byte[] data, byte[] signatureBytes, PublicKey publicKey) throws Exception {
		Signature signature = Signature.getInstance("SHA256withRSA");
		signature.initVerify(publicKey);
		signature.update(data);
		return signature.verify(signatureBytes);
	}

	public static KeyPair getKeyPair() throws Exception {
		if(null == keyPair){
			keyPair = generateKeyPair(2048);
		}
		return keyPair;
	}

	public static void main1(String[] args) throws Exception {
		String originalMessage = "Hello, RSA!";

		// 生成密钥对
		KeyPair keyPair = generateKeyPair(2048);
		PublicKey publicKey = keyPair.getPublic();
		PrivateKey privateKey = keyPair.getPrivate();

		// 加密
		byte[] encryptedData = encrypt(originalMessage.getBytes(), publicKey);

		// 解密
		byte[] decryptedData = decrypt(encryptedData, privateKey);

		// 数字签名
		byte[] signature = sign(originalMessage.getBytes(), privateKey);

		// 验证数字签名
		boolean isSignatureValid = verify(originalMessage.getBytes(), signature, publicKey);

		System.out.println("原始消息：" + originalMessage);
		System.out.println("加密后的数据：" + new String(encryptedData));
		System.out.println("解密后的数据：" + new String(decryptedData));
		System.out.println("数字签名验证结果：" + isSignatureValid);
	}

















}
