package com.ims.eval.controller;


import com.ims.common.utils.StringUtils;
import com.ims.eval.cache.CacheContext;
import com.ims.eval.config.CustomException;
import com.ims.eval.entity.BinStage;
import com.ims.eval.entity.IndicatorType;
import com.ims.eval.entity.dto.result.R;
import com.ims.eval.service.IBinStageService;
import com.ims.eval.service.IIndicatorTypeService;
import io.swagger.annotations.ApiOperation;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.*;

import java.util.Arrays;
import java.util.List;
import java.util.stream.Collectors;

/**
 * <p>
 * 经营阶段分类 前端控制器
 * </p>
 *
 * @author wang
 * @since 2023-03-12
 */
@RestController
@RequestMapping("//bin-stage")
public class BinStageController {

	@Autowired
	private IBinStageService binStageService;

	@Autowired
	private CacheContext cache;


	/**
	 * 查询
	 *
	 * @param stageName  名称
	 * @param type       类型
	 * @return
	 */
	//@ImsPreAuth("eval:binStage:view")
	@GetMapping(value = "list")
	public R list(@RequestParam(value = "stageName", required = false) String stageName,
				  @RequestParam(value = "type", required = false) String type) {
		List<BinStage> list = CacheContext.bseList;
		if(StringUtils.isNotEmpty(type)){
			 list =list.stream().filter(t->!t.getType().equals(type)).collect(Collectors.toList());
		}

		if(StringUtils.isNotEmpty(stageName)){
			list =list.stream().filter(t->t.getStageName().contains(stageName)).collect(Collectors.toList());
		}


		return R.ok().data(list);
	}


	/**
	 * 添加
	 *
	 * @param binStage
	 * @return
	 */

	//@ImsPreAuth("eval:binStage:edit")
	@PostMapping(value = "/save")
	@ApiOperation(value = "新增(修改)", notes = "新增（修改）")
	public R addAll(@RequestBody BinStage binStage) {

		try {
			boolean b = binStageService.saveOrUpdate(binStage);
			if (b) {
				cache.initBinStage();
				return R.ok().data(b);
			} else {
				return R.error().data("保存失败！");
			}
		} catch (CustomException e){
			return R.customError(e.getMessage()).data("失败！");
		}
	}


	/**
	 * 批量删除
	 *
	 * @param ids
	 * @return
	 */
	//@ImsPreAuth("eval:binStage:remove")
	@PostMapping(value = "/remove/{ids}")
	@ApiOperation(value = "删除", notes = "删除")
	public R deleteAll(@PathVariable("ids") String ids) {
		String[] strings = ids.split(",");
		boolean b = binStageService.removeByIds(Arrays.asList(strings));
		if (b) {
			cache.initBinStage();
			return R.ok().data(b);
		} else {
			return R.error().data("删除失败！");
		}
	}

}
