package com.ims.eval.controller;


import com.baomidou.mybatisplus.core.metadata.IPage;
import com.ims.eval.cache.CacheContext;
import com.ims.eval.config.CustomException;
import com.ims.eval.entity.dto.result.R;
import com.ims.eval.entity.DataDictionary;
import com.ims.eval.service.IDataDictionaryService;
import io.swagger.annotations.ApiOperation;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.*;

import java.util.Arrays;
import java.util.List;

/**
 * <p>
 * 数据字典表 前端控制器
 * </p>
 *
 * @author wang
 * @since 2023-02-26
 */
@RestController
@RequestMapping("//data-dictionary")
public class DataDictionaryController {

	@Autowired
	private IDataDictionaryService dataDictionaryService;

	@Autowired
	private CacheContext cache;

	/**
	 * 查询
	 *
	 * @param id       主键ID
	 * @param dataKey  数据key
	 * @param keyName  名称
	 * @param superKey 父类key
	 * @return
	 */
	//@ImsPreAuth("eval:dataDictionary:view")
	@GetMapping(value = "list")
	public R list(@RequestParam(value = "id", required = false) String id,
				  @RequestParam(value = "dataKey", required = false) String dataKey,
				  @RequestParam(value = "keyName", required = false) String keyName,
				  @RequestParam(value = "superKey", required = false) String superKey) {
		List<DataDictionary> list = dataDictionaryService.list(id, dataKey, keyName, superKey);
		return R.ok().data(list);
	}


	/**
	 * 分页查询
	 * @param pageNum
	 * @param pageSize
	 * @param dataKey
	 * @param keyName
	 * @param superKey
	 * @return
	 */
	//@ImsPreAuth("eval:dataDictionary:view")
	@GetMapping(value = "listPage")
	public R listPage(@RequestParam(value = "pageNum") Integer pageNum,
					  @RequestParam(value = "pageSize") Integer pageSize,
					  @RequestParam(value = "dataKey", required = false) String dataKey,
					  @RequestParam(value = "keyName", required = false) String keyName,
					  @RequestParam(value = "superKey", required = false) String superKey) {
		IPage<DataDictionary> list = dataDictionaryService.listPage(pageNum,pageSize, dataKey, keyName,superKey);
		return R.ok().data(list);
	}


	/**
	 * 添加
	 *
	 * @param dictionary
	 * @return
	 */

	//@ImsPreAuth("eval:dataDictionary:edit")
	@PostMapping(value = "/save")
	@ApiOperation(value = "新增(修改)", notes = "新增（修改）")
	public R addAll(@RequestBody DataDictionary dictionary) {

		try {
			boolean b = dataDictionaryService.saveOrUpdate(dictionary);
			if (b) {
				cache.initDataDictionary();
				return R.ok().data(b);
			} else {
				return R.error().data("保存失败！");
			}
		} catch (CustomException e){
			return R.customError(e.getMessage()).data("失败！");
		}
	}


	/**
	 * 批量删除
	 *
	 * @param ids
	 * @return
	 */
	//@ImsPreAuth("eval:dataDictionary:remove")
	@PostMapping(value = "/remove/{ids}")
	@ApiOperation(value = "删除", notes = "删除")
	public R deleteAll(@PathVariable("ids") String ids) {
		String[] strings = ids.split(",");
		boolean b = dataDictionaryService.removeByIds(Arrays.asList(strings));
		if (b) {
			cache.initDataDictionary();
			return R.ok().data(b);
		} else {
			return R.error().data("删除失败！");
		}
	}


}
