package com.ims.eval.controller;

import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.ims.common.utils.StringUtils;
import com.ims.eval.config.CustomException;
import com.ims.eval.entity.EvaluationDeptRating;
import com.ims.eval.entity.OrganizationStructure;
import com.ims.eval.entity.dto.result.R;
import com.ims.eval.service.IEvaluationDeptRatingService;
import com.ims.eval.service.IOrganizationStructureService;
import com.ims.eval.util.ExcelUtil;
import lombok.extern.slf4j.Slf4j;
import org.springframework.web.bind.annotation.*;
import org.springframework.web.multipart.MultipartFile;

import javax.annotation.Resource;
import javax.servlet.http.HttpServletRequest;
import java.io.InputStream;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.Date;
import java.util.List;

/**
 * 本部部门绩效考核评级
 *
 * @author hlf
 * @date 2023/4/28 16:22
 * 文件说明：
 */
@Slf4j
@RestController
@RequestMapping("//evaluation-dept-rating")
public class DepartmentPerformanceAppraisalRatingController {

	@Resource
	private IEvaluationDeptRatingService evaluationDeptRatingService;

	@Resource
	private IOrganizationStructureService organizationStructureService;

	@Resource
	private HttpServletRequest request;

	/**
	 * 部门绩效考核评级列表信息（分页）
	 *
	 * @param pageNum  当前记录起始索引
	 * @param pageSize 每页显示记录数
	 * @param deptName 部门名称
	 * @param annual   月度
	 * @return 结果
	 */
	@GetMapping(value = "/list")
	public R list(
		@RequestParam(value = "pageNum") Integer pageNum,
		@RequestParam(value = "pageSize") Integer pageSize,
		@RequestParam(value = "deptId") String deptId,
		@RequestParam(value = "deptName", required = false) String deptName,
		@RequestParam(value = "annual", required = false) String annual) {
		IPage<EvaluationDeptRating> list = evaluationDeptRatingService.listPage(pageNum, pageSize, deptId, deptName, annual);
		return R.ok().data(list);
	}

	/**
	 * 部门绩效考核评级信息
	 *
	 * @return 结果
	 */
	@GetMapping(value = "/listAll")
	public R listAll(@RequestParam(value = "deptId") String deptId,
					 @RequestParam(value = "deptName", required = false) String deptName,
					 @RequestParam(value = "annual", required = false) String annual) {
		List<EvaluationDeptRating> list = evaluationDeptRatingService.list(deptId, deptName, annual,request);
		return R.ok().data(list);
	}

	/**
	 * 新增/修改部门绩效考核评级信息
	 *
	 * @param evaluationDeptRating 部门绩效考核评级实体
	 * @return 结果
	 */
	@PostMapping(value = "/save")
	public R addAll(@RequestBody EvaluationDeptRating evaluationDeptRating) {
		try {
			boolean b = evaluationDeptRatingService.saveOrUpdate(evaluationDeptRating);
			if (b) {
				return R.ok().data(b);
			} else {
				return R.error("已存在" + evaluationDeptRating.getAnnual() + "年" + evaluationDeptRating.getDeptName() + "数据，保存失败！");
			}
		} catch (CustomException e) {
			return R.customError(e.getMessage()).data("失败！");
		}
	}

	/**
	 * 批量删除部门绩效考核评级信息
	 *
	 * @param ids 主键s
	 * @return 结果
	 */
	@PostMapping(value = "/removeAll/{ids}")
	public R deleteAll(@PathVariable("ids") String ids) {
		try {
			String[] strings = ids.split(",");
			boolean b = evaluationDeptRatingService.removeByIds(Arrays.asList(strings));
			if (b) {
				return R.ok().data(b);
			} else {
				return R.error("删除失败！");
			}
		} catch (CustomException e) {
			return R.customError(e.getMessage()).data("失败！");
		}
	}

	/**
	 * 导入
	 *
	 * @param file 文件
	 * @return 结果
	 */
	@PostMapping(value = "/import")
	public R importData(@RequestParam("file") MultipartFile file) {
		if (!file.isEmpty()) {
			try {
				//获取原始的文件名
				String originalFilename = file.getOriginalFilename();
				//获取文件类型
				String fileType = originalFilename.substring(originalFilename.lastIndexOf(".") + 1, originalFilename.length());
				//默认从第一行开始读取
				int startRows = 1;
				//获取输入流
				InputStream is = file.getInputStream();
				List<EvaluationDeptRating> evaluationDeptRatingList = new ArrayList<>();
				//Excel导入导出的单元类
				List<String[]> strings = ExcelUtil.readData(fileType, startRows, true, is);
				//遍历Excel表每一行的数据
				for (String[] str : strings) {
					EvaluationDeptRating evaluationDeptRating = new EvaluationDeptRating();
					evaluationDeptRating.setAnnual(str[2]);
					evaluationDeptRating.setJan(str[3]);
					evaluationDeptRating.setJanContent(str[4]);
					evaluationDeptRating.setFeb(str[5]);
					evaluationDeptRating.setFebContent(str[6]);
					evaluationDeptRating.setMar(str[7]);
					evaluationDeptRating.setMarContent(str[8]);
					evaluationDeptRating.setApr(str[9]);
					evaluationDeptRating.setAprContent(str[10]);
					evaluationDeptRating.setMay(str[11]);
					evaluationDeptRating.setMayContent(str[12]);
					evaluationDeptRating.setJun(str[13]);
					evaluationDeptRating.setJunContent(str[14]);
					evaluationDeptRating.setJul(str[15]);
					evaluationDeptRating.setJulContent(str[16]);
					evaluationDeptRating.setAug(str[17]);
					evaluationDeptRating.setAugContent(str[18]);
					evaluationDeptRating.setSep(str[19]);
					evaluationDeptRating.setSepContent(str[20]);
					evaluationDeptRating.setOct(str[21]);
					evaluationDeptRating.setOctContent(str[22]);
					evaluationDeptRating.setNov(str[23]);
					evaluationDeptRating.setNovContent(str[24]);
					evaluationDeptRating.setDece(str[25]);
					evaluationDeptRating.setDeceContent(str[26]);
					evaluationDeptRating.setAcount(str[27]);
					evaluationDeptRating.setCcount(str[28]);
					List<OrganizationStructure> organizationStructureList = organizationStructureService.getTree("23031001", 1, "140");
					OrganizationStructure organizationStructure = organizationStructureList.get(0).getChildren().stream().filter(item -> item.getId().equals(str[0])).findFirst().orElse(null);
					if (null != organizationStructure) {
						evaluationDeptRating.setDeptId(organizationStructure.getId());
						evaluationDeptRating.setDeptName(organizationStructure.getName());
						QueryWrapper<EvaluationDeptRating> qw = new QueryWrapper<>();
						if (StringUtils.isNotEmpty(evaluationDeptRating.getDeptId())) {
							qw.lambda().eq(EvaluationDeptRating::getDeptId, evaluationDeptRating.getDeptId());
						}
						if (StringUtils.isNotEmpty(evaluationDeptRating.getAnnual())) {
							qw.lambda().eq(EvaluationDeptRating::getAnnual, evaluationDeptRating.getAnnual());
						}
						List<EvaluationDeptRating> objList = evaluationDeptRatingService.list(qw);
						if (objList.size() == 1) {//修改
							evaluationDeptRating.setId(objList.get(0).getId());
							evaluationDeptRating.setUpdateTime(new Date());
							evaluationDeptRatingList.add(evaluationDeptRating);
						} else if (objList.size() == 0) {//新增
							evaluationDeptRating.setCreateTime(new Date());
							evaluationDeptRatingList.add(evaluationDeptRating);
						}
					}
				}
				boolean b = evaluationDeptRatingService.saveOrUpdateBatch(evaluationDeptRatingList);
				if (b) {
					return R.ok().data(b);
				} else {
					return R.error("导入失败！");
				}
			} catch (Exception e) {
				log.error("错误", e);
				return R.customError(e.getMessage()).data("失败！");
			}
		}
		return R.error("上传文件为空！");
	}
}
