package com.ims.eval.controller;

import com.alibaba.fastjson.JSONObject;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.ims.eval.config.CustomException;
import com.ims.eval.entity.EvaluationRevision;
import com.ims.eval.entity.OrganizationEvaluationRule;
import com.ims.eval.entity.dto.response.ResponsibilityIndicatorInfoResDTO;
import com.ims.eval.entity.dto.result.R;
import com.ims.eval.service.IEvaluationRevisionService;
import com.ims.eval.service.IOrganizationEvaluationRuleService;
import com.ims.eval.service.IResponsibilityIndicatorInfoService;
import lombok.extern.slf4j.Slf4j;
import org.springframework.http.HttpHeaders;
import org.springframework.http.MediaType;
import org.springframework.http.ResponseEntity;
import org.springframework.web.bind.annotation.*;
import org.springframework.web.multipart.MultipartFile;

import javax.annotation.Resource;
import javax.servlet.http.HttpServletRequest;
import java.util.Arrays;
import java.util.Calendar;
import java.util.List;
import java.util.Map;

/**
 * 单位考评目标修订
 *
 * @author hlf
 * @date 2023/5/23 9:54
 * 文件说明：
 */
@Slf4j
@RestController
@RequestMapping("//evaluation-revision")
public class EvaluationRevisionController {

	@Resource
	private IEvaluationRevisionService evaluationRevisionService;

	@Resource
	private IOrganizationEvaluationRuleService organizationEvaluationRuleService;

	@Resource
	private IResponsibilityIndicatorInfoService responsibilityIndicatorInfoService;

	@Resource
	private HttpServletRequest request;

	/**
	 * 单位考评目标修订列表信息（分页）
	 *
	 * @param pageNum            当前记录起始索引
	 * @param pageSize           每页显示记录数
	 * @param id                 主键
	 * @param responsibilityCode 业务编码
	 * @param cycleUnit          周期单位
	 * @param checkCycle         周期
	 * @param beginDate          考核开始时间
	 * @param endDate            考核截止时间
	 * @param stage              审批状态
	 * @param createBy           创建者
	 * @param year               年
	 * @param month              月
	 * @param des                描述
	 * @return 结果
	 */
	@GetMapping(value = "/list")
	public R list(@RequestParam(value = "pageNum") Integer pageNum,
				  @RequestParam(value = "pageSize") Integer pageSize,
				  @RequestParam(value = "id", required = false) String id,
				  @RequestParam(value = "responsibilityCode", required = false) String responsibilityCode,
				  @RequestParam(value = "cycleUnit", required = false) String cycleUnit,
				  @RequestParam(value = "checkCycle", required = false) List<String> checkCycle,
				  @RequestParam(value = "beginDate", required = false) String beginDate,
				  @RequestParam(value = "endDate", required = false) String endDate,
				  @RequestParam(value = "stage", required = false) String stage,
				  @RequestParam(value = "createBy", required = false) String createBy,
				  @RequestParam(value = "year", required = false) String year,
				  @RequestParam(value = "month", required = false) String month,
				  @RequestParam(value = "des", required = false) String des) {
		IPage<EvaluationRevision> list = evaluationRevisionService.listPage(pageNum, pageSize, id, responsibilityCode, cycleUnit, checkCycle, beginDate, endDate, stage, createBy, year, month, des);
		return R.ok().data(list);
	}

	/**
	 * 单位考评目标修订信息
	 *
	 * @return 结果
	 */
	@GetMapping(value = "/listAll")
	public R listAll() {
		List<EvaluationRevision> list = evaluationRevisionService.list();
		return R.ok().data(list);
	}

	/**
	 * 获取本年度所有单位
	 *
	 * @return 结果
	 */
	@GetMapping(value = "/getUnitAll")
	public R getUnitAll() {
		Calendar calendar = Calendar.getInstance();
		//获取当前年
		int year = calendar.get(Calendar.YEAR);
		QueryWrapper<OrganizationEvaluationRule> qw = new QueryWrapper<>();
		qw.lambda().eq(OrganizationEvaluationRule::getOrganizationType, "DWKP");
		qw.lambda().eq(OrganizationEvaluationRule::getEvaluationCycle, "NDKP");
		qw.lambda().eq(OrganizationEvaluationRule::getYear, String.valueOf(year));
		qw.lambda().eq(OrganizationEvaluationRule::getDelFlag, false);
		qw.lambda().eq(OrganizationEvaluationRule::getIsCheck, true);
		List<OrganizationEvaluationRule> list = organizationEvaluationRuleService.list(qw);
		return R.ok().data(list);
	}

	/**
	 * 新增/修改单位考评目标修订信息
	 *
	 * @param evaluationRevision 单位考评目标修订业务实体
	 * @return 结果
	 */
	@PostMapping(value = "/save")
	public R addAll(@RequestBody EvaluationRevision evaluationRevision) {
		try {
			boolean b = evaluationRevisionService.saveOrUpdate(evaluationRevision);
			if (b) {
				return R.ok().data(b);
			} else {
				return R.error("保存失败！");
			}
		} catch (CustomException e) {
			return R.customError(e.getMessage()).data("失败！");
		}
	}

	/**
	 * 批量删除单位考评目标修订信息
	 *
	 * @param ids 主键s
	 * @return 结果
	 */
	@PostMapping(value = "/removeAll/{ids}")
	public R deleteAll(@PathVariable("ids") String ids) {
		try {
			String[] strings = ids.split(",");
			boolean b = evaluationRevisionService.removeByIds(Arrays.asList(strings));
			if (b) {
				return R.ok().data(b);
			} else {
				return R.error("删除失败，流程已启动或流程未结束！");
			}
		} catch (CustomException e) {
			return R.customError(e.getMessage()).data("失败！");
		}
	}

	/**
	 * 详情-头部信息
	 *
	 * @param id 业务主键
	 * @return 结果
	 */
	@GetMapping(value = "/detailsHead/{id}")
	public R detailsHead(@PathVariable("id") String id) {
		EvaluationRevision evaluationRevision = evaluationRevisionService.getById(id);
		return R.ok().data(evaluationRevision);
	}

	/**
	 * 详情
	 *
	 * @param reviseId              修订业务主键
	 * @param indicatorId           indicatorId
	 * @param organizationShortName organizationShortName
	 * @param binSection            binSection
	 * @param binStage              binStage
	 * @return 结果
	 */
	@GetMapping(value = "/details")
	public R details(@RequestParam(value = "reviseId", required = false) String reviseId,
					 @RequestParam(value = "indicatorId", required = false) String indicatorId,
					 @RequestParam(value = "organizationShortName", required = false) String organizationShortName,
					 @RequestParam(value = "binSection", required = false) String binSection,
					 @RequestParam(value = "binStage", required = false) String binStage) {
		try {
			Map list = evaluationRevisionService.getResponsibilityIndicatorInfoList(reviseId, indicatorId, organizationShortName, binSection, binStage, request);
			return R.ok().data(list);
		} catch (CustomException e1) {
			e1.printStackTrace();
			return R.error().customError(e1.getMessage());
		} catch (Exception e) {
			e.printStackTrace();
			return R.error().customError("查询失败");
		}
	}

	/**
	 * 详情-修改
	 *
	 * @param jsonObjects 详情数据
	 * @return 结果
	 */
	@PostMapping(value = "/updateDetails")
	public R updateDetails(@RequestBody List<JSONObject> jsonObjects) {

		try {
			boolean b = responsibilityIndicatorInfoService.revisionResponsibilityIndicatorInfo(jsonObjects);
			if (b) {
				return R.ok().data(b);
			} else {
				return R.error("保存失败！");
			}
		} catch (Exception e) {
			log.error("错误", e);
			return R.customError(e.getMessage()).data("失败！");
		}
	}

	/**
	 * 详情-弹框数据
	 *
	 * @param id 考评目标主键
	 * @return 结果
	 */
	@GetMapping(value = "/popUpData")
	public R popUpData(@RequestParam(value = "id") String id) {
		List<ResponsibilityIndicatorInfoResDTO> list = responsibilityIndicatorInfoService.selectPopUpData(id, "", "0-0");
		return R.ok().data(list);
	}

	/**
	 * 修改状态
	 *
	 * @param id    主键
	 * @param stage 状态
	 * @return 结果
	 */
	@PostMapping(value = "/targetStart")
	public R targetStart(@RequestParam(value = "id") String id,
						 @RequestParam(value = "stage") String stage) {
		try {
			EvaluationRevision evaluationRevision = evaluationRevisionService.getById(id);
			evaluationRevision.setStage(stage);
			boolean b = evaluationRevisionService.updateById(evaluationRevision);
			if (b) {
				return R.ok().data(b);
			} else {
				return R.error("失败！");
			}
		} catch (CustomException e) {
			return R.customError(e.getMessage()).data("失败！");
		}
	}

	/**
	 * 详情-导出
	 *
	 * @param reviseId    业务主键
	 * @param indicatorId indicatorId
	 * @param binSection  binSection
	 * @param binStage    binStage
	 * @return 结果
	 * @throws Exception 异常
	 */
	@GetMapping("/download-excel")
	public ResponseEntity<byte[]> downloadExcel(
		@RequestParam(value = "reviseId", required = false) String reviseId,
		@RequestParam(value = "indicatorId", required = false) String indicatorId,
		@RequestParam(value = "binSection", required = false) String binSection,
		@RequestParam(value = "binStage", required = false) String binStage) throws Exception {
		byte[] excelBytes = evaluationRevisionService.downloadExcel(reviseId, indicatorId, binSection, binStage, request);
		HttpHeaders headers = new HttpHeaders();
		headers.setContentType(MediaType.APPLICATION_OCTET_STREAM);
		headers.setContentDispositionFormData("attachment", "example.xlsx");
		return ResponseEntity.ok().headers(headers).body(excelBytes);
	}

	/**
	 * 详情-导入
	 *
	 * @param file    文件
	 * @param request request
	 * @return 结果
	 * @throws Exception 异常
	 */
	@PostMapping("/import-excel")
	public R importExcel(@RequestParam("file") MultipartFile file, HttpServletRequest request) {
		boolean b = evaluationRevisionService.importExcel(file, request);
		if (b) {
			return R.ok().data(b);
		} else {
			return R.error().data("上传失败");
		}
	}
}
