package com.ims.eval.controller;


import com.ims.eval.config.CustomException;
import com.ims.eval.entity.dto.request.IndicatorDictionaryDTO;
import com.ims.eval.entity.dto.result.R;
import com.ims.eval.entity.IndicatorDictionary;
import com.ims.eval.service.IIndicatorDictionaryService;
import io.swagger.annotations.ApiOperation;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.*;

import java.util.Arrays;
import java.util.List;

/**
 * <p>
 * 指标字典表 前端控制器
 * </p>
 *
 * @author wang
 * @since 2023-02-26
 */
@RestController
@RequestMapping("//indicator-dictionary")
public class IndicatorDictionaryController {


	@Autowired
	private IIndicatorDictionaryService indicatorDictionaryService;

	/**
	 * @param id          主键ID
	 * @param indicatorId 指标ID
	 * @param optionCode  选项编码
	 * @param optionName  选项名称
	 * @return
	 */
	//@ImsPreAuth("eval:indicator:view")
	@RequestMapping(value = "list")
	public List<IndicatorDictionary> list(@RequestParam(value = "id", required = false) String id,
										  @RequestParam(value = "indicatorId", required = false) String indicatorId,
										  @RequestParam(value = "optionCode", required = false) String optionCode,
										  @RequestParam(value = "optionName", required = false) String optionName) {
		List<IndicatorDictionary> list = indicatorDictionaryService.list(id, indicatorId, optionCode, optionName);
		return list;
	}


	/**

	 * @param indicatorId 指标ID
	 * @param optionCode  选项编码
	 * @param optionName  选项名称
	 * @return
	 */
	//@ImsPreAuth("eval:indicator:view")
	@RequestMapping(value = "listGroup")
	public List<IndicatorDictionaryDTO> listGroup(
										  @RequestParam(value = "indicatorId", required = false) String indicatorId,
										  @RequestParam(value = "optionCode", required = false) String optionCode,
										  @RequestParam(value = "optionName", required = false) String optionName) {
		List<IndicatorDictionaryDTO> list = indicatorDictionaryService.list(indicatorId, optionCode, optionName);
		return list;
	}


	/**
	 * 添加
	 *
	 * @param dictionary
	 * @return
	 */

	//@ImsPreAuth("eval:dataDictionary:edit")
	@PostMapping(value = "/save")
	@ApiOperation(value = "新增(修改)", notes = "新增（修改）")
	public R addAll(@RequestBody IndicatorDictionary dictionary) {

		try {
			boolean b = indicatorDictionaryService.saveOrUpdate(dictionary);
			if (b) {
				return R.ok().data(b);
			} else {
				return R.error().data("保存失败！");
			}
		} catch (CustomException e){
			return R.customError(e.getMessage()).data("失败！");
		}
	}

	/**
	 * 添加
	 *
	 * @param dictionarys
	 * @return
	 */

	//@ImsPreAuth("eval:dataDictionary:edit")
	@PostMapping(value = "/saveBatch")
	@ApiOperation(value = "批量新增", notes = "批量新增")
	public R dictionarySaveBatch(@RequestBody List<IndicatorDictionaryDTO> dictionarys) {

		try {
			boolean b = indicatorDictionaryService.dictionarySaveBatch(dictionarys);
			if (b) {
				return R.ok().data(b);
			} else {
				return R.error().data("保存失败！");
			}
		} catch (CustomException e){
			return R.customError(e.getMessage()).data("失败！");
		}catch (Exception e1){
			return R.customError("操作失败").data("失败！");
		}
	}

	/**
	 * 批量删除
	 *
	 * @param ids
	 * @return
	 */
	//@ImsPreAuth("eval:dataDictionary:remove")
	@PostMapping(value = "/remove/{ids}")
	@ApiOperation(value = "删除", notes = "删除")
	public R deleteAll(@PathVariable("ids") String ids) {
		String[] strings = ids.split(",");
		boolean b = indicatorDictionaryService.removeByIds(Arrays.asList(strings));
		if (b) {
			return R.ok().data(b);
		} else {
			return R.error().data("删除失败！");
		}
	}

}
