package com.ims.eval.controller;


import com.alibaba.fastjson.JSONObject;
import com.ims.eval.config.CustomException;
import com.ims.eval.entity.OrganizationEvaluationInfo;
import com.ims.eval.entity.ResponsibilityIndicatorInfo;
import com.ims.eval.entity.dto.request.ResponsibilityIndicatorInfoUpdateDTO;
import com.ims.eval.entity.dto.result.R;
import com.ims.eval.service.IResponsibilityIndicatorInfoService;
import com.ims.eval.util.ExcelUtil;
import io.swagger.annotations.ApiOperation;
import lombok.extern.log4j.Log4j2;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.http.HttpHeaders;
import org.springframework.http.MediaType;
import org.springframework.http.ResponseEntity;
import org.springframework.web.bind.annotation.*;
import org.springframework.web.multipart.MultipartFile;

import javax.servlet.http.HttpServletRequest;
import java.io.InputStream;
import java.util.ArrayList;
import java.util.List;
import java.util.Map;

/**
 * <p>
 * 责任书指标明细

 前端控制器
 * </p>
 *
 * @author wang
 * @since 2023-03-08
 */
@Slf4j
@RestController
@RequestMapping("//responsibility-indicator-info")
public class ResponsibilityIndicatorInfoController {



	@Autowired
	private IResponsibilityIndicatorInfoService responsibilityIndicatorInfoService;


	@Autowired
	private HttpServletRequest request;

	/**
	 *
	 * @param deptResponsibilityIds 目标责任书id
	 * @return
	 */
	//@ImsPreAuth("eval:responsibilityIndicatorInfo:view")
	@GetMapping(value = "planValueList")
	public R planValueList(
				  @RequestParam(value = "deptResponsibilityIds", required = false) List<String> deptResponsibilityIds,
				  @RequestParam(value = "dept", required = false) String dept) {
		try {
			Map list = responsibilityIndicatorInfoService.planValueList(deptResponsibilityIds, dept,request);
			return R.ok().data(list);
		} catch (CustomException e) {
			e.printStackTrace();
			return R.customError(e.getMessage());
		}
	}





	/**
	 * 添加
	 *
	 * @param indicatorInfo
	 * @return
	 */

	//@ImsPreAuth("eval:responsibilityIndicatorInfo:edit")
	@PostMapping(value = "/save")
	@ApiOperation(value = "新增(修改)", notes = "新增（修改）")
	public R add(@RequestBody ResponsibilityIndicatorInfo indicatorInfo) {

		try {
			boolean b = responsibilityIndicatorInfoService.saveOrUpdate(indicatorInfo);
			if (b) {
				return R.ok().data(b);
			} else {
				return R.error().data("保存失败！");
			}
		} catch (CustomException e){
			return R.customError(e.getMessage()).data("失败！");
		}
	}


	/**
	 * 修改
	 *
	 * @param indicatorInfos
	 * @return
	 */

	//@ImsPreAuth("eval:responsibilityIndicatorInfo:edit")
	@PostMapping(value = "/saveDto")
	@ApiOperation(value = "新增(修改)", notes = "新增（修改）")
	public R saveDto(@RequestBody ResponsibilityIndicatorInfoUpdateDTO indicatorInfos) {

		try {
			boolean b = responsibilityIndicatorInfoService.saveOrUpdateDto(indicatorInfos);
			if (b) {
				return R.ok().data(b);
			} else {
				return R.error().data("保存失败！");
			}
		} catch (CustomException e){
			return R.customError(e.getMessage()).data("失败！");
		}
	}


	/**
	 * 批量修改
	 *
	 * @param indicatorInfos
	 * @return
	 */

	//@ImsPreAuth("eval:responsibilityIndicatorInfo:edit")
	@PostMapping(value = "/saveBatchDto")
	@ApiOperation(value = "批量新增(修改)", notes = "批量新增（修改）")
	public R saveBatchDto(@RequestBody List<ResponsibilityIndicatorInfoUpdateDTO> indicatorInfos) {

		try {
			boolean b = responsibilityIndicatorInfoService.saveBatchDto(indicatorInfos);
			if (b) {
				return R.ok().data(b);
			} else {
				return R.error().data("保存失败！");
			}
		} catch (CustomException e){
			return R.customError(e.getMessage()).data("失败！");
		}
	}


	@PostMapping(value = "/import")
	@ResponseBody
	public R importAlertrule(@RequestParam("file") MultipartFile file, HttpServletRequest request) {

		if (!file.isEmpty()) {
			try {
				//获取原始的文件名
				String originalFilename = file.getOriginalFilename();
				//获取文件类型
				String fileType = originalFilename.substring(originalFilename.lastIndexOf(".") + 1, originalFilename.length());
				//默认从第一行开始读取
				Integer startRows = 1;
				//获取输入流
				InputStream is = file.getInputStream();
				List<ResponsibilityIndicatorInfoUpdateDTO> bindingList = new ArrayList<>();
				//Excel导入导出的单元类
				List<String[]> strings = ExcelUtil.readData(fileType, startRows, true, is);
				//遍历Excel表每一行的数据
				for (String[] str : strings) {
					ResponsibilityIndicatorInfoUpdateDTO responsibilityIndicatorInfoUpdateDTO = new ResponsibilityIndicatorInfoUpdateDTO();
					responsibilityIndicatorInfoUpdateDTO.setId(str[0]);
					responsibilityIndicatorInfoUpdateDTO.setQuantifiedValue(Double.parseDouble(str[8]));
					bindingList.add(responsibilityIndicatorInfoUpdateDTO);
				}
				boolean b = responsibilityIndicatorInfoService.saveBatchDto(bindingList);
				if (b) {
					return R.ok().data(b);
				} else {
					return R.error().data("保存失败！");
				}
			} catch (Exception e) {
				log.error("错误", e);
				return R.customError(e.getMessage()).data("失败！");
			}
		}
		return R.customError("上传文件为空！");
	}



	/**
	 * 获取当前指标需要考评的指标
	 * @param responsibilityId
	 * @param binSection
	 * @param binStage
	 * @return
	 */
	@GetMapping(value = "getResponsibilityIndicatorList")
	public R getResponsibilityIndicatorList(
		@RequestParam(value = "responsibilityId", required = false) String responsibilityId,
		@RequestParam(value = "binSection", required = false) String binSection,
		@RequestParam(value = "binStage", required = false) String binStage) {
		List<Map> list = responsibilityIndicatorInfoService.getResponsibilityIndicatorList(responsibilityId,  binSection,  binStage, request);
		return R.ok().data(list);
	}



	/**
	 * 获取考核指标的详情
	 * @param responsibilityId
	 * @param indicatorId
	 * @return
	 */
	@GetMapping(value = "getResponsibilityInfoList")
	public R getEvaluationInfoList(
		@RequestParam(value = "responsibilityId", required = false) String responsibilityId,
		@RequestParam(value = "indicatorId", required = false) String indicatorId,
		@RequestParam(value = "organizationShortName", required = false) String organizationShortName,
		@RequestParam(value = "binSection", required = false) String binSection,
		@RequestParam(value = "binStage", required = false) String binStage) {


		try {
			Map list = responsibilityIndicatorInfoService.getResponsibilityInfoList(responsibilityId, indicatorId, organizationShortName,binSection,binStage, request);
			return R.ok().data(list);
		}catch (CustomException e1) {
			e1.printStackTrace();
			return R.error().customError(e1.getMessage());
		} catch (Exception e) {
			e.printStackTrace();
			return R.error().customError("查询失败");
		}

	}




	/**
	 * 批量修改
	 *
	 * @param jsonObjects
	 * @return
	 */
	@PostMapping(value = "/updateResponsibilityInfo")
	@ApiOperation(value = "批量新增(修改)", notes = "批量新增（修改）")
	public R updateResponsibilityIndicatorInfo(@RequestBody List<JSONObject> jsonObjects) {

		try {
			boolean b = responsibilityIndicatorInfoService.updateResponsibilityIndicatorInfo(jsonObjects);
			if (b) {
				return R.ok().data(b);
			} else {
				return R.error().data("保存失败！");
			}
		} catch (Exception e) {
			log.error("错误", e);
			return R.customError(e.getMessage()).data("失败！");
		}
	}



	@GetMapping("/download-excel")
	public ResponseEntity<byte[]> downloadExcel(
		@RequestParam(value = "responsibilityId", required = false) String responsibilityId,
		@RequestParam(value = "indicatorId", required = false) String indicatorId,
		@RequestParam(value = "binSection", required = false) String binSection,
		@RequestParam(value = "binStage", required = false) String binStage) throws Exception {


		byte[] excelBytes = responsibilityIndicatorInfoService.downloadExcel(responsibilityId, indicatorId,binSection,binStage, request);
		HttpHeaders headers = new HttpHeaders();
		headers.setContentType(MediaType.APPLICATION_OCTET_STREAM);
		headers.setContentDispositionFormData("attachment", "example.xlsx");

		return ResponseEntity.ok().headers(headers).body(excelBytes);

	}

	@PostMapping("/import-excel")
	public R importExcel(@RequestParam("file") MultipartFile file,HttpServletRequest request) throws Exception {

		boolean b = responsibilityIndicatorInfoService.importExcel(file,request);

		if (b) {
			return R.ok().data(b);
		} else {
			return R.error().data("上传失败");
		}
	}




}
