package com.ims.eval.controller;


import com.alibaba.fastjson.JSONObject;
import com.ims.eval.config.CustomException;
import com.ims.eval.entity.custom.DoAction;
import com.ims.eval.entity.custom.IsOkAction;
import com.ims.eval.entity.custom.TodoTask;
import com.ims.eval.entity.dto.result.R;
import com.ims.eval.entity.flow.FlowResult;
import com.ims.eval.service.custom.impl.WorkflowServiceImpl;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.*;

import javax.servlet.http.HttpServletRequest;

/**
 * <p>
 * 工作流前端控制器
 * </p>
 *
 * @author wang
 * @since 2023-03-03
 */
@Slf4j
@RestController
@RequestMapping("//workflow")
public class WorkflowController {

	@Autowired
	private WorkflowServiceImpl workflowServiceImpl;

	@Autowired
	private HttpServletRequest request;



	/**
	 * 流程启动
	 * @param doAction
	 * @return
	 */
	@PostMapping(value = "doAction")
	public R doAction(@RequestBody DoAction doAction) {
		JSONObject json = null;
		try {
			json = workflowServiceImpl.doAction(doAction,request);
		} catch (Exception e) {
			log.error("错误",e);
			return R.error().customError("流程发起任务失败");
		}
		return R.ok().data(json);
	}




	/**
	 * 获取待办任务
	 * @param todoTask
	 * @return
	 */
	@PostMapping(value = "todoTaskList")
	public R todoTaskList(@RequestBody TodoTask todoTask) {
		FlowResult json = null;
		try {
			json = workflowServiceImpl.todoTaskList(todoTask,request);
		} catch (Exception e) {
			log.error("错误",e);
			return R.error().customError("获取待办任务失败");
		}
		return R.ok().data(json);
	}


	/**
	 * 流程审批
	 * @param isOkAction
	 * @return
	 */
	@PostMapping(value = "isOkAction")
	public R isOkAction(@RequestBody IsOkAction isOkAction) {
		JSONObject json = null;
		try {
			json = workflowServiceImpl.isOkAction(isOkAction,request);
		} catch (CustomException ce) {
			log.error("错误",ce);
			return R.ok().data(R.customError(ce.getMessage()));
		}catch (Exception e){
			log.error("错误",e);
			return R.error().customError("流程发送失败");

		}
		return R.ok().data(json);
	}


	/**
	 * 获取流程实例
	 * @param id
	 * @param iamCode
	 * @return
	 */
	@PostMapping(value = "getInstanceAndChildren")
	public R getInstanceAndChildren(@RequestParam(value = "id", required = false) String id,
									@RequestParam(value = "iamCode", required = false) String iamCode) {
		JSONObject json = null;
		try {
			json = workflowServiceImpl.getInstanceAndChildren(id,iamCode,request);
		} catch (Exception e) {
			log.error("错误",e);
			return R.error().customError("流程发送失败");
		}
		return R.ok().data(json);
	}


	/**
	 * 获取流程审批意见
	 * @param instId
	 * @param iamCode
	 * @return
	 */
	@PostMapping(value = "getOpinion")
	public R getOpinion(@RequestParam(value = "instId", required = false) String instId,
									@RequestParam(value = "iamCode", required = false) String iamCode) {
		JSONObject json = null;
		try {
			json = workflowServiceImpl.getOpinion(instId,iamCode,request);
		} catch (Exception e) {
			log.error("错误",e);
			return R.error().customError("获取流程审批意见失败");
		}
		return R.ok().data(json);
	}



	/**
	 * 根据任务id;获取任务信息
	 * @param taskId
	 * @return
	 */
	@PostMapping(value = "getTaskData")
	public R getOpinion(@RequestParam(value = "taskId", required = false) String taskId) {
		JSONObject json = null;
		try {
			json = workflowServiceImpl.getTaskData(taskId,request);
		} catch (Exception e) {
			log.error("错误",e);
			return R.error().customError("获取任务信息见失败");
		}
		return R.ok().data(json);
	}


	/**
	 * 获取已办列表
	 * @param offset
	 * @return
	 */
	@PostMapping(value = "getAppRoveList")
	public R getOpinion(@RequestParam(value = "offset") Integer offset,
						@RequestParam(value = "limit") Integer limit) {
		JSONObject json = null;
		try {
			json = workflowServiceImpl.getAppRoveList(offset,limit,request);
		} catch (Exception e) {
			log.error("错误",e);
			return R.error().customError("获取已办列表");
		}
		return R.ok().data(json);
	}






}
