package com.ims.eval.service.custom.impl;

import com.alibaba.fastjson.JSON;
import com.alibaba.fastjson.JSONObject;
import com.ims.common.utils.StringUtils;
import com.ims.eval.config.ImaConfig;
import com.ims.eval.entity.DeptAssessmentDeclaration;
import com.ims.eval.entity.custom.DoAction;
import com.ims.eval.entity.custom.IsOkAction;
import com.ims.eval.entity.custom.RevocAtion;
import com.ims.eval.entity.custom.TodoTask;
import com.ims.eval.entity.flow.FlowResult;
import com.ims.eval.schedule.ScoreCalculationSchedule;
import com.ims.eval.service.*;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.http.*;
import org.springframework.stereotype.Service;
import org.springframework.util.LinkedMultiValueMap;
import org.springframework.util.MultiValueMap;
import org.springframework.web.client.RestTemplate;

import javax.servlet.http.HttpServletRequest;
import java.util.ArrayList;
import java.util.LinkedHashMap;
import java.util.List;
import java.util.Map;


/**
 * <p>
 * 工作流 服务实现类
 * </p>
 *
 * @author wang
 * @since 2023-03-12
 */
@Service
@Slf4j
public class WorkflowServiceImpl {

	@Autowired
	private RestTemplate restTemplate;

	@Autowired
	private ImaConfig imaConfig;

	@Autowired
	private IDeptResponsibilityService deptResponsibilityService;

	@Autowired
	private IOrganizationEvaluationService organizationEvaluationService;

	@Autowired
	private ScoreCalculationSchedule scoreCalculationSchedule;

	@Autowired
	private IEvaluationDeptBusinessPlanService evaluationDeptBusinessPlanService;

	@Autowired
	private IEvaluationDeptBusinessAssessmentService evaluationDeptBusinessAssessmentService;

	@Autowired
	private IEvaluationRevisionService evaluationRevisionService;

	@Autowired
	private IDeptAssessmentDeclarationService deptAssessmentDeclarationService;

	@Autowired
	private WorkflowMonthServiceImpl workflowMonthService;

	@Autowired
	private WorkflowSeasonServiceImpl workflowSeasonService;

	@Autowired
	private WorkflowYearsServiceImpl workflowYearsService;


	@Autowired
	private IOrganizationYearRatingService organizationYearRatingService;

	/**
	 * 发起流程
	 *
	 * @param doAction
	 * @return
	 */
	public JSONObject doAction(DoAction doAction, HttpServletRequest request) {
		HttpHeaders headers = new HttpHeaders();
		headers.setContentType(MediaType.APPLICATION_JSON_UTF8);
		headers.add("Blade-Auth", request.getHeader("Blade-Auth"));
		headers.add("code", request.getHeader("code"));
		Map<String, String> map = new LinkedHashMap<>();
		map.put("iamCode", doAction.getIamCode());
		map.put("defKey", doAction.getDefKey());
		map.put("businessKey", doAction.getBusinessKey());
		map.put("action", doAction.getAction());
		map.put("opinion", doAction.getOpinion());
		HttpEntity<Map> param = new HttpEntity<>(map, headers);
		ResponseEntity<String> responseEntity2 = restTemplate.exchange(imaConfig.getFlowUrl() + "/bpm/instance/doAction", HttpMethod.POST, param, String.class);
		log.info("\n code:{}\n header:{}\n body:{}\n", responseEntity2.getStatusCodeValue(), responseEntity2.getHeaders(), responseEntity2.getBody());
		if (200 == responseEntity2.getStatusCodeValue()) {
			FlowResult result = JSON.parseObject(responseEntity2.getBody()).toJavaObject(FlowResult.class);

			if (result.getIsOk()) {
				boolean biz = updateWorkflowState(result.getData(), doAction.getDefKey(), doAction.getBusinessKey(), "", "");
				if (biz) {
					log.info("保存流程实例id成功");
				}
			}
			return JSON.parseObject(responseEntity2.getBody());
		}
		return null;
	}

	/**
	 * 流程撤回
	 * @param doAction
	 * @param request
	 * @return
	 */
	public JSONObject revocAtion(RevocAtion doAction, HttpServletRequest request) {
		HttpHeaders headers = new HttpHeaders();
		headers.setContentType(MediaType.APPLICATION_JSON_UTF8);
		headers.add("Blade-Auth", request.getHeader("Blade-Auth"));
		headers.add("code", request.getHeader("code"));
		Map<String, String> map = new LinkedHashMap<>();
		map.put("instanceId", doAction.getInstanceId());
		map.put("defId", doAction.getDefId());
		map.put("action", doAction.getAction());
		map.put("opinion", doAction.getOpinion());
		HttpEntity<Map> param = new HttpEntity<>(map, headers);
		ResponseEntity<String> responseEntity2 = restTemplate.exchange(imaConfig.getFlowUrl() + "/bpm/instance/doAction", HttpMethod.GET, param, String.class);
		log.info("\n code:{}\n header:{}\n body:{}\n", responseEntity2.getStatusCodeValue(), responseEntity2.getHeaders(), responseEntity2.getBody());
		if (200 == responseEntity2.getStatusCodeValue()) {
			FlowResult result = JSON.parseObject(responseEntity2.getBody()).toJavaObject(FlowResult.class);
			return JSON.parseObject(responseEntity2.getBody());
		}
		return null;
	}



	/**
	 * 获取待办任务
	 *
	 * @param todoTask
	 * @return
	 */
	public FlowResult todoTaskList(TodoTask todoTask, HttpServletRequest request) {
		HttpHeaders headers = new HttpHeaders();
		headers.setContentType(MediaType.APPLICATION_JSON_UTF8);
		headers.add("Blade-Auth", request.getHeader("Blade-Auth"));
		headers.add("code", request.getHeader("code"));
		headers.setContentType(MediaType.MULTIPART_FORM_DATA);
		MultiValueMap<String, Object> map= new LinkedMultiValueMap<String, Object>();


//		Map<String, String> map = new LinkedHashMap<>();
		map.add("iamCode", todoTask.getIamCode());
		map.add("order", todoTask.getOrder());
//		map.add("sort", todoTask.getSort());
		map.add("offset", (todoTask.getOffset()-1)*todoTask.getLimit());
		map.add("limit", todoTask.getLimit());

//		HttpEntity<Map> param = new HttpEntity<>(map, headers);
//		ResponseEntity<String> responseEntity2 = restTemplate.exchange(imaConfig.getFlowUrl() + "/bpm/my/todoTaskList", HttpMethod.POST, param, String.class);

		HttpEntity<MultiValueMap<String, Object>> requestEntity = new HttpEntity<>(map, headers);
		//  执行HTTP请求
		ResponseEntity<String> responseEntity2  = restTemplate.postForEntity(imaConfig.getFlowUrl() + "/bpm/my/todoTaskList", requestEntity, String.class);

		log.info("\n code:{}\n header:{}\n body:{}\n", responseEntity2.getStatusCodeValue(), responseEntity2.getHeaders(), responseEntity2.getBody());
		//
		if (200 == responseEntity2.getStatusCodeValue()) {
			FlowResult result = JSON.parseObject(responseEntity2.getBody()).toJavaObject(FlowResult.class);
			if (result.getIsOk()) {
				List<JSONObject> ds = new ArrayList<>();
				for (int i = 0; i < result.getRows().size(); i++) {
					String typeId = result.getRows().getJSONObject(i).get("typeId").toString();
					if (!"440219646351376385".equals(typeId) && !"443254180359700481".equals(typeId)) {
						ds.add(result.getRows().getJSONObject(i));
						continue;
					}
				}
				result.getRows().removeAll(ds);
			}
			return result;
		}
		return null;
	}


	/**
	 * 获取待办任务
	 *
	 * @param todoTask
	 * @return
	 */
	public FlowResult todoTaskList2(TodoTask todoTask, HttpServletRequest request) {
		HttpHeaders headers = new HttpHeaders();
		headers.setContentType(MediaType.APPLICATION_JSON_UTF8);


		headers.add("Blade-Auth", request.getHeader("Blade-Auth"));
		headers.add("code", request.getHeader("code"));
//		MultiValueMap<String, Object> map = JSONObject.parseObject(JSONObject.toJSONString(todoTask), LinkedMultiValueMap.class);
		Map<String, Object> map = new LinkedHashMap<>();
		map.put("iamCode", todoTask.getIamCode());
		map.put("order", todoTask.getOrder());
//		map.put("sort", todoTask.getSort());
		map.put("offset", (todoTask.getOffset()-1)*todoTask.getLimit());
		map.put("limit", todoTask.getLimit());
		HttpEntity<Map> param = new HttpEntity<>(map, headers);
		ResponseEntity<String> responseEntity2 = restTemplate.exchange(imaConfig.getFlowUrl() + "/bpm/my/todoTaskList", HttpMethod.POST, param, String.class);
		log.info("\n code:{}\n header:{}\n body:{}\n", responseEntity2.getStatusCodeValue(), responseEntity2.getHeaders(), responseEntity2.getBody());
		//
		if (200 == responseEntity2.getStatusCodeValue()) {
			FlowResult result = JSON.parseObject(responseEntity2.getBody()).toJavaObject(FlowResult.class);
			if (result.getIsOk()) {
				List<JSONObject> ds = new ArrayList<>();
				for (int i = 0; i < result.getRows().size(); i++) {
					String typeId = result.getRows().getJSONObject(i).get("typeId").toString();
					if (!"440219646351376385".equals(typeId) && !"443254180359700481".equals(typeId)) {
						ds.add(result.getRows().getJSONObject(i));
						continue;
					}
				}

				result.getRows().removeAll(ds);
			}


			return result;
		}
		return null;
	}





	/**
	 * 检查当前数据（对应的用户）是或否存在代办
	 *
	 * @param instId
	 * @param request
	 * @return
	 */
	public boolean todoTaskList2(String instId, HttpServletRequest request) {
		try {
			HttpHeaders headers = new HttpHeaders();
			headers.setContentType(MediaType.APPLICATION_JSON_UTF8);
			headers.add("Blade-Auth", request.getHeader("Blade-Auth"));
			headers.add("code", request.getHeader("code"));
			Map<String, String> map = new LinkedHashMap<>();
			map.put("iamCode", request.getHeader("code"));
			map.put("order", "ASC");
			map.put("sort", "");
			map.put("offset", "");
			map.put("limit", "99999");
			HttpEntity<Map> param = new HttpEntity<>(map, headers);
			ResponseEntity<String> responseEntity2 = restTemplate.exchange(imaConfig.getFlowUrl() + "/bpm/my/todoTaskList", HttpMethod.POST, param, String.class);
			log.info("\n code:{}\n header:{}\n body:{}\n", responseEntity2.getStatusCodeValue(), responseEntity2.getHeaders(), responseEntity2.getBody());
			//
			if (200 == responseEntity2.getStatusCodeValue()) {
				FlowResult result = JSON.parseObject(responseEntity2.getBody()).toJavaObject(FlowResult.class);
				if (result.getIsOk()) {
					List<JSONObject> ds = new ArrayList<>();
					for (int i = 0; i < result.getRows().size(); i++) {
						String typeId = result.getRows().getJSONObject(i).get("typeId").toString();
						if (!"440219646351376385".equals(typeId)) {
							ds.add(result.getRows().getJSONObject(i));
							continue;
						}
						String resinstId = result.getRows().getJSONObject(i).get("instId").toString();
						if (instId.equals(resinstId)) {
							return true;
						}
					}
				}
			}
		} catch (Exception e) {
			e.printStackTrace();
			return false;
		}
		return false;
	}


	/**
	 * 流程审批
	 *
	 * @param isOkAction
	 * @return
	 */
	public JSONObject isOkAction(IsOkAction isOkAction, HttpServletRequest request) {
		HttpHeaders headers = new HttpHeaders();
		headers.setContentType(MediaType.APPLICATION_JSON_UTF8);
		headers.add("Blade-Auth", request.getHeader("Blade-Auth"));
		headers.add("code", request.getHeader("code"));

		//流程开始前的动作
		if (!"reject".equals(isOkAction.getAction())) {
			beforeCheckAction("", isOkAction.getDefKey(), isOkAction.getBizKey(), isOkAction.getTaskName());
		}

		Map<String, String> map = new LinkedHashMap<>();

		map.put("taskId", isOkAction.getTaskId());
		map.put("nodeId", isOkAction.getNodeId());
		map.put("action", isOkAction.getAction());
		//发起提条件不是同意
		if (!"agree".equals(isOkAction.getAction())) {
			map.put("destination", isOkAction.getDestination());
		}
		map.put("instanceId", isOkAction.getInstanceId());
		map.put("opinion", isOkAction.getOpinion());
		map.put("iamCode", isOkAction.getIamCode());
		if ("EndNoneEvent1".equals(isOkAction.getDestination())){
			map.put("destination", isOkAction.getDestination());
		}
		HttpEntity<Map> param = new HttpEntity<>(map, headers);
		ResponseEntity<String> responseEntity2 = restTemplate.exchange(imaConfig.getFlowUrl() + "/bpm/task/doAction", HttpMethod.POST, param, String.class);
		log.info("\n code:{}\n header:{}\n body:{}\n", responseEntity2.getStatusCodeValue(), responseEntity2.getHeaders(), responseEntity2.getBody());
		if (200 == responseEntity2.getStatusCodeValue()) {

			FlowResult result = JSON.parseObject(responseEntity2.getBody()).toJavaObject(FlowResult.class);
			if (result.getIsOk()) {
				boolean biz = updateWorkflowState("", isOkAction.getDefKey(), isOkAction.getBizKey(), isOkAction.getTaskName(), isOkAction.getDestination());
				if (biz) {
					log.info("状态修改成功");
				}
			}

			return JSON.parseObject(responseEntity2.getBody());
		}
		return null;
	}


	/**
	 * 获取业务id
	 *
	 * @param id
	 * @param iamCode
	 * @param request
	 * @return
	 */
	public JSONObject getInstanceAndChildren(String id, String iamCode, HttpServletRequest request) {
		HttpHeaders headers = new HttpHeaders();
		headers.setContentType(MediaType.APPLICATION_JSON_UTF8);
		headers.add("Blade-Auth", request.getHeader("Blade-Auth"));
		headers.add("code", request.getHeader("code"));

		HttpEntity<Map> param = new HttpEntity<>(null, headers);
		ResponseEntity<String> responseEntity2 = restTemplate.postForEntity(imaConfig.getFlowUrl() + "/bpm/instance/getInstanceAndChildren?id={1}&iamCode={2}",
			param, String.class, id, iamCode);

		log.info("\n code:{}\n header:{}\n body:{}\n", responseEntity2.getStatusCodeValue(), responseEntity2.getHeaders(), responseEntity2.getBody());
		if (200 == responseEntity2.getStatusCodeValue()) {
			return JSON.parseObject(responseEntity2.getBody());
		}
		return null;
	}


	/**
	 * 获取流程审批意见
	 *
	 * @param instId
	 * @param iamCode
	 * @param request
	 * @return
	 */
	public JSONObject getOpinion(String instId, String iamCode, HttpServletRequest request) {
		HttpHeaders headers = new HttpHeaders();
		headers.setContentType(MediaType.APPLICATION_JSON_UTF8);
		headers.add("Blade-Auth", request.getHeader("Blade-Auth"));
		headers.add("code", request.getHeader("code"));

		HttpEntity<Map> param = new HttpEntity<>(null, headers);
		ResponseEntity<String> responseEntity2 = restTemplate.postForEntity(imaConfig.getFlowUrl() + "/bpm/instance/getOpinion?instId={1}&iamCode={2}",
			param, String.class, instId, iamCode);

		log.info("\n code:{}\n header:{}\n body:{}\n", responseEntity2.getStatusCodeValue(), responseEntity2.getHeaders(), responseEntity2.getBody());
		if (200 == responseEntity2.getStatusCodeValue()) {
			return JSON.parseObject(responseEntity2.getBody());
		}
		return null;
	}



	public JSONObject getTaskData(String taskId,HttpServletRequest request) {

		HttpHeaders headers = new HttpHeaders();
		headers.setContentType(MediaType.APPLICATION_JSON_UTF8);
		headers.add("Blade-Auth", request.getHeader("Blade-Auth"));
		headers.add("code", request.getHeader("code"));
		headers.setContentType(MediaType.MULTIPART_FORM_DATA);
		MultiValueMap<String, Object> map= new LinkedMultiValueMap<String, Object>();

		map.add("taskId", taskId);

		HttpEntity<MultiValueMap<String, Object>> requestEntity = new HttpEntity<>(map, headers);
		//  执行HTTP请求
		ResponseEntity<String> responseEntity2  = restTemplate.postForEntity(imaConfig.getFlowUrl() + "/bpm/task/getTaskData", requestEntity, String.class);
		log.info("\n code:{}\n header:{}\n body:{}\n", responseEntity2.getStatusCodeValue(), responseEntity2.getHeaders(), responseEntity2.getBody());
		//
		if (200 == responseEntity2.getStatusCodeValue()) {
			return JSON.parseObject(responseEntity2.getBody());
		}
		return null;

	}


	public JSONObject getAppRoveList(int offset,int limit,HttpServletRequest request) {

		HttpHeaders headers = new HttpHeaders();
		headers.setContentType(MediaType.APPLICATION_JSON_UTF8);
		headers.add("Blade-Auth", request.getHeader("Blade-Auth"));
		headers.add("code", request.getHeader("code"));
		headers.setContentType(MediaType.MULTIPART_FORM_DATA);
		MultiValueMap<String, Object> map= new LinkedMultiValueMap<String, Object>();


		map.add("offset", (offset-1)*limit);
		map.add("limit", limit);


		HttpEntity<MultiValueMap<String, Object>> requestEntity = new HttpEntity<>(map, headers);
		//  执行HTTP请求
		ResponseEntity<String> responseEntity2  = restTemplate.postForEntity(imaConfig.getFlowUrl() + "/bpm/my/approveList", requestEntity, String.class);
		log.info("\n code:{}\n header:{}\n body:{}\n", responseEntity2.getStatusCodeValue(), responseEntity2.getHeaders(), responseEntity2.getBody());
		//
		if (200 == responseEntity2.getStatusCodeValue()) {
			return JSON.parseObject(responseEntity2.getBody());
		}
		return null;

	}



	private boolean updateWorkflowState(String instId, String defKey, String bizKey, String taskName, String destination) {

		switch (defKey) {
			case "dwkpmbqd"://单位考评目标启动
				deptResponsibilityService.editState(bizKey, taskName, instId);
				break;
			case "yddwkplc"://月度单位考评流程
				organizationEvaluationService.editState(bizKey, taskName, instId);
				break;
			case "jddwkplc"://季度单位考评流程
				organizationEvaluationService.editState(bizKey, taskName, instId);
				break;
			case "nddwkplc"://年度单位考评流程
				organizationEvaluationService.editState(bizKey, taskName, instId);
				break;
			case "dwkpmbxd"://单位考评目标修订
				evaluationRevisionService.editState(bizKey, taskName, instId);
				break;
			case "bbbmyjzbjh"://本部部门业绩指标计划
				evaluationDeptBusinessPlanService.editState(bizKey, taskName, instId);
				break;
			case "bbbmyjzbkh"://本部部门业绩指标考核
				evaluationDeptBusinessAssessmentService.editState(bizKey, taskName, instId);
				break;
			case "jxjgkhsb"://绩效结果考核申报
				deptAssessmentDeclarationService.editState(bizKey, taskName, instId, destination);
				break;
			case "nddwpj"://年度单位评级
				organizationYearRatingService.editState(bizKey, taskName, instId, destination);
				break;
		}
		return true;

	}


	public boolean beforeCheckAction(String instId, String defKey, String bizKey, String taskName) {
		switch (defKey) {
			case "dwkpmbqd"://单位考评目标启动
				break;
			case "yddwkplc"://月度单位考评流程
				workflowMonthService.beforeCheckAction(instId, defKey, bizKey, taskName);
				break;
			case "jddwkplc"://季度单位考评流程
				workflowSeasonService.beforeCheckAction(instId, defKey, bizKey, taskName);
				break;
			case "nddwkplc"://年度单位考评流程
				workflowYearsService.beforeCheckAction(instId, defKey, bizKey, taskName);
				break;
			case "dwkpmbxd"://单位考评目标修订
				break;
			case "bbbmyjzbjh"://本部部门业绩指标计划
				break;
			case "bbbmyjzbkh"://本部部门业绩指标考核
				break;
			case "jxjgkhsb"://绩效结果考核申报
				break;
		}
		return true;


	}


}
