package com.ims.eval.service.impl;

import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.ims.common.utils.StringUtils;
import com.ims.eval.dao.DataDictionaryMapper;
import com.ims.eval.config.CustomException;
import com.ims.eval.entity.DataDictionary;
import com.ims.eval.entity.Indicator;
import com.ims.eval.service.IDataDictionaryService;
import org.springframework.stereotype.Service;

import java.util.Date;
import java.util.List;

/**
 * <p>
 * 数据字典表 服务实现类
 * </p>
 *
 * @author wang
 * @since 2023-02-26
 */
@Service
public class DataDictionaryServiceImpl extends ServiceImpl<DataDictionaryMapper, DataDictionary> implements IDataDictionaryService {

	@Override
	public List<DataDictionary> list(String id, String dataKey, String keyName, String superKey) {

		QueryWrapper<DataDictionary> qw = new QueryWrapper<>();
		if (StringUtils.isNotEmpty(id)) {
			qw.lambda().eq(DataDictionary::getId, id);
		}
		if (StringUtils.isNotEmpty(dataKey)) {
			qw.lambda().like(DataDictionary::getDataKey, dataKey);
		}
		if (StringUtils.isNotEmpty(keyName)) {
			qw.lambda().like(DataDictionary::getKeyName, keyName);
		}
		if (StringUtils.isNotEmpty(superKey)) {
			qw.lambda().eq(DataDictionary::getSuperKey, superKey);
		}
		qw.lambda().orderByAsc(DataDictionary::getOrderNum);

		List<DataDictionary> list = baseMapper.selectList(qw);
		return list;
	}

	@Override
	public IPage<DataDictionary> listPage(Integer pageNum, Integer pageSize, String dataKey, String keyName,String superKey) {
		QueryWrapper<DataDictionary> qw = new QueryWrapper<>();
		Page<DataDictionary> page = new Page<>(pageNum, pageSize);

		if (null == pageNum || null == pageSize) {
			throw new CustomException("分页参数为空");
		}
		if (StringUtils.isNotEmpty(dataKey)) {
			qw.lambda().like(DataDictionary::getDataKey, dataKey);
		}
		if (StringUtils.isNotEmpty(keyName)) {
			qw.lambda().like(DataDictionary::getKeyName, keyName);
		}
		if (StringUtils.isNotEmpty(superKey)) {
			qw.lambda().eq(DataDictionary::getSuperKey, superKey);
		}
		qw.lambda().orderByAsc(DataDictionary::getOrderNum);

		IPage<DataDictionary> list = baseMapper.selectPage(page, qw);
		return list;
	}


	@Override
	public boolean saveOrUpdate(DataDictionary entity) {

		//业务判断
		if (null != entity && (null == entity.getId() || "".equals(entity.getId().trim()))) {
			QueryWrapper<DataDictionary> qw = new QueryWrapper<>();
			qw.lambda().eq(DataDictionary::getDataKey, entity.getDataKey());
			List<DataDictionary> list = baseMapper.selectList(qw);
			if (null != list && list.size() > 0) {
				throw new CustomException("当前字典key已存在");
			}

			entity.setCreateTime(new Date());
		} else {
			entity.setUpdateTime(new Date());
		}
		return super.saveOrUpdate(entity);
	}
}
