package com.ims.eval.service.impl;

import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.ims.common.utils.DateUtils;
import com.ims.common.utils.StringUtils;
import com.ims.eval.cache.CacheContext;
import com.ims.eval.config.CustomException;
import com.ims.eval.dao.ResponsibilityIndicatorInfoMapper;
import com.ims.eval.entity.*;
import com.ims.eval.dao.DeptResponsibilityMapper;
import com.ims.eval.entity.dto.response.DeptResponsibilityResDTO;
import com.ims.eval.entity.dto.response.IndicatorResDTO;
import com.ims.eval.entity.dto.response.ResponsibilityIndicatorInfoResDTO;
import com.ims.eval.service.*;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import javax.annotation.Resource;
import javax.servlet.http.HttpServletRequest;
import java.io.Serializable;
import java.util.*;
import java.util.stream.Collectors;

/**
 * <p>
 * 部门目标责任表 服务实现类
 * </p>
 *
 * @author wang
 * @since 2023-02-27
 */
@Service
public class DeptResponsibilityServiceImpl extends ServiceImpl<DeptResponsibilityMapper, DeptResponsibility> implements IDeptResponsibilityService {


	/**
	 * 指标主业务类
	 */
	@Autowired
	private IIndicatorService iIndicatorService;

	/**
	 * 指标子业务类
	 */
	@Autowired
	private IIndicatorDictionaryService indicatorDictionaryService;


	/**
	 * 单位/部门考评配置业务类
	 */
	@Autowired
	private IOrganizationEvaluationRuleService organizationEvaluationRuleService;

	/**
	 * 考评规则业务类
	 */
	@Autowired
	private IEvaluateRuleService evaluateRuleService;

	/**
	 * 考评规则明细业务类
	 */
	@Autowired
	private IEvaluateRuleInfoService evaluateRuleInfoService;


	@Resource
	private ResponsibilityIndicatorInfoMapper responsibilityIndicatorInfoMapper;

	@Resource
	private IResponsibilityIndicatorInfoService responsibilityIndicatorInfoService;


	@Autowired
	private IUserService userService;

	@Autowired
	private IMultipleBrandService multipleBrandService;


	@Override
	public IPage<DeptResponsibility> list(Integer pageNum, Integer pageSize, String id, String responsibilityCode, String cycleUnit, List<String> checkCycle, String beginDate, String endDate, String stage, String createBy, String year, String month, String des) {

		QueryWrapper<DeptResponsibility> qw = new QueryWrapper<>();
		if (null == pageNum || null == pageSize) {
			throw new CustomException("分页参数为空");
		}
		//构造分页构造器
		Page<DeptResponsibility> page = new Page<>(pageNum, pageSize);

		if (StringUtils.isNotEmpty(id)) {
			qw.lambda().eq(DeptResponsibility::getId, id);
		}

		if (StringUtils.isNotEmpty(responsibilityCode)) {
			qw.lambda().like(DeptResponsibility::getResponsibilityCode, responsibilityCode);
		}

		if (StringUtils.isNotEmpty(cycleUnit)) {
			qw.lambda().eq(DeptResponsibility::getCycleUnit, cycleUnit);
		}

		if (StringUtils.isNotEmpty(checkCycle)) {
			qw.lambda().in(DeptResponsibility::getCheckCycle, checkCycle);
		}

		if (StringUtils.isNotEmpty(beginDate) && StringUtils.isNotEmpty(endDate)) {
			qw.lambda().ge(DeptResponsibility::getBeginDate, beginDate);
			qw.lambda().le(DeptResponsibility::getEndDate, endDate);
		}

		if (StringUtils.isNotEmpty(stage)) {
			qw.lambda().eq(DeptResponsibility::getStage, stage);
		}

		if (StringUtils.isNotEmpty(createBy)) {
			qw.lambda().eq(DeptResponsibility::getCreateBy, createBy);
		}

		if (StringUtils.isNotEmpty(createBy)) {
			qw.lambda().eq(DeptResponsibility::getCreateBy, createBy);
		}
		if (StringUtils.isNotEmpty(year)) {
			qw.lambda().eq(DeptResponsibility::getYear, year);
		}
		if (StringUtils.isNotEmpty(month)) {
			qw.lambda().eq(DeptResponsibility::getMonth, month);
		}

		if (StringUtils.isNotEmpty(des)) {
			qw.lambda().like(DeptResponsibility::getDes, des);
		}

		qw.lambda().orderByDesc(DeptResponsibility::getCreateTime);

		IPage<DeptResponsibility> list = baseMapper.selectPage(page, qw);
		return list;
	}


	@Transactional
	@Override
	public boolean generateResponsibility(String id) {


		DeptResponsibility oriResponsibility = baseMapper.selectById(id);
		if (null == oriResponsibility) {
			throw new CustomException("此目标责任书不存在");
		}

		QueryWrapper<ResponsibilityIndicatorInfo> infoqw = new QueryWrapper<>();
		infoqw.lambda().eq(ResponsibilityIndicatorInfo::getDeptResponsibilityId, id);
		int infcount = responsibilityIndicatorInfoService.count(infoqw);
		if (infcount > 0) {
			throw new CustomException("编号" + oriResponsibility.getResponsibilityCode() + "的责任书已启动;不允许重复操作");
		}
		oriResponsibility.setStage("流程启动");
		baseMapper.updateById(oriResponsibility);

		String organizationType = oriResponsibility.getOrganizationType();//考评类型 DWKP
		String checkCycle = oriResponsibility.getCheckCycle();//考评周期-月度、季度
		String year = oriResponsibility.getYear();//年份

		//查询当前周期下对应的考评权重（需要考评的单位）
		List<OrganizationEvaluationRule> evaluationRules = organizationEvaluationRuleService.getOrganizationEvaluationRuleByYearAndCycle(organizationType, checkCycle, year, "");


		if(null == evaluationRules || evaluationRules.size()<=0){
			throw new CustomException(oriResponsibility.getYear()+"年单位权重未配置");
		}
		//遍历考核的单位
		for (OrganizationEvaluationRule evaluationRule : evaluationRules) {

			//规则集合
			List<EvaluateRule> evaluateRuleList = new ArrayList<>();
			//根据权重获取对应的考评规则
			//生产经营
			double scjyWeight = evaluationRule.getScjyWeight();
			if (scjyWeight > 0) {
				List<EvaluateRule> scjyevaluateRuleList = evaluateRuleService.getEvaluateRuleByYear(evaluationRule.getBinSection(), CacheContext.bseCodeObject.get("SCJY").getId(), "NDKP", evaluationRule.getYear());
				if (null != scjyevaluateRuleList && scjyevaluateRuleList.size() > 0) {
					evaluateRuleList.addAll(scjyevaluateRuleList);
				}
			}

			//判断多块牌子  是否在该序列加权平均
			List<MultipleBrand> dkpz = multipleBrandService.getMultipleBranList2(evaluationRule.getOrganizationId(), "", "", evaluationRule.getEvaluationCycle(), evaluationRule.getYear(), "");
			List<MultipleBrand> filteredList = dkpz.stream()
				.filter(multipleBrand -> !"0".equals(multipleBrand.getParentId()))
				.collect(Collectors.toList());
			if (null == filteredList ||  filteredList.size() <= 0) {
				//重点专项科技创新
				List<EvaluateRule> zkevaluateRuleList = evaluateRuleService.getEvaluateRuleByYear(CacheContext.bsnCodeObject.get("KJCX").getId() + "," + CacheContext.bsnCodeObject.get("ZDZX").getId(), CacheContext.bseCodeObject.get("ZDZX").getId() + "," + CacheContext.bseCodeObject.get("KJCX").getId(), "NDKP", evaluationRule.getYear());

				if (null != zkevaluateRuleList && zkevaluateRuleList.size() > 0) {
					evaluateRuleList.addAll(zkevaluateRuleList);
				}
			}

				//前期
				double qqWeight = evaluationRule.getQqWeight();
				if (qqWeight > 0) {
					List<EvaluateRule> qqevaluateRuleList = evaluateRuleService.getEvaluateRuleByYear(CacheContext.bsnCodeObject.get("QQ").getId(), CacheContext.bseCodeObject.get("QQXM").getId(), "NDKPALL", "0000");
					if (null != qqevaluateRuleList && qqevaluateRuleList.size() > 0) {
						evaluateRuleList.addAll(qqevaluateRuleList);
					}
				}
				//基建
				double jjWeight = evaluationRule.getJjWeight();
				if (jjWeight > 0) {
					List<EvaluateRule> jjevaluateRuleList = evaluateRuleService.getEvaluateRuleByYear(CacheContext.bsnCodeObject.get("JJ").getId(), CacheContext.bseCodeObject.get("JJXM").getId(), "NDKPALL", "0000");
					if (null != jjevaluateRuleList && jjevaluateRuleList.size() > 0) {
						evaluateRuleList.addAll(jjevaluateRuleList);
					}
				}

			//获取考评规则的id
			List<String> ruleIds = evaluateRuleList.stream().map(EvaluateRule::getId).collect(Collectors.toList());
			//通过规则id获取对应的规则明细
			if (ruleIds.size() <= 0) {
				continue;
			}
			List<EvaluateRuleInfo> ruleInfos = evaluateRuleInfoService.list(ruleIds);
			if (null == ruleInfos || ruleInfos.size() <= 0) {
				continue;
			}

			//获取考评规则中的指标id
			List<String> indicatorIds = ruleInfos.stream().map(EvaluateRuleInfo::getIndicatorId).collect(Collectors.toList());

			//获取指标对象
			List<IndicatorResDTO> indicators = iIndicatorService.getGroupChildCodeByIds(indicatorIds);
			if (null == indicators || indicators.size() <= 0) {
				continue;
			}
			List<ResponsibilityIndicatorInfo> infoList = new ArrayList<>();
			for (IndicatorResDTO dto : indicators) {
				ResponsibilityIndicatorInfo info = new ResponsibilityIndicatorInfo();
				info.setDeptResponsibilityId(oriResponsibility.getId());
				info.setOrganizationEvaluationRuleId(evaluationRule.getId());
				info.setIndicatorId(dto.getIndicatorId());
				info.setChildCode(dto.getChildCode());
				info.setDeptId(dto.getDeptId());
				info.setIndicatorDictionaryId(dto.getId());
				info.setOptionCode(dto.getOptionCode());
				info.setIsQuantified(dto.getIsQuantified());
				info.setCreateTime(new Date());
				info.setState("0");
				info.setDataState("1");
				info.setOrganizationId(evaluationRule.getOrganizationId());
				infoList.add(info);

			}
			boolean b2 = responsibilityIndicatorInfoService.saveOrUpdateBatch(infoList);
			if (!b2) {
				throw new CustomException("初始化明细失败");
			}

		}
		oriResponsibility.setStage("流程启动");
		baseMapper.updateById(oriResponsibility);
		return true;
	}


	@Override
	public DeptResponsibilityResDTO getByidAndInfo(String id, String dept, HttpServletRequest request) {
		DeptResponsibilityResDTO resDTO = baseMapper.selectById(id);

		Map<Object, List<ResponsibilityIndicatorInfoResDTO>> map = responsibilityIndicatorInfoService.planValueList(Arrays.asList(id), dept, request);
		if (null != resDTO) {
			resDTO.setMap(map);
		}

		return resDTO;
	}

	@Override
	public List<Map> targetValueeport(String id, String dept, HttpServletRequest request) {
		DeptResponsibilityResDTO resDTO = baseMapper.selectById(id);
		Myuser myuser = userService.getSysUser(request.getHeader("code"));
//		responsibilityIndicatorInfoMapper.listByresponsibilityId()

		return null;
	}

	@Override
	public boolean editState(String id, String state, String instId) {
		DeptResponsibility responsibility = baseMapper.selectById(id);
		if (StringUtils.isNotEmpty(state)) {
			if ("任务书签订".equals(state)) {
				state = "流程已结束";
			}
			responsibility.setStage(state);
		}
		if (StringUtils.isNotEmpty(instId)) {
			responsibility.setInstId(instId);
			responsibility.setStage("流程启动");
		}
		boolean b = super.saveOrUpdate(responsibility);
		return b;
	}

	@Transactional
	@Override
	public boolean saveOrUpdate(DeptResponsibility entity) {
		//获取指定部门的规则
		List<OrganizationEvaluationRule> list = organizationEvaluationRuleService.listByIsCheck(entity.getOrganizationType(), entity.getCheckCycle(),"", true, false);
		if (null == list || list.size() <= 0) {
			throw new CustomException("不存在单位权重配置");
		}
		//获取组织id;
		List<String> organizationIds = list.stream().map(OrganizationEvaluationRule::getOrganizationId).collect(Collectors.toList());
		List<String> organizationNames = list.stream().map(OrganizationEvaluationRule::getOrganizationName).collect(Collectors.toList());
		List<String> evaluationRuleIds = list.stream().map(OrganizationEvaluationRule::getId).collect(Collectors.toList());

		StringBuilder responsibilityCode = new StringBuilder();
		responsibilityCode.append("mbzr_");
		responsibilityCode.append(entity.getCheckCycle()).append("_");

		//判断该是否存在目标责任书
		List<DeptResponsibility> oriList = new ArrayList<>();
		if ("NDKP".equals(entity.getCheckCycle())) {
			oriList = list("", entity.getCheckCycle(), entity.getYear(), "");
			responsibilityCode.append(entity.getYear()).append("_");
			responsibilityCode.append(StringUtils.addZeroForStr(String.valueOf(oriList.size() + 1), 4, "l"));
		}
		if ("JDKP".equals(entity.getCheckCycle())) {
			Date newDate = DateUtils.toDate(entity.getYear() + "-" + entity.getMonth() + "-01");
			String season = String.valueOf(DateUtils.getSeason(newDate));//获取季度
			String month = String.valueOf(Integer.valueOf(entity.getMonth()) * Integer.valueOf(season));//季度考评将
			oriList = list("", entity.getCheckCycle(), entity.getYear(), month);
			List<DeptResponsibility> count = list("", entity.getCheckCycle(), entity.getYear(), "");
			responsibilityCode.append(entity.getYear()).append("_").append(month).append("_");
			responsibilityCode.append(StringUtils.addZeroForStr(String.valueOf(count.size() + 1), 4, "l"));
		}
		if ("YDKP".equals(entity.getCheckCycle())) {
			oriList = list("", entity.getCheckCycle(), entity.getYear(), entity.getMonth());
			responsibilityCode.append(entity.getYear()).append("_").append(entity.getMonth()).append("_");
			List<DeptResponsibility> count = list("", entity.getCheckCycle(), entity.getYear(), "");
			responsibilityCode.append(StringUtils.addZeroForStr(String.valueOf(count.size() + 1), 4, "l"));
		}


		if (null != entity && (null == entity.getId() || "".equals(entity.getId().trim()))) {
			if (null != oriList && oriList.size() > 0) {
				throw new CustomException("此单位在该阶段已存在目标责任书");
			}
			//保存目标责任书
			entity.setResponsibilityCode(responsibilityCode.toString().toLowerCase());

			entity.setOrganizationId(String.join(",", organizationIds));//考评组织ID
			entity.setOrganizationName(String.join(",", organizationNames));
			entity.setOrganizationEvaluationRuleId(String.join(",", evaluationRuleIds));
			entity.setStage("流程未启动");
			entity.setCreateTime(new Date());
		} else {
			DeptResponsibility update = baseMapper.selectById(entity.getId());
			if (null != oriList && oriList.size() > 1 && !update.getId().equals(oriList.get(0).getId())) {
				throw new CustomException("编辑记录未找到");
			}
			if (!"流程未启动".equals(update.getStage())) {
				throw new CustomException("流程已启动不允许修改");
			}
			entity.setOrganizationId(String.join(",", organizationIds));//考评组织ID
			entity.setOrganizationName(String.join(",", organizationNames));
			entity.setOrganizationEvaluationRuleId(String.join(",", evaluationRuleIds));
			entity.setUpdateTime(new Date());
		}
		boolean b = super.saveOrUpdate(entity);
		return b;
	}


	@Override
	public List<DeptResponsibility> list(String organizationId, String checkCycle, String year, String month) {
		QueryWrapper<DeptResponsibility> qw = new QueryWrapper<>();
		if (StringUtils.isNotEmpty(organizationId)) {
			qw.lambda().eq(DeptResponsibility::getOrganizationId, organizationId);
		}

		if (StringUtils.isNotEmpty(checkCycle)) {
			qw.lambda().eq(DeptResponsibility::getCheckCycle, checkCycle);
		}
		if (StringUtils.isNotEmpty(year)) {
			qw.lambda().eq(DeptResponsibility::getYear, year);
		}
		if (StringUtils.isNotEmpty(month)) {
			qw.lambda().in(DeptResponsibility::getMonth, month);
		}
		List<DeptResponsibility> list = baseMapper.selectList(qw);
		return list;
	}

	@Transactional
	@Override
	public boolean removeByIds(Collection<? extends Serializable> idList) {

		QueryWrapper<ResponsibilityIndicatorInfo> infoqw = new QueryWrapper<>();
		infoqw.lambda().in(ResponsibilityIndicatorInfo::getDeptResponsibilityId, idList);
		int infcount = responsibilityIndicatorInfoService.count(infoqw);
		if (infcount > 0) {
			throw new CustomException("已发起流程不允许删除");
		}

		boolean b = super.removeByIds(idList);
		if (!b) {
			throw new CustomException("删除失败");

		}
		return true;
	}


	@Override
	public DeptResponsibility getDeptResponsibility(String id) {
		if (StringUtils.isEmpty(id)) {
			return null;
		}
		DeptResponsibility res = this.getById(id);
		return res;
	}
}
