package com.ims.eval.service.impl;

import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.ims.common.utils.DateUtils;
import com.ims.common.utils.DoubleUtils;
import com.ims.common.utils.StringUtils;
import com.ims.eval.cache.CacheContext;
import com.ims.eval.config.CustomException;
import com.ims.eval.entity.*;
import com.ims.eval.dao.EvaluateReportMapper;
import com.ims.eval.entity.dto.response.EvaluateReportInfoResDTO;
import com.ims.eval.service.*;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import java.util.*;
import java.util.function.Function;
import java.util.stream.Collectors;

/**
 * <p>
 * 服务实现类
 * </p>
 *
 * @author wang
 * @since 2023-03-25
 */
@Service
public class EvaluateReportServiceImpl extends ServiceImpl<EvaluateReportMapper, EvaluateReport> implements IEvaluateReportService {


	@Autowired
	private IOrganizationEvaluationService organizationEvaluationService;


	@Autowired
	private IOrganizationEvaluationRuleService organizationEvaluationRuleService;

	@Autowired
	private IEvaluateRuleService evaluateRuleService;


	@Autowired
	private IEvaluationScoreCountService evaluationScoreCountService;

	@Autowired
	private IEvaluateReportInfoService evaluateReportInfoService;

	@Autowired
	private IMultipleBrandService multipleBrandService;


	@Autowired
	private CacheContext cache;

	@Autowired
	private IEvaluationPortalService evaluationPortalService;

	@Autowired
	private IYearOperatingCoefficientService yearOperatingCoefficientService;


	@Autowired
	private  IPartyBuildingMultiplierService partyBuildingMultiplierService;
	//考评结果报告
	//按照业务类型生成
	//业务阶综合展示



	@Transactional
	@Override
	public boolean generateEvaluateReport(String organizationEvaluationId, boolean backroll) {


		//业务
		List<BinSection> sections = CacheContext.bsnList;

		OrganizationEvaluation evaluation = organizationEvaluationService.getById(organizationEvaluationId);
		if (null == evaluation) {
			throw new CustomException("没有此靠考评");
		}
		//获取到对应组织考评规则
		String evaluationRuleId = evaluation.getOrganizationEvaluationRuleId();
		if (null == evaluationRuleId || evaluationRuleId.length() <= 0) {
			throw new CustomException("组织考评id为空");
		}
		String month = "";
		//获取单位配置规则
		List<String> evaluationRuleIds = Arrays.asList(evaluationRuleId.split(","));

		//获取所有当前考评配置所有权重单位
//		List<OrganizationEvaluationRule> ruleListall = organizationEvaluationRuleService.getOrganizationEvaluationRuleByIds(evaluationRuleIds, "");
		List<OrganizationEvaluationRule> ruleListall = organizationEvaluationRuleService.listByIsCheck(evaluation.getOrganizationType(), evaluation.getCheckCycle(),evaluation.getYear(),true,false);

		//按照组织id 分组
		Map<String, OrganizationEvaluationRule> ruleMap = ruleListall.stream()
			.collect(Collectors.toMap(OrganizationEvaluationRule::getOrganizationId, rule -> rule));

		for (BinSection bi : sections) {


			EvaluateReport evaluateReport = new EvaluateReport();

			StringBuilder des = new StringBuilder();
			StringBuilder reportName = new StringBuilder();

			reportName.append(evaluation.getYear()).append("年");
			if ("NDKP".equals(evaluation.getCheckCycle())) {
				if("FD".equals(bi.getSectionCode()) || "GD".equals(bi.getSectionCode()) ||
					"GLSX".equals(bi.getSectionCode()) || "KJCX".equals(bi.getSectionCode())||
					"ZDZX".equals(bi.getSectionCode())){
					continue;
				}
				des.append("年度");
				reportName.append("-");
				month = "";
				evaluateReport.setMonth("-");
			}
			if ("JDKP".equals(evaluation.getCheckCycle())) {
				des.append("季度");
				month = evaluation.getMonth();
				reportName.append(evaluation.getMonth()).append("季度-");
				evaluateReport.setMonth(evaluation.getMonth());
				if("XNYFG".equals(bi.getSectionCode()) || "ZHZC".equals(bi.getSectionCode())||
					"GCGS".equals(bi.getSectionCode()) || "TJGS".equals(bi.getSectionCode())|
					"GD".equals(bi.getSectionCode()) || "GLSX".equals(bi.getSectionCode()) ||
					"KJCX".equals(bi.getSectionCode()) || "ZDZX".equals(bi.getSectionCode())){
					continue;
				}

			}
			if ("YDKP".equals(evaluation.getCheckCycle())) {
				if("XNYFG".equals(bi.getSectionCode()) || "ZHZC".equals(bi.getSectionCode())||
					"GCGS".equals(bi.getSectionCode()) || "TJGS".equals(bi.getSectionCode())||
					"QQ".equals(bi.getSectionCode()) || "JJ".equals(bi.getSectionCode())||
					"GD".equals(bi.getSectionCode())|| "GLSX".equals(bi.getSectionCode()) ||
					"KJCX".equals(bi.getSectionCode()) || "ZDZX".equals(bi.getSectionCode())){
					continue;
				}
				des.append("月度");
				reportName.append(evaluation.getMonth()).append("月-");
				month = evaluation.getMonth();
				evaluateReport.setMonth(evaluation.getMonth());
			}
			des.append(bi.getSectionName());
			des.append("综合考核评分汇总表");

			reportName.append(bi.getSectionName());
			reportName.append("考核评分");


			evaluateReport.setEvaluateReportName(reportName.toString());
			evaluateReport.setDes(des.toString());
			evaluateReport.setOrganizationEvaluationId(evaluation.getId());
			evaluateReport.setBinSection(bi.getId());
			evaluateReport.setBinSectionName(bi.getSectionName());
			evaluateReport.setOrganizationType(evaluation.getOrganizationType());
			evaluateReport.setCheckCycle(evaluation.getCheckCycle());
			evaluateReport.setYear(evaluation.getYear());

			evaluateReport.setCreateTime(new Date());
			evaluateReport.setCreateBy("");//人员
			List<EvaluateReport> oriReports = list(evaluation.getId(), bi.getId());
			if (oriReports.size() > 0 && !backroll) {
				for(EvaluateReport er : oriReports){
					List<EvaluateReportInfo> rlist = evaluateReportInfoService.listByReportId(er.getId());

					if(null !=rlist && rlist.size()>0){
						boolean rlistb =  evaluateReportInfoService.removeByIds(rlist.stream().map(EvaluateReportInfo::getId).collect(Collectors.toList()));
						if (!rlistb) {
							throw new CustomException("综合考核评分汇总计算失败");
						}
					}
				}
				boolean oriReportsb = this.removeByIds(oriReports.stream().map(EvaluateReport::getId).collect(Collectors.toList()));
				if (!oriReportsb) {
					throw new CustomException("综合考核评分汇总计算失败");
				}
				//throw new CustomException("综合考核评分汇总已生成");
			} else {
				for (EvaluateReport r : oriReports) {
					boolean b = super.removeById(r.getId());
					if (!b) {
						throw new CustomException("综合考核评分汇总重算失败");
					}
					List<EvaluateReportInfo> infolist = evaluateReportInfoService.listByReportId(r.getId());
					if (null != infolist && infolist.size() > 0) {
						List<String> infolistIds = infolist.stream().map(EvaluateReportInfo::getId).collect(Collectors.toList());
						b = evaluateReportInfoService.removeByIds(infolistIds);
						if (!b) {
							throw new CustomException("综合考核评分汇总重算失败");
						}
					}
				}
			}
			boolean b = super.saveOrUpdate(evaluateReport);

			if (!b) {
				throw new CustomException("综合考核评分汇总表存储失败");
			}


			//获取当前模块得公司
//			List<OrganizationEvaluationRule> ruleList = organizationEvaluationRuleService.getOrganizationEvaluationRuleByIds(evaluationRuleIds, bi.getId());
			List<OrganizationEvaluationRule> ruleList = ruleListall.stream()
				.filter(rule -> rule.getBinSection().contains(bi.getId())).collect(Collectors.toList());

			//各板块明细存储
			List<EvaluateReportInfo> evaluateRuleInfoList = new ArrayList<>();

			//首页数据
			List<EvaluationPortal> portalList = new ArrayList<>();
			List<EvaluationPortalAttach> portalAttachList = new ArrayList<>();

			for (OrganizationEvaluationRule rule : ruleList) {


				if ("JJ".equals(bi.getSectionCode()) || "QQ".equals(bi.getSectionCode())) {

					if ( rule.getQqWeight() + rule.getJjWeight() < 1) {
						continue;
					}

					if ("QQ".equals(bi.getSectionCode())  && rule.getJjWeight() == 1) {
						continue;
					}
					if (("JJ".equals(bi.getSectionCode()) && rule.getJjWeight() < 1)) {
						continue;
					}

				}

				StringBuilder remark = new StringBuilder();
				//生产95%基建5%前期0%
				if (rule.getScjyWeight() > 0) {
					remark.append("生产").append(DoubleUtils.getRoundingNum(rule.getScjyWeight() * 100, 2)).append("%");
				}
				if (rule.getJjWeight() > 0) {
					remark.append("基建").append(DoubleUtils.getRoundingNum(rule.getJjWeight() * 100,2)).append("%");
				}
				if (rule.getQqWeight() > 0) {
					remark.append("前期").append(DoubleUtils.getRoundingNum(rule.getQqWeight() * 100,2)).append("%");
				}



				//判断多块牌子  是否在该序列加权平均
				List<MultipleBrand> dkpz = multipleBrandService.getMultipleBranList2(rule.getOrganizationId(), "", bi.getId(), evaluation.getCheckCycle(), evaluation.getYear(), month);
				List<MultipleBrand> filteredList = dkpz.stream()
					.filter(multipleBrand -> !"0".equals(multipleBrand.getParentId()))
					.collect(Collectors.toList());
				if(null != filteredList && filteredList.size()>0){
					//该单位在其它序列中 参与加权平均计算；跳过
					continue;
				}



				//存储多块牌子数据
				List<MultipleBrand> brands = multipleBrandService.getMultipleBranList2(rule.getOrganizationId(), "0", bi.getId(), evaluation.getCheckCycle(), evaluation.getYear(), month);

				Double totalProfit = 0.00;//累加收入
				Double spMultiply = 0.00;//收入
				Double scjyScore = 0.0;//分数

				String heighStandard = "";//高于基准分指标
				String lowStandard ="";//低于基准分指标

				if (null != brands && brands.size() > 0) {
					MultipleBrand mb = brands.get(0);
					//累加收入
					totalProfit = totalProfit + mb.getProfit();
					//累加收入和分数的积

					List<EvaluationScoreCount> scoreCount1 = evaluationScoreCountService.getEvaluationScoreCountList(organizationEvaluationId, rule.getId(), null, Arrays.asList("SCJY".split(",")),bi.getId());

					//基准分
					heighStandard = scoreCount1.stream().filter(s->s.getStandard().equals("1")).map(EvaluationScoreCount::getRemark).collect(Collectors.joining(System.lineSeparator()));
					lowStandard = scoreCount1.stream().filter(s->s.getStandard().equals("-1")).map(EvaluationScoreCount::getRemark).collect(Collectors.joining(System.lineSeparator()));


					//等分
					double score1 = scoreCount1.stream().mapToDouble(EvaluationScoreCount::getScore).sum();
					spMultiply = spMultiply + (mb.getProfit() * score1);

					List<MultipleBrand> parentBrands = multipleBrandService.getMultipleBranList2("", mb.getOrganizationId(),"", evaluation.getCheckCycle(), evaluation.getYear(), month);
					for (MultipleBrand pb : parentBrands) {
						//累加收入
						totalProfit = totalProfit + pb.getProfit();

						List<EvaluationScoreCount> scoreCount2 = evaluationScoreCountService.getEvaluationScoreCountList(organizationEvaluationId, ruleMap.get(pb.getOrganizationId()).getId(), null, Arrays.asList("SCJY".split(",")),pb.getBinSection());
						double score2 = scoreCount2.stream().mapToDouble(EvaluationScoreCount::getScore).sum();
						spMultiply = spMultiply + (pb.getProfit() * score2);

//						//基准分
//						heighStandard = heighStandard+scoreCount2.stream().filter(s->s.getStandard().equals("1")).map(EvaluationScoreCount::getRemark).collect(Collectors.joining(",","",","));
//						lowStandard = lowStandard+scoreCount2.stream().filter(s->s.getStandard().equals("-1")).map(EvaluationScoreCount::getRemark).collect(Collectors.joining(",","",","));

					}

					scjyScore = spMultiply / totalProfit;
				}



				//首页季度考评排行数据对象
				EvaluationPortal portal = new EvaluationPortal();
				portal.setId(StringUtils.getUUID());
				portal.setBinSection(bi.getSectionCode());
				portal.setCompany(rule.getOrganizationShortName());
				portal.setGrade("0");
				portal.setBaseScore("NDKP".equals(rule.getEvaluationCycle()) ?150:100);
				portal.setYear(evaluation.getYear());
				portal.setSeason(evaluation.getMonth());//季度和月份用同一个字段；不同维度存储数据
				portal.setMemo1(rule.getOrganizationId());//组织编码
				portal.setCreateTime(new Date());


				// 首页季度考评排行悬浮信息数据对象
				EvaluationPortalAttach portalAttach = new EvaluationPortalAttach();
				portalAttach.setId(StringUtils.getUUID());
				portalAttach.setPortalId(portal.getId());


				//报告详情数据对象
				EvaluateReportInfo reportInfo = new EvaluateReportInfo();
				reportInfo.setEvaluateReportId(evaluateReport.getId());
				reportInfo.setOrganizationId(rule.getOrganizationId());
				reportInfo.setOrganizationName(rule.getOrganizationShortName());

				//生产经营得分
				if(scjyScore>0){
					reportInfo.setScjyScore(DoubleUtils.keepPrecision(scjyScore, 2));//保存到考评报告

				}else {
					//保存到考评报告
					List<EvaluationScoreCount> scjyScoreCount = evaluationScoreCountService.getEvaluationScoreCountList(organizationEvaluationId, rule.getId(), null, Arrays.asList("SCJY".split(",")),bi.getId());
					scjyScore = scjyScoreCount.stream().mapToDouble(EvaluationScoreCount::getScore).sum();
					reportInfo.setScjyScore(DoubleUtils.keepPrecision(scjyScore, 2));

					//基准风
					heighStandard = scjyScoreCount.stream().filter(s->s.getStandard().equals("1")).map(EvaluationScoreCount::getRemark).collect(Collectors.joining(System.lineSeparator()));
					lowStandard = scjyScoreCount.stream().filter(s->s.getStandard().equals("-1")).map(EvaluationScoreCount::getRemark).collect(Collectors.joining(System.lineSeparator()));


				}

				//前期得分
				List<EvaluationScoreCount> qqScoreCount = evaluationScoreCountService.getEvaluationScoreCountList(organizationEvaluationId, rule.getId(), null, Arrays.asList("QQXM".split(",")),"");
				//前期得分
				Double qqScore = qqScoreCount.stream().mapToDouble(EvaluationScoreCount::getScore).sum();
				reportInfo.setQqScore(DoubleUtils.keepPrecision(qqScore, 2));
				//前期发展基准分
				heighStandard = heighStandard+System.lineSeparator()+qqScoreCount.stream().filter(s->s.getStandard().equals("1")).map(EvaluationScoreCount::getRemark).collect(Collectors.joining(System.lineSeparator()));
				lowStandard = lowStandard+System.lineSeparator()+qqScoreCount.stream().filter(s->s.getStandard().equals("-1")).map(EvaluationScoreCount::getRemark).collect(Collectors.joining(System.lineSeparator()));


				//基建得分
				List<EvaluationScoreCount> jjScoreCount = evaluationScoreCountService.getEvaluationScoreCountList(organizationEvaluationId, rule.getId(), null, Arrays.asList("JJXM".split(",")),"");
				//基建得分
				Double jjScore = jjScoreCount.stream().mapToDouble(EvaluationScoreCount::getScore).sum();
				reportInfo.setJjScore(DoubleUtils.keepPrecision(jjScore, 2));
				//基建基准分
				heighStandard = heighStandard+System.lineSeparator()+jjScoreCount.stream().filter(s->s.getStandard().equals("1")).map(EvaluationScoreCount::getRemark).collect(Collectors.joining(System.lineSeparator()));
				lowStandard = lowStandard+System.lineSeparator()+jjScoreCount.stream().filter(s->s.getStandard().equals("-1")).map(EvaluationScoreCount::getRemark).collect(Collectors.joining(System.lineSeparator()));



				List<EvaluationScoreCount> addSubScoreCount = evaluationScoreCountService.getEvaluationScoreCountList(organizationEvaluationId, rule.getId(), null, Arrays.asList("ZDZX,GLSX".split(",")),"");

				//重点专项
				List<EvaluationScoreCount> zdzxScoreCount  = addSubScoreCount.stream().filter(a->a.getBinStage().equals(CacheContext.bseCodeObject.get("ZDZX").getId())).collect(Collectors.toList());
				//重点专项-加分
				Double zdzxJfScore = zdzxScoreCount.stream().mapToDouble(EvaluationScoreCount::getJfScore).sum();
				String zdzxJfRemark = zdzxScoreCount.stream().map(EvaluationScoreCount::getJfRemark).collect(Collectors.joining(System.lineSeparator()));

				//重点专项-减分
				Double zdzxkfScore = zdzxScoreCount.stream().mapToDouble(EvaluationScoreCount::getKfScore).sum();
				String zdzxKfRemark = zdzxScoreCount.stream().map(EvaluationScoreCount::getKfRemark).collect(Collectors.joining(System.lineSeparator()));


				//管理事项
				List<EvaluationScoreCount> glsxScoreCount = addSubScoreCount.stream().filter(a -> a.getBinStage().equals(CacheContext.bseCodeObject.get("GLSX").getId())).collect(Collectors.toList());
				//管理事项-加分
				Double glsxjfScore = glsxScoreCount.stream().mapToDouble(EvaluationScoreCount::getJfScore).sum();
				String glsxJfRemark = glsxScoreCount.stream().map(EvaluationScoreCount::getJfRemark).filter(Objects::nonNull).collect(Collectors.joining(System.lineSeparator()));

				//管理事项-减分
				Double glsxkfScore = glsxScoreCount.stream().mapToDouble(EvaluationScoreCount::getKfScore).sum();
				String glsxKfRemark =  glsxScoreCount.stream().map(EvaluationScoreCount::getKfRemark).filter(Objects::nonNull).collect(Collectors.joining(System.lineSeparator()));



				//重点专项-加分事项
				if(zdzxJfScore>0){
					portal.setMatterHeigh(1);
					portalAttach.setMatterHeighNote(zdzxJfRemark);
				}
				//重点专项-扣分事项
				if(zdzxkfScore<0){
					portal.setMatterLow(-1);
					portalAttach.setMatterLowNote(zdzxKfRemark);
				}

				//管理事项-加分事项
				if(glsxjfScore>0){
					portal.setManageHeigh(1);
					portalAttach.setManageHeighNote(glsxJfRemark);
				}
				//管理事项-扣分事项
				if(glsxkfScore<0){
					portal.setManageLow(-1);
					portalAttach.setManageLowNote(glsxKfRemark);
				}


				//指标-高于基准分
				if(StringUtils.isNotEmpty(heighStandard)){
					portal.setStandardHeigh(1);
					portalAttach.setStandardHeighNote(heighStandard);
				}

				//指标-低于基准分 lowStandard
				if(StringUtils.isNotEmpty(lowStandard)){
					portal.setStandardLow(-1);
					portalAttach.setStandardLowNote(lowStandard);
				}

				//只有季度考评生成该记录
				if("JDKP".equals(rule.getEvaluationCycle())){
					portalList.add(portal);
					portalAttachList.add(portalAttach);
				}

				Double addSubScore = addSubScoreCount.stream().mapToDouble(EvaluationScoreCount::getScore).sum();
				//加分项
				double addScore = addSubScoreCount.stream().filter(a->a.getScore()>0).collect(Collectors.toList()).stream().mapToDouble(EvaluationScoreCount::getScore).sum();
				reportInfo.setAddScore(DoubleUtils.keepPrecision(addScore, 2));

				//减分项
				double subScore = addSubScoreCount.stream().filter(a->a.getScore()<0).collect(Collectors.toList()).stream().mapToDouble(EvaluationScoreCount::getScore).sum();
				reportInfo.setSubScore(DoubleUtils.keepPrecision(subScore, 2));

				//加减分合计
				reportInfo.setAddSubScore(DoubleUtils.keepPrecision(addSubScore, 2));

				//综合得分
				//经营分*比例 + 基建分*比例 + 前期分*比例 + 加减分合计
				Double zhScore = scjyScore*rule.getScjyWeight() + qqScore*rule.getQqWeight() + jjScore*rule.getJjWeight() + addSubScore;
				reportInfo.setZhScore(DoubleUtils.keepPrecision(zhScore, 2));
				//年度
				if("NDKP".equals(rule.getEvaluationCycle())){

					List<YearOperatingCoefficient> list = yearOperatingCoefficientService.getListByOrganizationId(rule.getOrganizationId(), rule.getBinSection(),evaluation.getYear());
					if(null != list && list.size()>0){
						//经营业绩系数
						reportInfo.setJyyjXs((null != list && list.size()>0) ? list.get(0).getK(): 1);
						//关键业绩得分
						//综合得分*经营业绩系数 +加扣分
						double gjyjXs = zhScore * list.get(0).getK() + addSubScore;
						reportInfo.setGjyjXs(gjyjXs);
					}else {
						//经营业绩系数
						reportInfo.setJyyjXs(1);
						//关键业绩系数
						reportInfo.setGjyjXs(1);
					}
					List<PartyBuildingMultiplier> multiplierList = partyBuildingMultiplierService.listByParam(null, rule.getOrganizationId(), null, evaluation.getYear());
					if(null != multiplierList && multiplierList.size()>0){
						//党建考核乘数
						reportInfo.setDjkhCs(Double.valueOf(multiplierList.get(0).getMultiplier()));
					}else {
						//党建考核乘数
						reportInfo.setDjkhCs(1);
					}
				}else {
					reportInfo.setScore(DoubleUtils.keepPrecision(zhScore, 2));
					reportInfo.setDiscountScore(1.0);
					portal.setScore(DoubleUtils.keepPrecision(zhScore, 2));//保存到首页排行
				}
				reportInfo.setRemark(remark.toString());
				evaluateRuleInfoList.add(reportInfo);
			}

			for (EvaluateReportInfo info : evaluateRuleInfoList) {
				b = evaluateReportInfoService.saveOrUpdate(info);
				if (!b) {
					throw new CustomException("综合考核评分汇总明细存储失败");
				}
			}

			if(null !=portalList && portalList.size()>0){

				Map<String, EvaluationPortalAttach> portalAttachMap = portalAttachList.stream()
					.collect(Collectors.toMap(EvaluationPortalAttach::getPortalId, Function.identity()));
				for (EvaluationPortal  po : portalList) {

					try {
						evaluationPortalService.saveSeasonPerformanceRanking(po,portalAttachMap.get(po.getId()));
					} catch (Exception e) {
						e.printStackTrace();
					}
				}
			}

		}
		return true;
	}


	@Override
	public IPage<EvaluateReport> list(Integer pageNum, Integer pageSize, String id, String evaluateReportName, String binSection, String organizationEvaluationId, String organizationType, String checkCycle, String year, String month) {
		Page<Indicator> page = new Page<>(pageNum, pageSize);
		if (null == pageNum || null == pageSize) {
			throw new CustomException("分页参数为空");
		}

		//季度
		if ("JDKP".equals(checkCycle)) {
			Date date = null;
			if (StringUtils.isNotEmpty(month)) {
				//随便填入年份和日期；只需要返回第几季度即可
				date = DateUtils.toDate("2023-" + month + "-01");
				month  =String.valueOf(DateUtils.getSeason(date));
			}

		}

		IPage<EvaluateReport> list = null;
		try {
			list = baseMapper.selectListPage(page, id, evaluateReportName, binSection, organizationEvaluationId, organizationType, checkCycle, year, month);
		} catch (Exception e) {
			e.printStackTrace();
		}

		return list;
	}


	@Override
	public List<EvaluateReport> list(String organizationEvaluationId, String binSection) {
		List<EvaluateReport> list = baseMapper.selectListAll(organizationEvaluationId,binSection);
		return list;
	}

	@Override
	public List<EvaluateReportInfoResDTO> list(String organizationEvaluationId, String binSection, String checkCycle, String year, String month) {



		return null;
	}


}

