package com.ims.eval.service.impl;

import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.ims.common.utils.DateUtils;
import com.ims.common.utils.StringUtils;
import com.ims.eval.cache.CacheContext;
import com.ims.eval.dao.EvaluationScoreCountMapper;
import com.ims.eval.entity.EvaluationScoreCount;
import com.ims.eval.service.IEvaluationScoreCountService;
import org.springframework.stereotype.Service;

import java.math.BigDecimal;
import java.util.*;
import java.util.stream.Collectors;

/**
 * <p>
 * 考评得分统计 服务实现类
 * </p>
 *
 * @author xugp
 * @since 2023-03-20
 */
@Service
public class EvaluatioinScoreCountServiceImpl extends ServiceImpl<EvaluationScoreCountMapper, EvaluationScoreCount> implements IEvaluationScoreCountService {




	@Override
	public List<EvaluationScoreCount> getListByEvaluationRuleId(String organizationEvaluationId, String organizationEvaluationRuleId, String binStage) {
		List<EvaluationScoreCount> list = baseMapper.selelctListByEvaluationRuleId(organizationEvaluationId, organizationEvaluationRuleId, binStage);
		return list;
	}


	@Override
	public Map<String, Object> evaluationList(String binSection, String binStage, String type, String condition, String date) {
		String binSectionId = CacheContext.bsnCodeObject.get(binSection).getId();
		Map<String, Object> result = new HashMap<>();
		List<Map<String, Object>> list = new ArrayList<>();
		String month = null;
		Calendar cal = Calendar.getInstance();
		cal.setTime(DateUtils.strToDate(date, DateUtils.format3));
		String year = String.valueOf(cal.get(Calendar.YEAR));
		if (condition.equals("JDKP")){
			month = String.valueOf(DateUtils.getSeason(cal.getTime()));
		}
		if (condition.equals("YDKP")){
			month = String.valueOf(cal.get(Calendar.MONTH) + 1);
		}
		try {
			list.addAll(baseMapper.selectEvaluationList(binSectionId, binStage, type, condition, year, month));
		} catch (Exception e) {
			return result;
		}

		Map<Object, List<Map<String, Object>>> collect = list.stream().collect(Collectors.groupingBy(c -> c.get("organization_name"), Collectors.toList()));
		boolean flag = true;
		List<Object> value = new ArrayList<>();
		Map<String, Object> title = new HashMap<>();
		for (Map.Entry<Object, List<Map<String, Object>>> entry : collect.entrySet()) {
			List<Map<String, Object>> v = entry.getValue();
			Map<String, Object> mp = new HashMap<>();
			for (int i = 0; i < v.size(); i++) {
				Map<String, Object> item = v.get(i);
				mp.put(item.get("indicator_code").toString(), item.get("score"));
				mp.put("organization_name", item.get("organization_name"));
				mp.put(item.get("indicator_code").toString() + "_flag", item.get("score") == item.get("standard_score") ? "0"
					: Double.valueOf(item.get("score").toString()) > Double.valueOf(item.get("standard_score").toString()) ? "1" : "-1");
			}
			DoubleSummaryStatistics sum = v.stream().collect(Collectors.summarizingDouble(e -> Double.valueOf(e.get("score").toString())));
			BigDecimal bd = new BigDecimal(sum.getSum());
			mp.put("score_total", bd.setScale(2, BigDecimal.ROUND_FLOOR).doubleValue());
			value.add(mp);

			// 添加标题
			if (flag) {
				Map<Object, List<Map<String, Object>>> types = v.stream().collect(Collectors.groupingBy(c -> c.get("type_name")));
				for (Map.Entry<Object, List<Map<String, Object>>> e : types.entrySet()) {
					List<Object> ls = new ArrayList<>();
					e.getValue().forEach(el -> {
						Map<String, Object> mt = new HashMap<>();
						mt.put("key", el.get("indicator_name"));
						mt.put("code", el.get("indicator_code").toString());
						ls.add(mt);
					});
					title.put((String) e.getKey(), ls);
				}
				flag = false;
			}
		}

		result.put("title", title);
		result.put("value", value);
		return result;
	}


	@Override
	public List<EvaluationScoreCount> getEvaluationScoreCountList(String organizationEvaluationId, String organizationEvaluationRuleId, String indicatorId,String binSection) {


		QueryWrapper<EvaluationScoreCount> qw = new QueryWrapper();


		if (StringUtils.isNotEmpty(organizationEvaluationId)) {
			qw.lambda().eq(EvaluationScoreCount::getOrganizationEvaluationId,organizationEvaluationId);
		}

		if (StringUtils.isNotEmpty(organizationEvaluationRuleId)) {
			qw.lambda().eq(EvaluationScoreCount::getOrganizationEvaluationRuleId,organizationEvaluationRuleId);
		}
		if (StringUtils.isNotEmpty(indicatorId)) {
			qw.lambda().eq(EvaluationScoreCount::getIndicatorId,indicatorId);
		}

		if (StringUtils.isNotEmpty(binSection)) {
			qw.lambda().eq(EvaluationScoreCount::getBinSection,binSection);
		}

		List<EvaluationScoreCount> list = baseMapper.selectList(qw);
		return list;
	}

	@Override
	public List<EvaluationScoreCount> getEvaluationScoreCountList(String organizationEvaluationId, String organizationEvaluationRuleId, List<String> indicatorIds) {


		QueryWrapper<EvaluationScoreCount> qw = new QueryWrapper();

		if (StringUtils.isNotEmpty(organizationEvaluationId)) {
			qw.lambda().eq(EvaluationScoreCount::getOrganizationEvaluationId,organizationEvaluationId);
		}
		if (StringUtils.isNotEmpty(organizationEvaluationRuleId)) {
			qw.lambda().eq(EvaluationScoreCount::getOrganizationEvaluationRuleId,organizationEvaluationRuleId);
		}
		if (null !=indicatorIds && indicatorIds.size()>0) {
			qw.lambda().in(EvaluationScoreCount::getIndicatorId,indicatorIds);
		}

		List<EvaluationScoreCount> list = baseMapper.selectList(qw);
		return list;
	}

	@Override
	public List<EvaluationScoreCount> getEvaluationScoreCountList(String organizationEvaluationId, String organizationEvaluationRuleId, List<String> indicatorIds, List<String> stageCode,String binSection) {
		List<EvaluationScoreCount>  list = baseMapper.selectEvaluationScoreCountList( organizationEvaluationId,  organizationEvaluationRuleId,  indicatorIds,  stageCode,binSection);
		return list;
	}
}
