package com.ims.eval.service.impl;

import cn.hutool.core.date.DateUtil;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.ims.common.utils.StringUtils;
import com.ims.eval.dao.EvaluationDeptBusinessContentMapper;
import com.ims.eval.entity.EvaluationDeptBusinessContent;
import com.ims.eval.entity.OrganizationStructure;
import com.ims.eval.service.IEvaluationDeptBusinessContentService;
import com.ims.eval.service.IOrganizationStructureService;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.io.UnsupportedEncodingException;
import java.net.URLDecoder;
import java.util.ArrayList;
import java.util.List;

/**
 * @author hlf
 * @date 2023/5/6 10:40
 * 文件说明：
 */
@Service
public class EvaluationDeptBusinessContentServiceImpl extends ServiceImpl<EvaluationDeptBusinessContentMapper, EvaluationDeptBusinessContent> implements IEvaluationDeptBusinessContentService {

	@Autowired
	private IOrganizationStructureService organizationStructureService;

	@Override
	public List<EvaluationDeptBusinessContent> detailsList(String businessPlanId, String deptId, String deptName) {
		if ("23031009".equals(deptId)) {//组织人事部
			deptId = "";
		}
		if ("null".equals(deptName)) {
			deptName = "";
		}
		return baseMapper.detailsList(businessPlanId, deptId, deptName);
	}

	@Override
	public boolean removeByBusinessPlanId(String businessPlanId) {
		return baseMapper.removeByBusinessPlanId(businessPlanId);
	}

	@Override
	public boolean saveOrUpdateBatch(List<EvaluationDeptBusinessContent> evaluationDeptBusinessContentList) throws UnsupportedEncodingException {
		List<EvaluationDeptBusinessContent> list = new ArrayList<>();
		for (EvaluationDeptBusinessContent evaluationDeptBusinessContent : evaluationDeptBusinessContentList) {
			if (!"".equals(evaluationDeptBusinessContent.getTargetValue()) && null != evaluationDeptBusinessContent.getTargetValue()) {
				String targetValue = URLDecoder.decode(evaluationDeptBusinessContent.getTargetValue(), "UTF-8");
				evaluationDeptBusinessContent.setTargetValue(targetValue);
			}
			if (!"".equals(evaluationDeptBusinessContent.getCompletionStatus()) && null != evaluationDeptBusinessContent.getCompletionStatus()) {
				String completionStatus = URLDecoder.decode(evaluationDeptBusinessContent.getCompletionStatus(), "UTF-8");
				evaluationDeptBusinessContent.setCompletionStatus(completionStatus);
			}
			if (evaluationDeptBusinessContent.getId() == null || "".equals(evaluationDeptBusinessContent.getId())) {//新增
				List<OrganizationStructure> organizationStructureList = organizationStructureService.getTree("23031001", 1, "140");
				OrganizationStructure organizationStructure = organizationStructureList.get(0).getChildren().stream().filter(item -> item.getId().equals(evaluationDeptBusinessContent.getDeptId())).findFirst().orElse(null);
				if (null != organizationStructure) {
					evaluationDeptBusinessContent.setDeptName(organizationStructure.getName());
					evaluationDeptBusinessContent.setAuditStatusKp("-1");
					evaluationDeptBusinessContent.setCreateTime(DateUtil.date());
					list.add(evaluationDeptBusinessContent);
				}
			} else {//修改
				list.add(evaluationDeptBusinessContent);
			}
		}
		return super.saveOrUpdateBatch(list);
	}

	@Override
	public List<EvaluationDeptBusinessContent> selectEvaluationDeptBusinessContentListByBusinessPlanId(String businessPlanId, String deptId) {
		return baseMapper.selectEvaluationDeptBusinessContentListByBusinessPlanId(businessPlanId, deptId);
	}

	@Override
	public boolean importPlanDataList(List<EvaluationDeptBusinessContent> evaluationDeptBusinessContentList) {
		List<EvaluationDeptBusinessContent> list = new ArrayList<>();
		for (EvaluationDeptBusinessContent evaluationDeptBusinessContent : evaluationDeptBusinessContentList) {
			QueryWrapper<EvaluationDeptBusinessContent> qw = new QueryWrapper<>();
			if (StringUtils.isNotEmpty(evaluationDeptBusinessContent.getBusinessPlanId())) {
				qw.lambda().eq(EvaluationDeptBusinessContent::getBusinessPlanId, evaluationDeptBusinessContent.getBusinessPlanId());
			}
			if (StringUtils.isNotEmpty(evaluationDeptBusinessContent.getTargetName())) {
				qw.lambda().eq(EvaluationDeptBusinessContent::getTargetName, evaluationDeptBusinessContent.getTargetName());
			}
			List<EvaluationDeptBusinessContent> objList = super.list(qw);
			if (objList.size() == 1) {//修改
				evaluationDeptBusinessContent.setId(objList.get(0).getId());
				evaluationDeptBusinessContent.setUpdateTime(DateUtil.date());
				list.add(evaluationDeptBusinessContent);
			} else if (objList.size() == 0) {//新增
				List<OrganizationStructure> organizationStructureList = organizationStructureService.getTree("23031001", 1, "140");
				OrganizationStructure organizationStructure = organizationStructureList.get(0).getChildren().stream().filter(item -> item.getId().equals(evaluationDeptBusinessContent.getDeptId())).findFirst().orElse(null);
				if (null != organizationStructure) {
					evaluationDeptBusinessContent.setAuditStatus("-1");
					evaluationDeptBusinessContent.setAuditStatusKp("-1");
					evaluationDeptBusinessContent.setDeptName(organizationStructure.getName());
					evaluationDeptBusinessContent.setCreateTime(DateUtil.date());
					list.add(evaluationDeptBusinessContent);
				}
			}
		}
		return super.saveOrUpdateBatch(list);
	}

	@Override
	public boolean importAssessmentDataList(List<EvaluationDeptBusinessContent> evaluationDeptBusinessContentList) {
		List<EvaluationDeptBusinessContent> list = new ArrayList<>();
		for (EvaluationDeptBusinessContent evaluationDeptBusinessContent : evaluationDeptBusinessContentList) {
			QueryWrapper<EvaluationDeptBusinessContent> qw = new QueryWrapper<>();
			if (StringUtils.isNotEmpty(evaluationDeptBusinessContent.getBusinessPlanId())) {
				qw.lambda().eq(EvaluationDeptBusinessContent::getBusinessPlanId, evaluationDeptBusinessContent.getBusinessPlanId());
			}
			if (StringUtils.isNotEmpty(evaluationDeptBusinessContent.getTargetName())) {
				qw.lambda().eq(EvaluationDeptBusinessContent::getTargetName, evaluationDeptBusinessContent.getTargetName());
			}
			List<EvaluationDeptBusinessContent> objList = super.list(qw);
			if (objList.size() == 1) {//修改
				evaluationDeptBusinessContent.setId(objList.get(0).getId());
				evaluationDeptBusinessContent.setUpdateTime(DateUtil.date());
				list.add(evaluationDeptBusinessContent);
			}
		}
		return super.updateBatchById(list);
	}
}
