package com.ims.eval.service.impl;

import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.ims.common.utils.StringUtils;
import com.ims.eval.cache.CacheContext;
import com.ims.eval.dao.IndicatorDictionaryMapper;
import com.ims.eval.config.CustomException;
import com.ims.eval.entity.IndicatorDictionary;
import com.ims.eval.entity.dto.request.IndicatorDictionaryDTO;
import com.ims.eval.service.IIndicatorDictionaryService;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;


import java.net.URLDecoder;
import java.util.*;
import java.util.stream.Collectors;

/**
 * <p>
 * 指标字典表 服务实现类
 * </p>
 *
 * @author wang
 * @since 2023-02-26
 */
@Service
public class IndicatorDictionaryServiceImpl extends ServiceImpl<IndicatorDictionaryMapper, IndicatorDictionary> implements IIndicatorDictionaryService {



	public List<IndicatorDictionary> list(String id, String indicatorId, String optionCode, String optionName) {
		QueryWrapper<IndicatorDictionary> qw = new QueryWrapper<>();

		if (StringUtils.isNotEmpty(id)) {
			qw.lambda().eq(IndicatorDictionary::getId, id);
		}

		if (StringUtils.isNotEmpty(indicatorId)) {
			qw.lambda().eq(IndicatorDictionary::getIndicatorId, indicatorId);
		}
		if (StringUtils.isNotEmpty(optionCode)) {
			qw.lambda().eq(IndicatorDictionary::getOptionCode, optionCode);
		}
		if (StringUtils.isNotEmpty(optionName)) {
			qw.lambda().like(IndicatorDictionary::getOptionName, optionName);
		}

		qw.lambda().orderByAsc(IndicatorDictionary::getOptionOrder);

		List<IndicatorDictionary> list = baseMapper.selectList(qw);

		return list;
	}


	@Override
	public boolean saveOrUpdate(IndicatorDictionary entity) {

		if (null != entity && (null == entity.getId() || "".equals(entity.getId().trim()))) {
			QueryWrapper<IndicatorDictionary> qw = new QueryWrapper<>();
			qw.lambda().eq(IndicatorDictionary::getIndicatorId, entity.getIndicatorId());
			qw.lambda().eq(IndicatorDictionary::getChildCode, entity.getChildCode());
			qw.lambda().eq(IndicatorDictionary::getOptionCode, entity.getOptionCode());
			List<IndicatorDictionary> list = baseMapper.selectList(qw);
			if (null != list && list.size() > 0) {
				throw new CustomException("当前属性code已存在");
			}
			entity.setCreateTime(new Date());
		} else {
			entity.setUpdateTime(new Date());
		}
		entity.setOptionName(null == CacheContext.ddNameMap.get(entity.getOptionCode()) ? "未知属性" : CacheContext.ddNameMap.get(entity.getOptionCode()));
		return super.saveOrUpdate(entity);
	}



	@Transactional
	@Override
	public boolean dictionarySaveBatch(List<IndicatorDictionaryDTO> dictionarys) throws  Exception{


		boolean b = false;
		for (IndicatorDictionaryDTO dto : dictionarys) {


			if(null == dto.getChildCode() || "".equals(dto.getChildCode().trim()) || null == dto.getChildName() || "".equals(dto.getChildName().trim())){
				throw new CustomException("子指标名或编码为空");
			}

			for (IndicatorDictionary i : dto.getList()) {

				String decode = URLDecoder.decode(i.getFormula(), "UTF-8");
				String formula = decode.replaceAll(" ", "+");

				if (null != i && (null == i.getId() || "".equals(i.getId().trim()))) {
					QueryWrapper<IndicatorDictionary> qw = new QueryWrapper<>();
					qw.lambda().eq(IndicatorDictionary::getIndicatorId, i.getIndicatorId());
					qw.lambda().eq(IndicatorDictionary::getChildCode, dto.getChildCode());
					qw.lambda().eq(IndicatorDictionary::getOptionCode, i.getOptionCode());
					List<IndicatorDictionary> list = baseMapper.selectList(qw);
					if (null != list && list.size() > 0) {
						throw new CustomException("当前属性code已存在");
					}
					i.setChildCode(dto.getChildCode());
					i.setFormula(formula);
					i.setChildName(dto.getChildName());
					i.setCreateTime(new Date());
					i.setOptionName(null == CacheContext.ddNameMap.get(i.getOptionCode()) ? "未知属性" : CacheContext.ddNameMap.get(i.getOptionCode()));
				} else {
					i.setChildCode(dto.getChildCode());
					i.setFormula(formula);
					i.setChildName(dto.getChildName());
					i.setOptionName(null == CacheContext.ddNameMap.get(i.getOptionCode()) ? "未知属性" : CacheContext.ddNameMap.get(i.getOptionCode()));
					i.setUpdateTime(new Date());
				}

				b = super.saveOrUpdate(i);
			}
		}
		return b;
	}

	@Override
	public List<IndicatorDictionaryDTO> list(String indicatorId, String optionCode, String optionName) {

		List<IndicatorDictionaryDTO> dtoList = new ArrayList<>();

		QueryWrapper<IndicatorDictionary> qw = new QueryWrapper<>();
		if (StringUtils.isNotEmpty(indicatorId)) {
			qw.lambda().eq(IndicatorDictionary::getIndicatorId, indicatorId);
		}
		if (StringUtils.isNotEmpty(optionCode)) {
			qw.lambda().eq(IndicatorDictionary::getOptionCode, optionCode);
		}
		if (StringUtils.isNotEmpty(optionName)) {
			qw.lambda().like(IndicatorDictionary::getOptionName, optionName);
		}
		qw.lambda().orderByAsc(IndicatorDictionary::getOptionOrder);

		List<IndicatorDictionary> list = baseMapper.selectList(qw);

		Map<String, List<IndicatorDictionary>> map = list.stream().collect(Collectors.groupingBy(ord -> ord.getChildCode() + "," + ord.getChildName()));

		map.forEach((k, v) -> {
			String[] zb = k.split(",");
			IndicatorDictionaryDTO dto = new IndicatorDictionaryDTO();
			dto.setChildCode(zb[0]);
			dto.setChildName(zb[1]);
			dto.setList(v);
			dtoList.add(dto);
		});

		return dtoList;
	}


	@Override
	public IndicatorDictionary getIndicatorDictionaryByCode(String indicatorId, String childOptionCode) {

		QueryWrapper<IndicatorDictionary> qw = new QueryWrapper<>();
		if (StringUtils.isNotEmpty(indicatorId)) {
			qw.lambda().eq(IndicatorDictionary::getIndicatorId, indicatorId);
		}
		if (StringUtils.isNotEmpty(childOptionCode)) {
			qw.eq("CONCAT(child_code, '_', option_code)", childOptionCode);
		}
		List<IndicatorDictionary> list = baseMapper.selectList(qw);
		if(null != list && list.size()>0){

			return list.get(0);
		}
		return null;
	}
}
