package com.ims.eval.service.impl;

import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.conditions.update.LambdaUpdateWrapper;
import com.baomidou.mybatisplus.core.conditions.update.UpdateWrapper;
import com.ims.eval.config.CustomException;
import com.ims.eval.entity.IndicatorType;
import com.ims.eval.dao.IndicatorTypeMapper;
import com.ims.eval.service.IIndicatorTypeService;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import java.io.Serializable;
import java.util.Collection;
import java.util.List;

/**
 * <p>
 * 指标分类 服务实现类
 * </p>
 *
 * @author wang
 * @since 2023-03-09
 */
@Service
public class IndicatorTypeServiceImpl extends ServiceImpl<IndicatorTypeMapper, IndicatorType> implements IIndicatorTypeService {

	@Override
	public List<IndicatorType> list(String id, String typeName, String typeCode) {

		QueryWrapper<IndicatorType> qw = new QueryWrapper();
		qw.lambda().eq(IndicatorType::getId, id);
		qw.lambda().like(IndicatorType::getTypeName, typeName);
		qw.lambda().eq(IndicatorType::getTypeCode, typeCode);
		qw.lambda().orderByAsc(IndicatorType::getOrderNum);
		List<IndicatorType> list = baseMapper.selectList(qw);

		return list;
	}

	@Transactional
	@Override
	public boolean saveOrUpdate(IndicatorType entity) {

		if (null != entity && (null != entity.getId() && !"".equals(entity.getId().trim()))) {
			IndicatorType type = baseMapper.selectById(entity.getId());
			type.setDelFlag(true);
			baseMapper.updateById(type);
		}

		QueryWrapper<IndicatorType> qw = new QueryWrapper();
		qw.lambda().eq(IndicatorType::getTypeCode, entity.getTypeCode());
		qw.lambda().eq(IndicatorType::getDelFlag, false);
		List<IndicatorType> list = baseMapper.selectList(qw);
		if (null != list && list.size() > 0) {
			throw new CustomException("当前分类已存在");
		}

		int count = baseMapper.selectCount(null);
		entity.setId(null);
		entity.setOrderNum(count);
		entity.setDelFlag(false);
		return super.saveOrUpdate(entity);
	}

	@Override
	public boolean removeByIds(Collection<? extends Serializable> idList) {

		try {
			if (idList.size()>0) {
				for (Object  id : idList){
					IndicatorType type = baseMapper.selectById(id.toString());
					type.setDelFlag(true);
					baseMapper.updateById(type);
				}
			}
		} catch (Exception e) {
			e.printStackTrace();
			return false;
		}

		return true;
	}
}
