package com.ims.eval.service.impl;

import cn.hutool.core.collection.CollectionUtil;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.ims.eval.entity.Indicator;
import com.ims.eval.entity.MultipleBrand;
import com.ims.eval.dao.MultipleBrandMapper;
import com.ims.eval.service.IMultipleBrandService;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import org.springframework.stereotype.Service;

import java.util.ArrayList;
import java.util.List;
import java.util.Map;
import java.util.stream.Collectors;

/**
 * <p>
 *  服务实现类
 * </p>
 *
 * @author wang
 * @since 2023-05-22
 */
@Service
public class MultipleBrandServiceImpl extends ServiceImpl<MultipleBrandMapper, MultipleBrand> implements IMultipleBrandService {

	@Override
	public IPage<MultipleBrand> getMultipleBranTree(Integer pageNum, Integer pageSize,String id, String parentId,String  binSection,String  checkCycle,String year,String month) {
		Page<Indicator> page = new Page<>(pageNum, pageSize);
		IPage<MultipleBrand> list  =  baseMapper.selectMultipleBranTree(page,id,"0",binSection,checkCycle,year,month);
		List<MultipleBrand> list2  =  baseMapper.selectMultipleBranList(id,"","",checkCycle,year,month);

		List<MultipleBrand> tree = convert(list.getRecords(),list2);
		list.setRecords(tree);
		return list;
	}

	@Override
	public List<MultipleBrand> getMultipleBranList(String id, String parentId,String  binSection,String  checkCycle,String year,String month) {
		List<MultipleBrand> list  = baseMapper.selectMultipleBranList(id,parentId,binSection,checkCycle,year,month);
		return list;
	}

	@Override
	public List<MultipleBrand> getMultipleBranList2( String organizationId, String parentId, String binSection, String checkCycle, String year, String month) {
		List<MultipleBrand> list  = baseMapper.selectMultipleBranList2(organizationId,parentId,binSection,checkCycle,year,month);
		return list;
	}


	/**
	 * 转换为有树形结构的列表
	 *
	 * @param brands 源数据所有数据
	 * @return
	 */
	public List<MultipleBrand> convert(List<MultipleBrand> result,List<MultipleBrand> brands) {
		if (CollectionUtil.isEmpty(brands)) {
			return new ArrayList<>();
		}

		brands = brands.stream().filter(f->!"0".equals(f.getParentId())).collect(Collectors.toList());
//		Map<String ,List<MultipleBrand>> brandMap = brands.stream().collect(Collectors.groupingBy(MultipleBrand::getParentId));

//		Map<String, List<MultipleBrand>> groupedBrands = brands.stream()
//			.collect(Collectors.groupingBy(brand -> brand.getParentId() + "_" + brand.getCheckCycle()));
		for (MultipleBrand brandsVO : result) {
			// 循环一级之后子级
			List<MultipleBrand>  newbrands = null;
			if("NDKP".equals(brandsVO.getCheckCycle())){

				newbrands = brands.stream().filter(f->brandsVO.getOrganizationId().equals(f.getParentId()) && brandsVO.getCheckCycle().equals(f.getCheckCycle())).collect(Collectors.toList());
			}else{
				newbrands = brands.stream().filter(f->brandsVO.getOrganizationId().equals(f.getParentId()) && brandsVO.getCheckCycle().equals(f.getCheckCycle()) && brandsVO.getMonth().equals(f.getMonth())).collect(Collectors.toList());
			}
			brandsVO.setChildren(newbrands);
		}
		return result;
	}


	/**
	 * 循环设置子级
	 *
	 * @param list         所有元素列表
	 * @param treeSelectVO 父级元素
	 * @return 父级元素
	 */
	public static MultipleBrand getChildren(List<MultipleBrand> list, MultipleBrand treeSelectVO) {
		for (MultipleBrand MultipleBrand : list) {
			if (treeSelectVO.getOrganizationId().equals(MultipleBrand.getParentId())) {
				List<MultipleBrand> children = CollectionUtil.isEmpty(treeSelectVO.getChildren()) ? new ArrayList<>() : treeSelectVO.getChildren();
				children.add(getChildren(list, MultipleBrand));
				treeSelectVO.setChildren(children);

			}
		}
		return treeSelectVO;
	}
}
