package com.ims.eval.service.impl;

import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.ims.common.utils.DoubleUtils;
import com.ims.common.utils.StringUtils;
import com.ims.eval.cache.CacheContext;
import com.ims.eval.config.CustomException;
import com.ims.eval.entity.*;
import com.ims.eval.dao.OrganizationEvaluationCommonInfoMapper;
import com.ims.eval.entity.dto.request.AddEvaluationInfoDTO;
import com.ims.eval.entity.dto.response.IndicatorResDTO;
import com.ims.eval.service.*;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.ims.eval.util.MathCalculatorUtil;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import java.util.*;
import java.util.stream.Collectors;

/**
 * <p>
 *  服务实现类
 * </p>
 *
 * @author wang
 * @since 2023-09-11
 */
@Service
public class OrganizationEvaluationCommonInfoServiceImpl extends ServiceImpl<OrganizationEvaluationCommonInfoMapper, OrganizationEvaluationCommonInfo> implements IOrganizationEvaluationCommonInfoService {



	@Autowired
	private IOrganizationEvaluationService organizationEvaluationService;

	@Autowired
	private IOrganizationEvaluationRuleService organizationEvaluationRuleService;

	@Autowired
	private IIndicatorService indicatorService;

	@Autowired
	private IIndicatorDictionaryService indicatorDictionaryService;


	@Autowired
	private IOrganizationEvaluationInfoService organizationEvaluationInfoService;


	@Transactional
	@Override
	public boolean addEvaluationCommonInfo(List<AddEvaluationInfoDTO> addEvaluationInfoDTOs) {


		//获取公共性息；为后面汇总业务做准备业务做准备
		String organizationEvaluationId = null;
		String organizationId = null;
		String indicatorId = null;




		//0.移除现有数据
		boolean flag = true;
		Integer indx = 1;
		for (AddEvaluationInfoDTO addEvaluationInfoDTO : addEvaluationInfoDTOs) {
			if(flag){
				flag = false;
				//5.删除原有记录
				List<OrganizationEvaluationCommonInfo> infoResDTOList = baseMapper.selectListAll(addEvaluationInfoDTO.getOrganizationEvaluationId(),addEvaluationInfoDTO.getIndicatorId(),addEvaluationInfoDTO.getOrganizationId(),"");
				//判断是否已存在记录
				if(null != infoResDTOList && infoResDTOList.size()>0){
					//当前部门存在记录；移除保存
					List<String> commonIds = infoResDTOList.stream().map(OrganizationEvaluationCommonInfo::getId).collect(Collectors.toList());
					boolean b = this.removeByIds(commonIds);
					if(!b){
						return b;
					}
				}
			}

			//1.获取考评id(获取此考评记录)
			organizationEvaluationId = addEvaluationInfoDTO.getOrganizationEvaluationId();
			organizationId = addEvaluationInfoDTO.getOrganizationId();
			indicatorId = addEvaluationInfoDTO.getIndicatorId();

			if (StringUtils.isEmpty(organizationEvaluationId)) {
				throw new CustomException("保存失败");
			}
			OrganizationEvaluation organizationEvaluation = organizationEvaluationService.getById(organizationEvaluationId);

			if (null == organizationEvaluation) {
				throw new CustomException("保存失败");
			}
			organizationEvaluation.getCheckCycle();

			//2.获取组织id 和所属板块（判断该板块下是否有此组织的考核）
			String organizationType = organizationEvaluation.getOrganizationType();//考评类型 DWKP
			String checkCycle = organizationEvaluation.getCheckCycle();//考评周期-月度、季度
			String year = organizationEvaluation.getYear();//年份

			//3.查询当前对应的考评权重（需要考评的单位）
			List<OrganizationEvaluationRule> evaluationRules = organizationEvaluationRuleService.getOrganizationEvaluationRuleByYearAndCycle(organizationType, checkCycle, year, addEvaluationInfoDTO.getOrganizationId());
			if (null == evaluationRules || evaluationRules.size() <= 0) {
				throw new CustomException("当前组织没有参与考评");
			}
			OrganizationEvaluationRule evaluationRule = evaluationRules.get(0);

			//4.获取指标信息
			List<String> indicatorIds = Arrays.asList(addEvaluationInfoDTO.getIndicatorId().split(","));
			List<IndicatorResDTO> dictionaryList = indicatorService.listByIds(indicatorIds);
			Map<String, List<IndicatorResDTO>> groupedChildCode = dictionaryList.stream()
				.collect(Collectors.groupingBy(d -> d.getChildCode() + "," + d.getChildName()));
			Map<String, Object> optionMap = addEvaluationInfoDTO.getOptionMap();



			//5.获取部门性息
			DataDictionary  di =null;
			if (StringUtils.isNotEmpty(optionMap.get("deptName").toString()) && CacheContext.ddSuperKeyMap.containsKey("BM0001")){
				Optional<DataDictionary> any = CacheContext.ddSuperKeyMap.get("BM0001").stream().filter(t -> t.getKeyName().equals(optionMap.get("deptName").toString())).findAny();
				 di =  any.isPresent() ?  any.get() : null;

			}




			//6.保存指标
			for (Map.Entry<String, List<IndicatorResDTO>> childCodeEntry : groupedChildCode.entrySet()) {

				for (IndicatorResDTO d : childCodeEntry.getValue()) {
					OrganizationEvaluationCommonInfo info = new OrganizationEvaluationCommonInfo();
					info.setOrganizationEvaluationId(addEvaluationInfoDTO.getOrganizationEvaluationId());
					info.setOrganizationId(evaluationRule.getOrganizationId());
					info.setIndicatorId(addEvaluationInfoDTO.getIndicatorId());
					info.setIndicatorDictionaryId(d.getId());
					info.setOptionCode(d.getOptionCode());
					info.setIsQuantified(d.getIsQuantified());
					if (null != optionMap.get(d.getChildCode() + "_" + d.getOptionCode())) {
						Object value = optionMap.get(d.getChildCode() + "_" +d.getOptionCode());
						if (null != value) {
							info.setNonQuantifiedValue(String.valueOf(value));
							if (d.getIsQuantified()) {

								if (MathCalculatorUtil.isNumber(value.toString())) {
									info.setQuantifiedValue(Double.valueOf(value.toString()));
								} else {
									info.setQuantifiedValue(0);
								}
							}
						}
					}
					info.setChildCode(d.getChildCode());
					info.setDeptId(di.getDataKey());
					info.setDeptName(optionMap.get("deptName").toString());
					info.setIndx(indx);//标记
					saveOrUpdate(info);
				}
			}
			indx++;
		}

		//获取明细
		List<OrganizationEvaluationInfo> evaluationInfoList = organizationEvaluationInfoService.getConvergeCommonInfo(organizationEvaluationId,organizationId,indicatorId);
		List<OrganizationEvaluationCommonInfo> evaluationCommonInfos = getEvaluationCommonInfo(organizationEvaluationId,organizationId,indicatorId,"");

		//按照指标项id分组
		Map<String, List<OrganizationEvaluationCommonInfo>> groupOptionData = evaluationCommonInfos.stream()
			.collect(Collectors.groupingBy(OrganizationEvaluationCommonInfo::getIndicatorDictionaryId));

		for (OrganizationEvaluationInfo oei : evaluationInfoList){
			List<OrganizationEvaluationCommonInfo> oeci = null != groupOptionData.get(oei.getIndicatorDictionaryId()) ? groupOptionData.get(oei.getIndicatorDictionaryId()) :null;
			if(null !=oeci ){
				if(oei.getIsQuantified()){
					double ageSum = oeci.stream().mapToDouble(OrganizationEvaluationCommonInfo::getQuantifiedValue).sum();
					oei.setQuantifiedValue(DoubleUtils.getRoundingNum(ageSum,2));
					oei.setNonQuantifiedValue(DoubleUtils.getRoundingNum(ageSum,2)+"");
				}else {
					oei.setNonQuantifiedValue("见详情");
				}
			}
		}
		boolean b = organizationEvaluationInfoService.saveOrUpdateBatch(evaluationInfoList);
		return b;
	}


	@Override
	public Map getListAll(String organizationEvaluationId, String indicatorId, String organizationId, String deptId) {

		//生成标题
		List<IndicatorDictionary> dictionaryList = indicatorDictionaryService.list("", indicatorId, "", "");
		Map<String, List<IndicatorDictionary>> groupedChildCode = dictionaryList.stream()
			.collect(Collectors.groupingBy(d -> d.getChildCode() + "," + d.getChildName()));
			Map data = new HashMap();
			Map title = new LinkedHashMap();
			for (Map.Entry<String, List<IndicatorDictionary>> childCodeEntry : groupedChildCode.entrySet()) {
				List<Map> titleArray = new ArrayList<>();
				for (IndicatorDictionary d : childCodeEntry.getValue()) {
					Map titlemap = new LinkedHashMap();
					if (!d.getIsShow()) {
						continue;
					}
					titlemap.put("key", d.getOptionName());//名称
					titlemap.put("code",d.getChildCode()+"_"+d.getOptionCode());//名称
					titlemap.put("flag", d.getIsQuantified()?"1":"2");//是否量化
					titleArray.add(titlemap);
				}
				title.put(childCodeEntry.getKey().split(",")[1], titleArray);
			}
			data.put("title", title);
			//获取数据

		List<OrganizationEvaluationCommonInfo> list = baseMapper.selectListAll(organizationEvaluationId,indicatorId,organizationId,deptId);

		//根据部门id分组
//		Map<String, List<OrganizationEvaluationCommonInfo>> groupedData = list.stream()
//			.collect(Collectors.groupingBy(OrganizationEvaluationCommonInfo::getDeptId));

		Map<String, List<OrganizationEvaluationCommonInfo>> groupedData = list.stream()
			.collect(Collectors.groupingBy(item -> item.getDeptId() + "_" + item.getIndx()));



		List<Map> mapList = new ArrayList<>();
		//遍历按照部门分组的数据
 		for (Map.Entry<String, List<OrganizationEvaluationCommonInfo>> entry : groupedData.entrySet()) {

			Map indicatormap = new HashMap();
			List<OrganizationEvaluationCommonInfo> dtoList = entry.getValue();
			Map<String, OrganizationEvaluationCommonInfo> resultMap = dtoList.stream()
				.collect(Collectors.toMap(
					dto -> dto.getChildCode() + "_" + dto.getOptionCode(),
					dto -> dto,
					(oldValue, newValue) -> oldValue));

			boolean mark = true;//标记给单位
			for (IndicatorDictionary d : dictionaryList) {
				if (!d.getIsShow()) {
					continue;
				}

				if (mark) {
					indicatormap.put("deptName", resultMap.get(d.getChildCode()+"_"+d.getOptionCode()).getDeptName());//公司名
					indicatormap.put("deptId", resultMap.get(d.getChildCode()+"_"+d.getOptionCode()).getDeptId());//公司名
					indicatormap.put("organizationId", resultMap.get(d.getChildCode()+"_"+d.getOptionCode()).getOrganizationId());//公司id
					mark = false;
				}

				//指标属性
				indicatormap.put(d.getChildCode() + "_" + d.getOptionCode(), null == resultMap.get(d.getChildCode()+"_"+d.getOptionCode()).getNonQuantifiedValue() ? "" : resultMap.get(d.getChildCode()+"_"+d.getOptionCode()).getNonQuantifiedValue());
				//指标id
				indicatormap.put("ID_" + d.getChildCode() + "_" + d.getOptionCode(), resultMap.get(d.getChildCode()+"_"+d.getOptionCode()).getId());
				//是否量化
				indicatormap.put("IS_LH_" +d.getChildCode() + "_" + d.getOptionCode(),resultMap.get(d.getChildCode()+"_"+d.getOptionCode()).getIsQuantified()?"1":"2");

			}
			mapList.add(indicatormap);

		}
		data.put("value", mapList);

		return data;
	}

	@Override
	public List<OrganizationEvaluationCommonInfo> getEvaluationCommonInfo(String organizationEvaluationId, String organizationId, String indicatorId, String indicatorDictionaryId) {
		QueryWrapper<OrganizationEvaluationCommonInfo> qw = new QueryWrapper<>();

		if (StringUtils.isNotEmpty(organizationEvaluationId)) {
			qw.lambda().eq(OrganizationEvaluationCommonInfo::getOrganizationEvaluationId, organizationEvaluationId);
		}
		if (StringUtils.isNotEmpty(organizationId)) {
			qw.lambda().eq(OrganizationEvaluationCommonInfo::getOrganizationId, organizationId);
		}
		if (StringUtils.isNotEmpty(indicatorId)) {
			qw.lambda().eq(OrganizationEvaluationCommonInfo::getIndicatorId, indicatorId);
		}

		if (StringUtils.isNotEmpty(indicatorDictionaryId)) {
			qw.lambda().eq(OrganizationEvaluationCommonInfo::getIndicatorDictionaryId, indicatorDictionaryId);
		}

		List<OrganizationEvaluationCommonInfo> list = baseMapper.selectList(qw);
		return list;
	}

	@Override
	public boolean hbCommonInfo(String organizationEvaluationId, String organizationId, String indicatorId) {
		//获取明细
		List<OrganizationEvaluationInfo> evaluationInfoList = organizationEvaluationInfoService.getConvergeCommonInfo(organizationEvaluationId,organizationId,indicatorId);
		List<OrganizationEvaluationCommonInfo> evaluationCommonInfos = getEvaluationCommonInfo(organizationEvaluationId,organizationId,indicatorId,"");

		//按照指标项id分组
		Map<String, List<OrganizationEvaluationCommonInfo>> groupOptionData = evaluationCommonInfos.stream()
			.collect(Collectors.groupingBy(OrganizationEvaluationCommonInfo::getIndicatorDictionaryId));

		for (OrganizationEvaluationInfo oei : evaluationInfoList){
			List<OrganizationEvaluationCommonInfo> oeci = null != groupOptionData.get(oei.getIndicatorDictionaryId()) ? groupOptionData.get(oei.getIndicatorDictionaryId()) :null;
			if(null !=oeci ){
				if(oei.getIsQuantified()){
					double ageSum = oeci.stream().mapToDouble(OrganizationEvaluationCommonInfo::getQuantifiedValue).sum();
					oei.setQuantifiedValue(DoubleUtils.getRoundingNum(ageSum,2));
					oei.setNonQuantifiedValue(DoubleUtils.getRoundingNum(ageSum,2)+"");
				}else {
					oei.setNonQuantifiedValue("见详情");
				}
			}
		}
		boolean b = organizationEvaluationInfoService.saveOrUpdateBatch(evaluationInfoList);
		return b;
	}

	@Transactional
	@Override
	public boolean removeCommonInfos(String[] ids) {
		OrganizationEvaluationCommonInfo info = this.getById(ids[0]);
		boolean b = this.removeByIds(Arrays.asList(ids));
		if(!b){
			return b;
		}
		b = hbCommonInfo(info.getOrganizationEvaluationId(),info.getOrganizationId(),info.getIndicatorId());
		return b;
	}


	//	@Override
//	public boolean removeCommonInfos(String organizationEvaluationId, String indicatorId, String organizationId, String deptId) {
//		QueryWrapper<OrganizationEvaluationCommonInfo> qw = new QueryWrapper<>();
//
//		if (StringUtils.isNotEmpty(organizationEvaluationId)) {
//			qw.lambda().eq(OrganizationEvaluationCommonInfo::getOrganizationEvaluationId, organizationEvaluationId);
//		}
//		if (StringUtils.isNotEmpty(indicatorId)) {
//			qw.lambda().eq(OrganizationEvaluationCommonInfo::getIndicatorId, indicatorId);
//		}
//		if (StringUtils.isNotEmpty(organizationId)) {
//			qw.lambda().eq(OrganizationEvaluationCommonInfo::getOrganizationId, organizationId);
//		}
//		if (StringUtils.isNotEmpty(deptId)) {
//			qw.lambda().eq(OrganizationEvaluationCommonInfo::getDeptId, deptId);
//		}
//		List<OrganizationEvaluationCommonInfo> list = baseMapper.selectList(qw);
//
//		//获取
//		List<String> commonIds = list.stream().map(OrganizationEvaluationCommonInfo::getId).collect(Collectors.toList());
//		boolean b = this.removeByIds(commonIds);
//
//		return b;
//	}
}
