package com.ims.eval.service.impl;

import com.alibaba.fastjson.JSONObject;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.ims.common.utils.DoubleUtils;
import com.ims.common.utils.FormulaUtils;
import com.ims.common.utils.StringUtils;
import com.ims.eval.cache.CacheContext;
import com.ims.eval.config.CustomException;
import com.ims.eval.dao.CalculateIndicatorItemInfoMapper;
import com.ims.eval.entity.*;
import com.ims.eval.dao.OrganizationEvaluationInfoMapper;
import com.ims.eval.entity.dto.request.AddEvaluationInfoDTO;
import com.ims.eval.entity.dto.response.IndicatorResDTO;
import com.ims.eval.entity.dto.response.MyuserResDTO;
import com.ims.eval.entity.dto.response.OrganizationEvaluationInfoResDTO;
import com.ims.eval.service.*;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.ims.eval.service.custom.impl.WorkflowServiceImpl;
import com.ims.eval.util.MathCalculatorUtil;
import lombok.extern.log4j.Log4j2;
import org.apache.commons.io.output.ByteArrayOutputStream;
import org.apache.poi.hssf.usermodel.HSSFCell;
import org.apache.poi.ss.usermodel.*;
import org.apache.poi.ss.util.CellRangeAddress;
import org.apache.poi.ss.util.CellRangeAddressList;
import org.apache.poi.xssf.usermodel.XSSFWorkbook;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.web.multipart.MultipartFile;

import javax.annotation.Resource;
import javax.servlet.http.HttpServletRequest;
import java.io.IOException;
import java.io.InputStream;
import java.math.BigDecimal;
import java.util.*;
import java.util.regex.Matcher;
import java.util.regex.Pattern;
import java.util.stream.Collectors;

import static com.ims.eval.util.ExcelUtil.readExcel;

/**
 * <p>
 * 考评指标明细

 服务实现类
 * </p>
 *
 * @author wang
 * @since 2023-03-16
 */
@Service
@Log4j2
public class OrganizationEvaluationInfoServiceImpl extends ServiceImpl<OrganizationEvaluationInfoMapper, OrganizationEvaluationInfo> implements IOrganizationEvaluationInfoService {

	@Autowired
	private ICalculateIndicatorItemInfoService iCalculateIndicatorItemInfoService;


	@Autowired
	private IIndicatorDictionaryService indicatorDictionaryService;

	@Autowired
	private IUserService userService;

	@Autowired
	private IEvaluationScoreCountService evaluatioinScoreCountService;


	@Resource
	private CalculateIndicatorItemInfoMapper calculateIndicatorItemInfoMapper;

	@Autowired
	private IIndicatorService indicatorService;


	@Autowired
	private IOrganizationEvaluationService organizationEvaluationService;

	@Autowired
	private  IOrganizationEvaluationRuleService organizationEvaluationRuleService;

	@Autowired
	private WorkflowServiceImpl workflowService;


	@Autowired
	private ICiteCalculationIndicatorService citeCalculationIndicatorService;





	@Override
	public Map finishValueList(String organizationEvaluationId, String dept, HttpServletRequest request) {

		List<OrganizationEvaluationInfoResDTO> list = baseMapper.selectListEvaluationInfoId(organizationEvaluationId,dept,"","");//默认获取指定的怎目标
		list.stream().forEach(i->{
			i.setUpdateMark(true);
		});
		Map<String  ,List<OrganizationEvaluationInfoResDTO>> map  = list.stream().collect(Collectors.groupingBy(OrganizationEvaluationInfoResDTO::getIsQuantified));
		map.put("quantifiedList", map.remove(true));
		map.put("nonQuantifiedList", map.remove(false));
		return map;
	}

	@Transactional
	@Override
	public boolean saveBatchDto(List<OrganizationEvaluationInfo> organizationEvaluationInfoList) throws Exception {
		for (OrganizationEvaluationInfo entity : organizationEvaluationInfoList) {

			if (null != entity && null != entity.getId() && !"".equals(entity.getId().trim())) {
				OrganizationEvaluationInfo info = baseMapper.selectById(entity.getId());

				info.setQuantifiedValue(entity.getQuantifiedValue());//量化值
				if(!entity.getIsQuantified() && "JKF".equals(info.getOptionCode())){//加扣分
					if(StringUtils.isNumber(entity.getNonQuantifiedValue())){
						info.setQuantifiedValue(Double.valueOf(entity.getNonQuantifiedValue()));
					}
				}

				if(!entity.getIsQuantified() && "JCF".equals(info.getOptionCode())){//基础分
					if(StringUtils.isNumber(entity.getNonQuantifiedValue())){
						info.setQuantifiedValue(Double.valueOf(entity.getNonQuantifiedValue()));
					}
				}

				info.setNonQuantifiedValue(entity.getNonQuantifiedValue());//非量化值
				info.setUpdateTime(new Date());//更新时间
				info.setUpdateBy(entity.getUpdateBy());//更新者
				info.setRemark(entity.getRemark());//备注
				info.setState(entity.getState());//初始状态
				boolean b = super.saveOrUpdate(info);
				if(!b){
					throw new CustomException("更新失败");
				}
			}else {
				entity.setCreateTime(new Date());

				if (StringUtils.isEmpty(entity.getIndicatorId())) {
					throw new CustomException("保存失败");
				}
				List<IndicatorDictionary> dictionaryList =  indicatorDictionaryService.list("",entity.getIndicatorId(),"","");
				for(IndicatorDictionary d : dictionaryList){

					OrganizationEvaluationInfo addinfo = new OrganizationEvaluationInfo();
					addinfo.setOrganizationEvaluationId(entity.getOrganizationEvaluationId());
					addinfo.setOrganizationEvaluationRuleId(entity.getOrganizationEvaluationRuleId());
					addinfo.setIndicatorId(entity.getIndicatorId());
					addinfo.setIndicatorDictionaryId(d.getId());
					addinfo.setOptionCode(d.getOptionCode());
					addinfo.setChildCode(d.getChildCode());
					addinfo.setIsQuantified(entity.getIsQuantified());
					addinfo.setQuantifiedValue(entity.getQuantifiedValue());

					if(!entity.getIsQuantified() && "JKF".equals(d.getOptionCode())){//加扣分
						if(StringUtils.isNumber(entity.getNonQuantifiedValue())){
							addinfo.setQuantifiedValue(Double.valueOf(entity.getNonQuantifiedValue()));
						}
					}

					if(!entity.getIsQuantified() && "JCF".equals(d.getOptionCode())){//基础分
						if(StringUtils.isNumber(entity.getNonQuantifiedValue())){
							addinfo.setQuantifiedValue(Double.valueOf(entity.getNonQuantifiedValue()));
						}
					}
					addinfo.setNonQuantifiedValue(entity.getNonQuantifiedValue());
					addinfo.setState("-1");//初始状态
					addinfo.setCreateTime(new Date());

					boolean b = super.saveOrUpdate(addinfo);
					if(!b){
						throw new CustomException("保存失败");
					}
				}

			}

		}
//废弃方法
//		OrganizationEvaluationInfo organizationEvaluationInfo = baseMapper.selectById(organizationEvaluationInfoList.get(0).getId());
//		//查询本次考评所有考评指标明细
//		List<OrganizationEvaluationInfo> organizationEvaluationInfos = baseMapper.selectByEvaluationInfoId(organizationEvaluationInfo.getOrganizationEvaluationId());
//
//		//调用需要计算指标项的方法
//		boolean calculate = calculateIdicatorItem(organizationEvaluationInfos);
//
//		//调用计算存储最大值，最小值的方法
//		if (calculate){
//			iCalculateIndicatorItemInfoService.saveIndicatorInfo(organizationEvaluationInfos);
//		}


		return true;
	}






	@Override
	public List<OrganizationEvaluationInfoResDTO> finishValueList(String organizationEvaluationId, String dept, String organizationShortName, String indicatorName, String binSection,String isQuantified, HttpServletRequest request) {
		List<OrganizationEvaluationInfoResDTO> list = baseMapper.selectListEvaluationInfoId2(organizationEvaluationId,dept,organizationShortName,indicatorName,binSection,isQuantified,"","");//默认获取指定的怎目标
		return list;
	}

	@Override
	public boolean calculationByEvaluationId(String id, String indicatorId,String organizationEvaluationId,String optionCode) {
		try {
			List<OrganizationEvaluationInfoResDTO> list = baseMapper.selectByIdAndEvaluationInfoId(id,indicatorId,organizationEvaluationId,"");
			//根据板块分组
			Map<String, List<OrganizationEvaluationInfoResDTO>> binSectionGropList = list.stream().collect(Collectors.groupingBy(OrganizationEvaluationInfoResDTO::getBinSection));

			//计算前置属性（差值、利润贡献、单位千瓦利润）的最大最小值
			for (Map.Entry<String, List<OrganizationEvaluationInfoResDTO>> binSectionGropEntry : binSectionGropList.entrySet()){

				List<OrganizationEvaluationInfo> listUpdateInfos = new ArrayList<>();
				if (binSectionGropEntry.getValue().size() > 0){
					//按照指标分组
					Map<String, List<OrganizationEvaluationInfoResDTO>> indicatorGropList = binSectionGropEntry.getValue().stream().collect(Collectors.groupingBy(OrganizationEvaluationInfoResDTO::getIndicatorId));
					for (Map.Entry<String, List<OrganizationEvaluationInfoResDTO>> indicatorGropListEntry : indicatorGropList.entrySet()) {

						if (indicatorGropListEntry.getValue().size() > 0) {
							//按照公司规则分组
							Map<String, List<OrganizationEvaluationInfoResDTO>> ruleIdGropList = indicatorGropListEntry.getValue().stream().collect(Collectors.groupingBy(OrganizationEvaluationInfoResDTO::getOrganizationEvaluationRuleId));

							//遍历公司指标
							for (Map.Entry<String, List<OrganizationEvaluationInfoResDTO>> ruleIdGropListEntry : ruleIdGropList.entrySet()) {
								if (ruleIdGropListEntry.getValue().size() > 0) {

									List<OrganizationEvaluationInfoResDTO> indicator = ruleIdGropListEntry.getValue();
									Map<String, List<OrganizationEvaluationInfoResDTO>> childCodeGropList = indicator.stream().collect(Collectors.groupingBy(OrganizationEvaluationInfoResDTO::getChildCode));

//									double totalScore = 0.00;
									for (Map.Entry<String, List<OrganizationEvaluationInfoResDTO>> childCodeGropListEntry : childCodeGropList.entrySet()) {
										for (OrganizationEvaluationInfoResDTO dto : childCodeGropListEntry.getValue()) {

											if (!(dto.getOptionCode().equals("LRZE") || dto.getOptionCode().equals("CZ") ||  dto.getOptionCode().equals("LRGXKH")|| dto.getOptionCode().equals("LRGXL") || dto.getOptionCode().equals("DWQWLR")) || StringUtils.isEmpty(dto.getFormula())) {
												System.out.println("前置跳过--|"+dto.getOptionCode());
												continue;//当前属性没有配置公式跳过
											}

											//获取去参与计算的属性
											List<String> optionCodes = getOptionCode(dto.getFormula());

											List<OrganizationEvaluationInfoResDTO> newInfoResInfo = childCodeGropListEntry.getValue().stream()
																									.filter(item -> optionCodes.contains(item.getOptionCode()))
																									.collect(Collectors.toList());

											//循环替换属性
											for (OrganizationEvaluationInfoResDTO f : newInfoResInfo) {
												dto.setFormula(dto.getFormula().replace("[" + f.getOptionCode() + "]", f.getQuantifiedValue() + ""));
											}

											double score = FormulaUtils.calculateFormula(dto.getFormula());
//											if (StringUtils.inStringIgnoreCase("DF", dto.getOptionCode())) {
//												totalScore = totalScore + score;
//											}
											log.info(dto.getOrganizationShortName() + "|" + dto.getChildName() + "|" + dto.getOptionCode() + "|" + dto.getFormula() + "=" + score);
											OrganizationEvaluationInfo info = new OrganizationEvaluationInfo();
											//将计算结果保存
											info.setId(dto.getId());
											info.setQuantifiedValue(score);
											info.setNonQuantifiedValue(score + "");
											dto.setQuantifiedValue(score);
											dto.setNonQuantifiedValue(score + "");

											listUpdateInfos.add(info);
//											this.saveOrUpdate(info);
										}
									}
								}
							}
						}
						//计算差值等的最大值最小值

						if(null != listUpdateInfos && listUpdateInfos.size()>0){
							this.saveOrUpdateBatch(listUpdateInfos);
						}

						iCalculateIndicatorItemInfoService.saveIndicatorInfoDTO(organizationEvaluationId,indicatorGropListEntry.getValue(),indicatorGropListEntry.getKey());
					}
				}
			}

			//获取计算的差值最大最小值
			List<CalculateIndicatorItemInfo> calculateIndicatorItemInfos = calculateIndicatorItemInfoMapper.selectList("","","","","",organizationEvaluationId,"");
			Map<String, List<CalculateIndicatorItemInfo>> groupedItemInfoMap = calculateIndicatorItemInfos.stream()
				.collect(Collectors.groupingBy(item -> item.getIndicatorId() + "_" + item.getChildCode()));

			//计算得分
			for (Map.Entry<String, List<OrganizationEvaluationInfoResDTO>> binSectionGropEntry : binSectionGropList.entrySet()){

				if (binSectionGropEntry.getValue().size() > 0){
					List<OrganizationEvaluationInfo> listUpdateInfos = new ArrayList<>();
					//按照指标分组
					Map<String, List<OrganizationEvaluationInfoResDTO>> indicatorGropList = binSectionGropEntry.getValue().stream().collect(Collectors.groupingBy(OrganizationEvaluationInfoResDTO::getIndicatorId));

					for (Map.Entry<String, List<OrganizationEvaluationInfoResDTO>> indicatorGropListEntry : indicatorGropList.entrySet()){
						for (OrganizationEvaluationInfoResDTO d : indicatorGropListEntry.getValue()){
							System.out.println(d.getOrganizationShortName()+"|"+d.getIndicatorName()+"|"+d.getOptionName()+"|"+d.getQuantifiedValue());
						}

						if (indicatorGropListEntry.getValue().size() > 0){
							//按照公司规则分组
							Map<String, List<OrganizationEvaluationInfoResDTO>> ruleIdGropList = indicatorGropListEntry.getValue().stream().collect(Collectors.groupingBy(OrganizationEvaluationInfoResDTO::getOrganizationEvaluationRuleId));


							//遍历公司指标
							for (Map.Entry<String, List<OrganizationEvaluationInfoResDTO>> ruleIdGropListEntry : ruleIdGropList.entrySet()){
								if (ruleIdGropListEntry.getValue().size() > 0){

									List<OrganizationEvaluationInfoResDTO> indicator = ruleIdGropListEntry.getValue();
									Map<String, List<OrganizationEvaluationInfoResDTO>> childCodeGropList = indicator.stream().collect(Collectors.groupingBy(OrganizationEvaluationInfoResDTO::getChildCode));
									double totalScore = 0.00;
									EvaluationScoreCount scoreCount = new EvaluationScoreCount();
									double standardScore = 0;
									for (Map.Entry<String, List<OrganizationEvaluationInfoResDTO>> childCodeGropListEntry : childCodeGropList.entrySet()){

//										List<CalculateIndicatorItemInfo> calculateIndicatorItemInfo = calculateIndicatorItemInfoMapper.selectList("","","","",childCodeGropListEntry.getValue().get(0).getIndicatorId(),childCodeGropListEntry.getValue().get(0).getOrganizationEvaluationId(),childCodeGropListEntry.getValue().get(0).getChildCode());
										List<CalculateIndicatorItemInfo> calculateIndicatorItemInfo = groupedItemInfoMap.get(childCodeGropListEntry.getValue().get(0).getIndicatorId()+"_"+childCodeGropListEntry.getValue().get(0).getChildCode());

										Map<String ,Double>  map = new HashMap<>();
										if (null != calculateIndicatorItemInfo && calculateIndicatorItemInfo.size() > 0) {
											for (CalculateIndicatorItemInfo itemInfo : calculateIndicatorItemInfo) {
												map.put(itemInfo.getOptionCode(), itemInfo.getQuantifiedValue());
											}
										}

										for (OrganizationEvaluationInfoResDTO dto: childCodeGropListEntry.getValue()){

											//保存加减分原因
											if("YY".equals(dto.getOptionCode()) || "JFYY".equals(dto.getOptionCode()) ||"KFYY".equals(dto.getOptionCode())){
												scoreCount.setJfRemark(("YY".equals(dto.getOptionCode()) || "JFYY".equals(dto.getOptionCode()))?dto.getNonQuantifiedValue():"");
												scoreCount.setKfRemark("KFYY".equals(dto.getOptionCode()) ? dto.getNonQuantifiedValue():"");
											}

											//保存加减分
											if("JF".equals(dto.getOptionCode())){
												scoreCount.setJfScore(dto.getQuantifiedValue());
											}
											if("KF".equals(dto.getOptionCode())){
												scoreCount.setKfScore(dto.getQuantifiedValue());
											}


											if ((dto.getOptionCode().endsWith("MAX") ||dto.getOptionCode().endsWith("MIN")
												||dto.getOptionCode().equals("CZ") || dto.getOptionCode().equals("LRGXL")
												|| dto.getOptionCode().equals("DWQWLR")) || StringUtils.isEmpty(dto.getFormula())){

												if(!(dto.getOptionCode().equals("DF")||dto.getOptionCode().equals("DF2")||dto.getOptionCode().equals("JHDF"))){
													continue;//当前属性没有配置公式跳过或者不需要参与此次计算的属性
												}

											}
											scoreCount.setOrganizationEvaluationId(dto.getOrganizationEvaluationId());
											scoreCount.setOrganizationEvaluationRuleId(dto.getOrganizationEvaluationRuleId());
											scoreCount.setIndicatorId(dto.getIndicatorId());
											scoreCount.setBinStage(dto.getBinStage());
											scoreCount.setBinSection(dto.getBinSection());

											//获取去参与计算的属性
											List<String> optionCodes = getOptionCode(dto.getFormula());
											List<OrganizationEvaluationInfoResDTO> newInfoResInfo = childCodeGropListEntry.getValue().stream()
													.filter(item -> optionCodes.contains(item.getOptionCode()))
													.collect(Collectors.toList());


											//替换属性值
											for (OrganizationEvaluationInfoResDTO f : newInfoResInfo){
												if(f.getOptionCode().endsWith("MIN") || f.getOptionCode().endsWith("MAX")){
													f.setQuantifiedValue(map.get(f.getOptionCode()));
												}
												dto.setFormula(dto.getFormula().replace("["+f.getOptionCode()+"]", f.getQuantifiedValue() + ""));
											}
											log.info("计算公式为："+dto.getFormula());
											double score = 0.00;
											if(StringUtils.isNotEmpty(dto.getFormula())){
												score = FormulaUtils.calculateFormula(dto.getFormula());
											}else {
												score = dto.getQuantifiedValue();
											}

											if(dto.getOptionCode().equals("DF")||dto.getOptionCode().equals("DF2")
												||dto.getOptionCode().equals("JHDF")){
												standardScore = dto.getStandardScore();
												totalScore = totalScore+score;
												scoreCount.setRemark(dto.getIndicatorName());
											}

											log.info(dto.getOrganizationShortName()+"|"+dto.getIndicatorName()+"|"+dto.getOptionCode()+"----------"+dto.getFormula()+"="+score);
											OrganizationEvaluationInfo info  = new OrganizationEvaluationInfo();
											//将计算结果保存
											info.setId(dto.getId());
											info.setQuantifiedValue(score);
											info.setNonQuantifiedValue(score+"");
											dto.setQuantifiedValue(score);
											dto.setNonQuantifiedValue(score+"");
											listUpdateInfos.add(info);
//											this.saveOrUpdate(info);
										}
									}

									log.info("综合得分："+totalScore);
									if(null==scoreCount.getOrganizationEvaluationId()|| null== scoreCount.getOrganizationEvaluationRuleId() || null == scoreCount.getIndicatorId()){
										continue;
									}
									if(standardScore>0 && totalScore != standardScore){
										scoreCount.setStandard(totalScore>standardScore?"1":"-1");
									}else {
										scoreCount.setStandard("0");
									}
									scoreCount.setScore(totalScore);
									scoreCount.setObversionScore(totalScore);
									scoreCount.setIsQuantified("1");

									List<EvaluationScoreCount>  listcount = evaluatioinScoreCountService.getEvaluationScoreCountList(scoreCount.getOrganizationEvaluationId(),scoreCount.getOrganizationEvaluationRuleId(),scoreCount.getIndicatorId(),scoreCount.getBinSection());
									if(null  != listcount && listcount.size() >0){
										List<String> ids = listcount.stream().map(EvaluationScoreCount::getId).collect(Collectors.toList());
										boolean cod = evaluatioinScoreCountService.removeByIds(ids);
									}
									evaluatioinScoreCountService.save(scoreCount);
								}
							}
						}
					}
					this.saveOrUpdateBatch(listUpdateInfos);
				}
			}



		} catch (Exception e) {
			e.printStackTrace();
			return false;
		}
		return true;
	}


	@Override
	public List<Map> getEvaluationIndicatorList(String organizationEvaluationId, String binSection, String binStage, HttpServletRequest request) {
		//判断是否是（重点专项和管理事项）
		List<Map> indicatorMap = new ArrayList<>();
//		if("ZDZX".equals(CacheContext.bseIdObject.get(binStage).getStageCode()) || "GLSX".equals(CacheContext.bseIdObject.get(binStage).getStageCode())){
//
//			OrganizationEvaluation organizationEvaluation = organizationEvaluationService.getById(organizationEvaluationId);
//			if(null == organizationEvaluation){
//				return null;
//			}
//			List<Indicator> indicatorList = indicatorService.listAll("","","",binSection,binStage,"","",organizationEvaluation.getCheckCycle());
//			if(null != indicatorList && indicatorList.size()>0){
//				for (Indicator i : indicatorList){
//					Map map = new HashMap();
//					map.put("id",i.getId());
//					map.put("indicator_name",i.getIndicatorName());
//					indicatorMap.add(map);
//				}
//			}
//
//		}else {
			indicatorMap = baseMapper.selectEvaluationIndicatorList(organizationEvaluationId,binSection,binStage);
//		}

		return indicatorMap;
	}

	@Override
	public Map getEvaluationInfoList(String organizationEvaluationId,String organizationShortName, String indicatorId, String binSection, String binStage, HttpServletRequest request) {

		Map data = new HashMap();

		//编辑标记
//		OrganizationEvaluation evaluation =  organizationEvaluationService.getById(organizationEvaluationId);
//		boolean editFlag = false;
//		if(StringUtils.isNotEmpty(evaluation.getInstId())){
//			editFlag = workflowService.todoTaskList2(evaluation.getInstId(),request);
//
//		}
//		data.put("editFlag",editFlag);



		//获取用户信息 23031009
		boolean deptFlag = false;
		try {
			MyuserResDTO user = userService.getSysUser(request);
			if(null != user){
//				String deptId = "23031009";
				Optional<DataDictionary> any = CacheContext.ddSuperKeyMap.get("BM0001").stream().filter(t -> t.getKeyName().equals(user.getDeptName())).findAny();
				if(any.isPresent()){
					deptFlag = 	"ZZRSB".equals(any.get().getDataKey()) ? true: false;
				}

			}
		} catch (Exception e) {
			e.printStackTrace();
		}

		boolean titlemark = true;//标记生成标题
		List<OrganizationEvaluationInfoResDTO> infoResDTOList = baseMapper.selectEvaluationInfoList(organizationEvaluationId,organizationShortName, indicatorId,binSection,binStage,"");
		List<IndicatorDictionary> dictionaryList = indicatorDictionaryService.list("", indicatorId, "", "");
		Indicator indicator = indicatorService.getById(indicatorId);
		//根据组织（公司）id分组
		Map<String, List<OrganizationEvaluationInfoResDTO>> groupedData = infoResDTOList.stream()
			.collect(Collectors.groupingBy(OrganizationEvaluationInfoResDTO::getOrganizationId));
		List<Map> mapList = new ArrayList<>();

		//设置数据标题
		//将子指标分组
		Map<String, List<IndicatorDictionary>> groupedChildCode = dictionaryList.stream()
			.collect(Collectors.groupingBy(d -> d.getChildCode() + "," + d.getChildName()));
		if (titlemark) {
			Map title = new LinkedHashMap();
			for (Map.Entry<String, List<IndicatorDictionary>> childCodeEntry : groupedChildCode.entrySet()) {
				List<Map> titleArray = new ArrayList<>();
				for (IndicatorDictionary d : childCodeEntry.getValue()) {
					Map titlemap = new LinkedHashMap();
					if (!d.getIsShow()) {
						continue;
					}
					titlemap.put("key", d.getOptionName());//名称
					titlemap.put("code",d.getChildCode()+"_"+d.getOptionCode());//名称
					titlemap.put("flag", d.getIsQuantified()?"1":"2");//是否量化
					titleArray.add(titlemap);

				}
				title.put(childCodeEntry.getKey().split(",")[1]+"-对标", titleArray);
			}
			if(groupedChildCode.size()>1 || dictionaryList.get(0).getChildCode().endsWith("LRZE") || dictionaryList.get(0).getChildCode().endsWith("JLR")){
				Map zhdfMap = new LinkedHashMap();
				List<Map> zhdfArray = new ArrayList<>();
				zhdfMap.put("key", "得分");//名称
				zhdfMap.put("code","totalScore");//名称
				zhdfMap.put("flag", "1");//数值
				zhdfArray.add(zhdfMap);
				title.put("综合得分", zhdfArray);
			}
			if(deptFlag){
				Map deptstateMap = new LinkedHashMap();
				List<Map> titlestateArray = new ArrayList<>();
				deptstateMap.put("key", "状态");//名称
				deptstateMap.put("code","state");//名称
				deptstateMap.put("flag", "3");//下拉
				titlestateArray.add(deptstateMap);
				title.put("填报状态", titlestateArray);
			}

			if ("PUBLIC".equals(indicator.getDeptId())) {

				Map operationMap = new LinkedHashMap();
				List<Map> operationArray = new ArrayList<>();
				operationMap.put("key", "操作");//名称
				operationMap.put("code", "operation");//名称
				operationMap.put("flag", "4");//下拉
				operationArray.add(operationMap);
				title.put("", operationArray);
			}
			titlemark = false;
			data.put("title", title);
		}


		//获取行数据
		//遍历按照公司分组后的数据
		for (Map.Entry<String, List<OrganizationEvaluationInfoResDTO>> entry : groupedData.entrySet()) {
			Map indicatormap = new HashMap();
			List<OrganizationEvaluationInfoResDTO> dtoList = entry.getValue();

			Map<String, OrganizationEvaluationInfoResDTO> resultMap = dtoList.stream()
				.collect(Collectors.toMap(
					dto -> dto.getChildCode() + "_" + dto.getOptionCode(),
					dto -> dto,
					(oldValue, newValue) -> oldValue)); // 解决键重复的情况
			boolean mark = true;//标记给公司名赋值

			double score = 0.00;
			double minScore = 0.00;
			double maxScore = 0.00;
			for (IndicatorDictionary d : dictionaryList) {
				if (!d.getIsShow()) {
					continue;
				}
				if (mark) {
					indicatormap.put("organizationShortName", resultMap.get(d.getChildCode()+"_"+d.getOptionCode()).getOrganizationShortName());//公司名
					indicatormap.put("organizationId", resultMap.get(d.getChildCode()+"_"+d.getOptionCode()).getOrganizationId());//公司id
					indicatormap.put("state",resultMap.get(d.getChildCode()+"_"+d.getOptionCode()).getState());
					indicatormap.put("operation","PUBLIC".equals(resultMap.get(d.getChildCode()+"_"+d.getOptionCode()).getDeptId()));
					indicatormap.put("IS_LH_state","3");

					mark = false;
				}
				//指标属性
				indicatormap.put(d.getChildCode() + "_" + d.getOptionCode(), null == resultMap.get(d.getChildCode()+"_"+d.getOptionCode()).getNonQuantifiedValue() ? "" : resultMap.get(d.getChildCode()+"_"+d.getOptionCode()).getNonQuantifiedValue());
				//指标id
				indicatormap.put("ID_" + d.getChildCode() + "_" + d.getOptionCode(), resultMap.get(d.getChildCode()+"_"+d.getOptionCode()).getId());
				//是否量化
				indicatormap.put("IS_LH_" +d.getChildCode() + "_" + d.getOptionCode(),resultMap.get(d.getChildCode()+"_"+d.getOptionCode()).getIsQuantified2()?"1":"2");

				//累计分数
				if(d.getOptionCode().equals("DF")||d.getOptionCode().equals("DF2")||d.getOptionCode().equals("JHDF")){
					score = score + resultMap.get(d.getChildCode()+"_"+d.getOptionCode()).getQuantifiedValue();

					//判断最大最小值 ；是否存在值
					if(null != resultMap.get(d.getChildCode()+"_"+d.getOptionCode()).getMinScore()){
						minScore =resultMap.get(d.getChildCode()+"_"+d.getOptionCode()).getMinScore();
					}
					if(null != resultMap.get(d.getChildCode()+"_"+d.getOptionCode()).getMaxScore()){
						maxScore =resultMap.get(d.getChildCode()+"_"+d.getOptionCode()).getMaxScore();
					}

				}

			}

			indicatormap.put("totalScore",score);

			if(score!=0 && (minScore!=0 || maxScore!=0)){
				boolean valueFlag =  DoubleUtils.isIncludeValue(score,minScore,maxScore);
				indicatormap.put("valueFlag",valueFlag);
			}else {
				indicatormap.put("valueFlag",true);
			}
			mapList.add(indicatormap);
		}
		data.put("value", mapList);
		return data;
	}

	@Override
	public byte[] downloadExcel(String organizationEvaluationId, String indicatorId, String binSection, String binStage,HttpServletRequest request) throws Exception{

		//获取用户信息 23031009
		boolean deptFlag = false;
		MyuserResDTO user = userService.getSysUser(request);
		if(null != user){
			String deptId = "23031009";
			if(deptId.equals(user.getDeptId())){
				deptFlag = true;
			}
		}
		String[] values = new String[]{"不合格", "待确认", "已确认"};

		Map<String ,List<Map<String,String>>> title = new LinkedHashMap();//标题
		List<Map<String,String>> mapList = new ArrayList<>();//数据list
		boolean titlemark = true;//标记生成标题
		List<OrganizationEvaluationInfoResDTO> infoResDTOList = baseMapper.selectEvaluationInfoList(organizationEvaluationId, "",indicatorId,binSection,binStage,"");
		List<IndicatorDictionary> dictionaryList = indicatorDictionaryService.list("", indicatorId, "", "");

		//根据组织（公司）id分组
		Map<String, List<OrganizationEvaluationInfoResDTO>> groupedData = infoResDTOList.stream()
			.collect(Collectors.groupingBy(OrganizationEvaluationInfoResDTO::getOrganizationId));

		//遍历按照公司分组后的数据
		for (Map.Entry<String, List<OrganizationEvaluationInfoResDTO>> entry : groupedData.entrySet()) {

			Map indicatormap = new LinkedHashMap();
			List<OrganizationEvaluationInfoResDTO> dtoList = entry.getValue();

			Map<String, OrganizationEvaluationInfoResDTO> resultMap = dtoList.stream()
				.collect(Collectors.toMap(
					dto -> dto.getChildCode() + "_" + dto.getOptionCode(),
					dto -> dto,
					(oldValue, newValue) -> oldValue)); // 解决键重复的情况
			boolean mark = true;//标记给公司名赋值

			//将子指标分组
			Map<String, List<IndicatorDictionary>> groupedChildCode = dictionaryList.stream()
				.collect(Collectors.groupingBy(d -> d.getChildCode() + "," + d.getChildName()));

			if (titlemark) {
				for (Map.Entry<String, List<IndicatorDictionary>> childCodeEntry : groupedChildCode.entrySet()) {


					List<Map<String,String>> titleArray = new ArrayList<>();
					Map titlemap = new LinkedHashMap();
					for (IndicatorDictionary d : childCodeEntry.getValue()) {
						if (!d.getIsShow()) {
							continue;
						}
						titlemap.put(d.getChildCode()+"_"+d.getOptionCode(), d.getOptionName());//名称
					}
					titleArray.add(titlemap);
					title.put(childCodeEntry.getKey().split(",")[1], titleArray);

				}
				//状态标题头
				List<Map<String,String>> titleStateArray = new ArrayList<>();
				Map titleStateMap = new LinkedHashMap();
				titleStateMap.put("state","填报状态");
				titleStateArray.add(titleStateMap);
				title.put("状态",titleStateArray);

				titlemark = false;
			}

			String state = "不合格";
			for (Map.Entry<String, List<IndicatorDictionary>> childCodeEntry : groupedChildCode.entrySet()) {

				for (IndicatorDictionary d : childCodeEntry.getValue()) {
					if (!d.getIsShow()) {
						continue;
					}
					if (mark) {
						indicatormap.put("organizationShortName", resultMap.get(d.getChildCode() + "_" + d.getOptionCode()).getOrganizationShortName());//公司名
						indicatormap.put("organizationId", resultMap.get(d.getChildCode() + "_" + d.getOptionCode()).getOrganizationId());//公司编号

						String  state2 = resultMap.get(d.getChildCode()+"_"+d.getOptionCode()).getState();//(-1:不合格；0:待确认；1:已确认)
						state = stateConvert(state2,true);
						mark = false;
					}
					//指标属性
					indicatormap.put(d.getChildCode() + "_" + d.getOptionCode(), null == resultMap.get(d.getChildCode() + "_" + d.getOptionCode()).getNonQuantifiedValue() ? "" : resultMap.get(d.getChildCode() + "_" + d.getOptionCode()).getNonQuantifiedValue());
				}
			}
			indicatormap.put("state",state);//状态
			mapList.add(indicatormap);
		}


		// 创建Workbook对象
		Workbook workbook = new XSSFWorkbook();

		// 创建工作表
		Sheet sheet = workbook.createSheet(infoResDTOList.get(0).getIndicatorName());

		// 创建单元格样式
		CellStyle cellStyle = workbook.createCellStyle();
		cellStyle.setAlignment(HorizontalAlignment.CENTER);
		cellStyle.setVerticalAlignment(VerticalAlignment.CENTER);

		// 设置边框样式
		cellStyle.setBorderTop(BorderStyle.THIN);
		cellStyle.setBorderBottom(BorderStyle.THIN);
		cellStyle.setBorderLeft(BorderStyle.THIN);
		cellStyle.setBorderRight(BorderStyle.THIN);

		// 创建1、2行作为标题行
		Row headerRow0 = sheet.createRow(0);
		Row headerRow1 = sheet.createRow(1);
		Row headerRow2 = sheet.createRow(2);
		Row headerRow3 = sheet.createRow(3);


		// 设置首行的单元格值
		Cell header0Cell0 = headerRow0.createCell(0);
		header0Cell0.setCellValue(infoResDTOList.get(0).getIndicatorName());


		Cell headerCell0 = headerRow1.createCell(0);
		headerCell0.setCellValue("序号");
		headerCell0.setCellStyle(cellStyle);
		headerCell0.setCellStyle(cellStyle);
		sheet.addMergedRegion(new CellRangeAddress(1, 2, 0, 0));

		Cell headerCell1 = headerRow1.createCell(1);
		headerCell1.setCellValue("单位名称");
		headerCell1.setCellStyle(cellStyle);
		headerCell1.setCellStyle(cellStyle);
		sheet.addMergedRegion(new CellRangeAddress(1, 2, 1, 1));

		Cell headerCell2 = headerRow1.createCell(2);
		headerCell2.setCellValue("单位编码");
		headerCell2.setCellStyle(cellStyle);
		headerCell2.setCellStyle(cellStyle);
		sheet.addMergedRegion(new CellRangeAddress(1, 2, 2, 2));

		Cell header3Cell0 = headerRow3.createCell(0);
		header3Cell0.setCellValue("zhidmap,evalId="+organizationEvaluationId+",indId="+indicatorId);

		Cell header3Cell1 = headerRow3.createCell(1);
		header3Cell1.setCellValue("organizationShortName");

		Cell header3Cell2 = headerRow3.createCell(2);
		header3Cell2.setCellValue("organizationId");


		Integer colsize = 2;
		Integer optioncols = 3;
		for (String  key : title.keySet()) {//存在多个标题
			List<Map<String,String>> titleArray2 = title.get(key);
			Cell headerCellcolsize = headerRow1.createCell(colsize+1);
			headerCellcolsize.setCellValue(key);
			headerCellcolsize.setCellStyle(cellStyle);
			headerCellcolsize.setCellStyle(cellStyle);

			System.out.println("key="+key);

			for (Map<String,String>  key2 : titleArray2) {
				if(key2.size()>1){
					sheet.addMergedRegion(new CellRangeAddress(1, 1, colsize+1, colsize + key2.size()));
				}
				colsize = colsize + key2.size();
				for (String  key3 : key2.keySet()) {

					Cell header3Cell3 = headerRow3.createCell(optioncols);
					header3Cell3.setCellValue(key3);
					header3Cell3.setCellStyle(cellStyle);

					Cell header2Cell2 = headerRow2.createCell(optioncols);
					header2Cell2.setCellValue(key2.get(key3));
					header2Cell2.setCellStyle(cellStyle);

					// 隐藏列
					if(key3.startsWith("ID_") || key3.startsWith("IS_LH_")){
						sheet.setColumnHidden(optioncols, true);
					}
					optioncols++;
				}
			}
		}

		Integer rowdate = 4;
		for (Map<String,String >  datamaps  : mapList) {
			Integer colsdata = 1;
			Row headerRowdate = sheet.createRow(rowdate);
			Cell dataCell0 = headerRowdate.createCell(0);
			dataCell0.setCellValue(rowdate-3);
			dataCell0.setCellStyle(cellStyle);
			for (String datakey: datamaps.keySet()) {

				Cell dataCell1 = headerRowdate.createCell(colsdata);
				dataCell1.setCellValue(datamaps.get(datakey).toString());
				dataCell1.setCellStyle(cellStyle);

				if("state".equals(datakey)){
					// 创建一个数据验证对象
					DataValidationHelper validationHelper = sheet.getDataValidationHelper();
					DataValidationConstraint constraint = validationHelper.createExplicitListConstraint(values);
					// 设置下拉列表的位置范围为
					CellRangeAddressList addressList = new CellRangeAddressList(rowdate, rowdate, colsdata, colsdata); // 行索引和列索引从0开始计数
					// 创建数据验证对象
					DataValidation validation = validationHelper.createValidation(constraint, addressList);

					// 将数据验证对象应用于单元格
					sheet.addValidationData(validation);
				}

				colsdata++;
			}
			rowdate++;
		}

		//隐藏行
		sheet.getRow(3).setZeroHeight(true);
		sheet.addMergedRegion(new CellRangeAddress(0, 0, 0, colsize));
		header0Cell0.setCellStyle(cellStyle);

		// 将Workb
		// ook写入字节数组输出流
		ByteArrayOutputStream outputStream = new ByteArrayOutputStream();
		workbook.write(outputStream);

		// 将字节数组作为响应体返回，并设置响应头
		byte[] excelBytes = outputStream.toByteArray();
		return excelBytes;
	}



	@Override
	public boolean importExcel(MultipartFile file,HttpServletRequest request) throws IOException {

		List<LinkedHashMap<String, Object>> dataList = new ArrayList<>();
		try {
			//获取原始的文件名
			String originalFilename = file.getOriginalFilename();
			//获取文件类型
			String fileType = originalFilename.substring(originalFilename.lastIndexOf(".") + 1, originalFilename.length());
			//获取输入流
			InputStream is = file.getInputStream();
			Workbook workbook = readExcel(fileType, is);

			Sheet sheet = workbook.getSheetAt(0);

			// 获取总列数和起始行索引
			int totalColumns = sheet.getRow(3).getLastCellNum();
			int startingRowIndex = 4;

			Row headerRow = sheet.getRow(3);

			// 遍历每一行数据
			for (int i = startingRowIndex; i <= sheet.getLastRowNum(); i++) {
				Row row = sheet.getRow(i);
				LinkedHashMap<String, Object> dataMap = new LinkedHashMap<>();

				// 遍历每一列数据
				for (int j = 0; j < totalColumns; j++) {
					Cell headerCell = headerRow.getCell(j);
					Cell cell = row.getCell(j);

					String columnName = headerCell.getStringCellValue();
					String cellValue = "";

					// 根据单元格类型读取对应内容
					if (cell != null) {
						switch (cell.getCellType()) {
							case HSSFCell.CELL_TYPE_STRING:
								cellValue = cell.getStringCellValue();
								break;
							case HSSFCell.CELL_TYPE_NUMERIC:
								cellValue = String.valueOf(cell.getNumericCellValue());
								break;
							case HSSFCell.CELL_TYPE_BOOLEAN:
								cellValue = String.valueOf(cell.getBooleanCellValue());
								break;
							case HSSFCell.CELL_TYPE_BLANK:
								cellValue = "";
								break;
						}
					}

					dataMap.put(columnName, cellValue);
				}

				dataList.add(dataMap);
			}
			workbook.close();
			for(Map<String, Object> map : dataList){
				Map<String,String > idMap =  new HashMap();
				Map<String,String > valueMap =  new HashMap();
				for (String key : map.keySet()) {
					String value = String.valueOf(map.get(key));
					if(key.startsWith("zhidmap")){
						String[] zhidmap = key.split(",");
						idMap.put("evalId",zhidmap[1].split("=")[1]);
						idMap.put("indId", zhidmap[2].split("=")[1]);
						continue;
					}if(key.startsWith("organization")){
						idMap.put(key,value);
						continue;
					}if(key.startsWith("state")){
						idMap.put(key,value);
						continue;
					}else {
						valueMap.put(key,value);
						continue;
					}
				}
				List<OrganizationEvaluationInfo> evaluationInfos =new ArrayList<>();
				for (Map.Entry<String,String > entry : valueMap.entrySet()) {
					try {
						//organizationEvaluationId 考评记录id
						// indicatorId 指标id
						// organizationId 组织id
						// childOptionCode 组合子指标和指标项
						List<OrganizationEvaluationInfo>  organizationEvaluationInfoList = 	getEvaluationInfoByOptionCodeList(idMap.get("evalId"),idMap.get("indId"),idMap.get("organizationId"),entry.getKey());
						OrganizationEvaluationInfo info = new OrganizationEvaluationInfo();
						OrganizationEvaluationInfo oriinfo = null;
						if(null !=organizationEvaluationInfoList &&  organizationEvaluationInfoList.size()>0){
							 oriinfo = organizationEvaluationInfoList.get(0);//原始数据
							info.setId(oriinfo.getId());
						} else {
							continue;
						}
						if (oriinfo.getIsQuantified()) {

							if (MathCalculatorUtil.isNumber(String.valueOf(entry.getValue()))) {
								info.setQuantifiedValue(Double.valueOf(entry.getValue()));
							} else {
								info.setQuantifiedValue(0);
							}
						}
						info.setNonQuantifiedValue(entry.getValue());
						String  state2 = idMap.get("state").toString();
						String state = stateConvert(state2,false);
						info.setState(state);
						evaluationInfos.add(info);
					} catch (NumberFormatException e) {
						e.printStackTrace();
						log.info("当前行出现异常；继续执行后面的记录");
					}
				}
				this.saveOrUpdateBatch(evaluationInfos);
			}
		} catch (IOException e) {
			e.printStackTrace();
			return false;
		}
		return true;
	}

	@Transactional
	@Override
	public boolean updateEvaluationInfo(List<JSONObject> jsonObjects) {
		for (JSONObject json :jsonObjects){

			Map<String,Object > idMap = new HashMap();
			Map valueMap =  new HashMap();
			for (String key : json.keySet()) {
				Object value = json.get(key);
				if(key.startsWith("ID_")){
					idMap.put(key,value);
				}else {
					valueMap.put(key,value);
				}
			}

			List<OrganizationEvaluationInfo> evaluationInfos =new ArrayList<>();
			//总分 加扣分
			double score = 0.00;
			double jfScore = 0.00;
			double kfScore = 0.00;

			//加扣分原因
			String  jfRemark ="";
			String  kfRemark ="";

			String scoreid ="";

			for (Map.Entry<String,Object > entry : idMap.entrySet()) {
				OrganizationEvaluationInfo info = this.getById(entry.getValue().toString());
				info.setId(entry.getValue().toString());
				String quantified = valueMap.get(entry.getKey().replace("ID_","IS_LH_")).toString();
				scoreid = entry.getValue().toString();
				if(null != valueMap.get(entry.getKey().replace("ID_", ""))) {

//					//加分 扣分 加扣分原因
					if (entry.getKey().endsWith("_JFYY")) {
						jfRemark = valueMap.get(entry.getKey().replace("ID_", "")).toString();
					}

					if (entry.getKey().endsWith("_KFYY")) {
						kfRemark = valueMap.get(entry.getKey().replace("ID_", "")).toString();
					}


					if ("1".equals(quantified)) {

						if (MathCalculatorUtil.isNumber(String.valueOf(valueMap.get(entry.getKey().replace("ID_", "")).toString()))) {
							info.setQuantifiedValue(Double.valueOf(valueMap.get(entry.getKey().replace("ID_", "")).toString()));

							//得分
							if (entry.getKey().endsWith("_DF") || entry.getKey().endsWith("_DF2") || entry.getKey().endsWith("_JHDF")) {
								score = score + info.getQuantifiedValue();

							}

							//加分 扣分
							if (entry.getKey().endsWith("_JF")) {
								jfScore = Double.valueOf(valueMap.get(entry.getKey().replace("ID_", "")).toString());
							}
							if (entry.getKey().endsWith("_KF")) {
								kfScore = Double.valueOf(valueMap.get(entry.getKey().replace("ID_", "")).toString());
							}

						} else {
							info.setQuantifiedValue(0);
						}
					}

					info.setNonQuantifiedValue(valueMap.get(entry.getKey().replace("ID_", "")).toString());
				}else {
					info.setQuantifiedValue(0);
					info.setNonQuantifiedValue("");
				}
				info.setState(valueMap.get("state").toString());//
				evaluationInfos.add(info);
			}

			if(null == evaluationInfos || evaluationInfos.size()<=0){
				return false;
			}
			boolean b = this.saveOrUpdateBatch(evaluationInfos);
			if(b && null !=evaluationInfos && evaluationInfos.size()>0){
				OrganizationEvaluationInfo info = this.getById(scoreid);
				List<EvaluationScoreCount>  listcount = evaluatioinScoreCountService.getEvaluationScoreCountList(info.getOrganizationEvaluationId(),info.getOrganizationEvaluationRuleId(),info.getIndicatorId(),"");
				if(null != listcount && listcount.size()>0){
					EvaluationScoreCount count = listcount.get(0);
					count.setJfScore(jfScore);
					count.setKfScore(kfScore);
					count.setJfRemark(jfRemark);
					count.setKfRemark(kfRemark);
					count.setScore(score);
					count.setObversionScore(score);
					evaluatioinScoreCountService.saveOrUpdate(count);
				}else {
					calculationByEvaluationId("", info.getIndicatorId(),info.getOrganizationEvaluationId(),"");
				}
			}
		}
		return true;
	}

	@Transactional
	@Override
	public boolean addEvaluationInfo(List<AddEvaluationInfoDTO> addEvaluationInfoDTOs) {

		for (AddEvaluationInfoDTO addEvaluationInfoDTO : addEvaluationInfoDTOs) {

//			List<OrganizationEvaluationInfoResDTO> infoResDTOList = baseMapper.selectEvaluationInfoList(addEvaluationInfoDTO.getOrganizationEvaluationId(),"", addEvaluationInfoDTO.getIndicatorId(),addEvaluationInfoDTO.getBinSection(),addEvaluationInfoDTO.getBinStage(),addEvaluationInfoDTO.getOrganizationId());
//
//			if(null != infoResDTOList && infoResDTOList.size()>0){
//				throw new CustomException("该组织已存在当前指标");
//			}

			//1.获取考评id(获取此考评记录)
			String organizationEvaluationId = addEvaluationInfoDTO.getOrganizationEvaluationId();

			if (StringUtils.isEmpty(organizationEvaluationId)) {
				throw new CustomException("保存失败");
			}
			OrganizationEvaluation organizationEvaluation = organizationEvaluationService.getById(organizationEvaluationId);

			if (null == organizationEvaluation) {
				throw new CustomException("保存失败");
			}
			organizationEvaluation.getCheckCycle();

			//2.获取组织id 和所属板块（判断该板块下是否有此组织的考核）
			String organizationType = organizationEvaluation.getOrganizationType();//考评类型 DWKP
			String checkCycle = organizationEvaluation.getCheckCycle();//考评周期-月度、季度
			String year = organizationEvaluation.getYear();//年份

			//3.查询当前对应的考评权重（需要考评的单位）
			List<OrganizationEvaluationRule> evaluationRules = organizationEvaluationRuleService.getOrganizationEvaluationRuleByYearAndCycle(organizationType, checkCycle, year, addEvaluationInfoDTO.getOrganizationId());
			if (null == evaluationRules || evaluationRules.size() <= 0) {
				throw new CustomException("当前组织没有参与考评");
			}
			OrganizationEvaluationRule evaluationRule = evaluationRules.get(0);

			//4.获取指标信息
			List<String> indicatorIds = Arrays.asList(addEvaluationInfoDTO.getIndicatorId().split(","));
			List<IndicatorResDTO> dictionaryList = indicatorService.listByIds(indicatorIds);
			Map<String, List<IndicatorResDTO>> groupedChildCode = dictionaryList.stream()
				.collect(Collectors.groupingBy(d -> d.getChildCode() + "," + d.getChildName()));
			Map<String, Object> optionMap = addEvaluationInfoDTO.getOptionMap();

			//5.保存指标
			for (Map.Entry<String, List<IndicatorResDTO>> childCodeEntry : groupedChildCode.entrySet()) {

				for (IndicatorResDTO d : childCodeEntry.getValue()) {
					OrganizationEvaluationInfo info = new OrganizationEvaluationInfo();
					info.setOrganizationEvaluationId(addEvaluationInfoDTO.getOrganizationEvaluationId());
					info.setOrganizationEvaluationRuleId(evaluationRule.getId());
					info.setOrganizationId(evaluationRule.getOrganizationId());
					info.setIndicatorId(addEvaluationInfoDTO.getIndicatorId());
					info.setIndicatorDictionaryId(d.getId());
					info.setOptionCode(d.getOptionCode());
					info.setIsQuantified(d.getIsQuantified());
					if (null != optionMap.get(d.getChildCode() + "_" + d.getOptionCode())) {
						Object value = optionMap.get(d.getChildCode() + "_" +d.getOptionCode());
						if (null != value) {
							info.setNonQuantifiedValue(String.valueOf(value));
							if (d.getIsQuantified()) {

								if (MathCalculatorUtil.isNumber(value.toString())) {
									info.setQuantifiedValue(Double.valueOf(value.toString()));
								} else {
									info.setQuantifiedValue(0);
								}
							}
						}
					}
					info.setChildCode(d.getChildCode());
					info.setDeptId(d.getDeptId());
					info.setCreateTime(new Date());
					saveOrUpdate(info);
				}
			}
		}

		return true;
	}

	//todo  后续添加需要计算的指标项
	public boolean calculateIdicatorItem(List<OrganizationEvaluationInfo> organizationEvaluationInfos){


		//根据考评规则分组
		Map<String, List<OrganizationEvaluationInfo>> collect1 = organizationEvaluationInfos.stream().collect(Collectors.groupingBy(OrganizationEvaluationInfo::getOrganizationEvaluationRuleId));
		for (Map.Entry<String, List<OrganizationEvaluationInfo>> entry1 : collect1.entrySet()){
			//根据指标id分组
			Map<String, List<OrganizationEvaluationInfo>> collect = entry1.getValue().stream().collect(Collectors.groupingBy(OrganizationEvaluationInfo::getIndicatorId));
			for(Map.Entry<String, List<OrganizationEvaluationInfo>> entry : collect.entrySet()){
				//过滤差值考评指标明细
				List<OrganizationEvaluationInfo> cz = entry.getValue().stream().filter(r -> r.getOptionCode().equals("CZ")).collect(Collectors.toList());
				if (cz.size() == 1){
					List<OrganizationEvaluationInfo> wcz = entry.getValue().stream().filter(r -> r.getOptionCode().equals("WCZ")).collect(Collectors.toList());
					List<OrganizationEvaluationInfo> value = entry.getValue().stream().filter(r -> r.getOptionCode().equals("JHZ") || r.getOptionCode().equals("DBZ") || r.getOptionCode().equals("MBZ")).collect(Collectors.toList());

					//完成值与目标值 ||计划值||对标值进行计算
					if (wcz.size() == 1 && value.size() == 1){
						double wczValue = wcz.get(0).getQuantifiedValue();
						double dbzvalue = value.get(0).getQuantifiedValue();
						cz.get(0).setQuantifiedValue(wczValue - dbzvalue);
						cz.get(0).setCreateTime(new Date());
						boolean b = super.saveOrUpdate(cz.get(0));
						if(!b){
							throw new CustomException("更新失败");
						}
					}
				}

				//过滤单位千瓦利润考评指标明细
				List<OrganizationEvaluationInfo> dwqwlr = entry.getValue().stream().filter(r -> r.getOptionCode().equals("DWQWLR")).collect(Collectors.toList());
				if (dwqwlr.size() == 1){
					List<OrganizationEvaluationInfo> lrgxl = entry.getValue().stream().filter(r -> r.getOptionCode().equals("LRGXL")).collect(Collectors.toList());
					List<OrganizationEvaluationInfo> zjrl = entry.getValue().stream().filter(r -> r.getOptionCode().equals("ZJRL")).collect(Collectors.toList());

					//完成值与目标值 ||计划值||对标值进行计算
					if (lrgxl.size() == 1 && zjrl.size() == 1){
						double wczValue = lrgxl.get(0).getQuantifiedValue();
						double dbzvalue = zjrl.get(0).getQuantifiedValue();
						if (dbzvalue == 0 || dbzvalue == 0.0){
							dwqwlr.get(0).setQuantifiedValue(0);
							dwqwlr.get(0).setCreateTime(new Date());
							boolean b = super.saveOrUpdate(dwqwlr.get(0));
							if(!b){
								throw new CustomException("更新失败");
							}
						}else{
							//double保留两位小数
							double v = new BigDecimal(wczValue / dbzvalue).setScale(2, BigDecimal.ROUND_HALF_UP).doubleValue();
							dwqwlr.get(0).setQuantifiedValue(v);
							dwqwlr.get(0).setCreateTime(new Date());
							boolean b = super.saveOrUpdate(dwqwlr.get(0));
							if(!b){
								throw new CustomException("更新失败");
							}
						}
					}
				}

			}
		}
		return true;
	}


	@Override
	public List<OrganizationEvaluationInfo> getEvaluationInfoByOptionCodeList(String organizationEvaluationId, String indicatorId, String organizationId, String childOptionCode) {

		QueryWrapper<OrganizationEvaluationInfo> qw = new QueryWrapper<>();

		if (StringUtils.isNotEmpty(organizationEvaluationId)) {
			qw.lambda().eq(OrganizationEvaluationInfo::getOrganizationEvaluationId, organizationEvaluationId);
		}

		if (StringUtils.isNotEmpty(indicatorId)) {
			qw.lambda().eq(OrganizationEvaluationInfo::getIndicatorId, indicatorId);
		}

		if (StringUtils.isNotEmpty(organizationId)) {
			qw.lambda().eq(OrganizationEvaluationInfo::getOrganizationId, organizationId);
		}
		if (StringUtils.isNotEmpty(childOptionCode)) {
			qw.eq("CONCAT(child_code, '_', option_code)", childOptionCode);
		}
		List<OrganizationEvaluationInfo> list = baseMapper.selectList(qw);
		return list;
	}

	@Override
	public int getEvaluationInfoByStateCount(String organizationEvaluationId, String deptId, List<String> state) {
		QueryWrapper<OrganizationEvaluationInfo> qw = new QueryWrapper<>();

		if (StringUtils.isNotEmpty(organizationEvaluationId)) {
			qw.lambda().eq(OrganizationEvaluationInfo::getOrganizationEvaluationId, organizationEvaluationId);
		}

		if (StringUtils.isNotEmpty(deptId)) {
			qw.lambda().eq(OrganizationEvaluationInfo::getDeptId, deptId);
		}

		if (StringUtils.isNotEmpty(state)) {
			qw.lambda().in(OrganizationEvaluationInfo::getState, state);
		}

//		Integer count = baseMapper.selectCount(qw);
		Integer count = baseMapper.selectEvaluationInfoByStateCount(organizationEvaluationId,deptId,state);
		return count;

	}

	@Override
	public List<OrganizationEvaluationInfo> getConvergeCommonInfo(String organizationEvaluationId, String organizationId, String indicatorId) {
		QueryWrapper<OrganizationEvaluationInfo> qw = new QueryWrapper<>();

		if (StringUtils.isNotEmpty(organizationEvaluationId)) {
			qw.lambda().eq(OrganizationEvaluationInfo::getOrganizationEvaluationId, organizationEvaluationId);
		}
		if (StringUtils.isNotEmpty(organizationId)) {
			qw.lambda().eq(OrganizationEvaluationInfo::getOrganizationId, organizationId);
		}
		if (StringUtils.isNotEmpty(indicatorId)) {
			qw.lambda().eq(OrganizationEvaluationInfo::getIndicatorId, indicatorId);
		}
		List<OrganizationEvaluationInfo> list = baseMapper.selectList(qw);
		return list;
	}

	/**
	 * 状态抓换 (-1:不合格；0:待确认；1:已确认)
	 * @return
	 */
	private String stateConvert(String state,boolean flge) {
//		(-1:不合格；0:待确认；1:已确认)
		String state2 = "";
		if (flge) {
			if(StringUtils.isEmpty(state)){
				state2 = "待确认";
				return state2;
			}

			switch (state) {
				case "-1":
					state2 = "不合格";
					break;
				case "0":
					state2 = "待确认";
					break;
				case "1":
					state2 = "已确认";
					break;
				default:
					state2 = "待确认";
					break;
			}

		} else {
			if(StringUtils.isEmpty(state)){
				state2 = "0";
				return state2;
			}
			switch (state) {
				case "不合格":
					state2 = "-1";
					break;
				case "待确认":
					state2 = "0";
					break;
				case "已确认":
					state2 = "1";
					break;
				default:
					state2 = "0";
					break;
			}
		}
		return state2;
	}


	private List<String> getOptionCode(String formula) {
		List<String> setOptionCode =  new ArrayList<>();
		if (StringUtils.isEmpty(formula)) {
			return setOptionCode;
		}
		List<String> optionCode = new ArrayList<>();
		String pattern = "\\[(.*?)\\]";
		Pattern regex = Pattern.compile(pattern);
		Matcher matcher = regex.matcher(formula);
		while (matcher.find()) {
			String match = matcher.group(1);
			optionCode.add(match);
		}
		Set<String> uniqueOptionCode = new HashSet<>(optionCode);
		 setOptionCode = new ArrayList<>(uniqueOptionCode);
		return setOptionCode;
	}


}
