package com.ims.eval.service.impl;

import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.ims.common.utils.StringUtils;
import com.ims.eval.config.CustomException;
import com.ims.eval.entity.DeptResponsibility;
import com.ims.eval.entity.OrganizationYearRating;
import com.ims.eval.dao.OrganizationYearRatingMapper;
import com.ims.eval.entity.dto.result.R;
import com.ims.eval.service.IOrganizationYearRatingService;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import org.springframework.stereotype.Service;

/**
 * <p>
 *  服务实现类
 * </p>
 *
 * @author wang
 * @since 2023-09-15
 */
@Service
public class OrganizationYearRatingServiceImpl extends ServiceImpl<OrganizationYearRatingMapper, OrganizationYearRating> implements IOrganizationYearRatingService {

	@Override
	public IPage<OrganizationYearRating> list(Integer pageNum, Integer pageSize, String yearRatingCode, String year, String des) {

		if (null == pageNum || null == pageSize) {
			throw new CustomException("分页参数为空");
		}
		QueryWrapper<OrganizationYearRating> qw = new QueryWrapper<>();
		if (StringUtils.isNotEmpty(yearRatingCode)) {
			qw.lambda().like(OrganizationYearRating::getYearRatingCode,yearRatingCode);
		}

		if (StringUtils.isNotEmpty(year)) {
			qw.lambda().eq(OrganizationYearRating::getYear,year);
		}

		if (StringUtils.isNotEmpty(des)) {
			qw.lambda().eq(OrganizationYearRating::getDes,des);
		}

		Page<OrganizationYearRating> page = new Page<>(pageNum, pageSize);
		IPage<OrganizationYearRating> list = baseMapper.selectPage(page, qw);
		return list;
	}

	@Override
	public boolean editState(String id, String state, String instId, String destination) {
		OrganizationYearRating yearRating = baseMapper.selectById(id);
		if (StringUtils.isNotEmpty(state)) {
			if ("评审委员会审核".equals(state)) {
				state = "流程已结束";
			}
			yearRating.setStage(state);
		}
		if (StringUtils.isNotEmpty(instId)) {
			yearRating.setInstId(instId);
			yearRating.setStage("流程已启动");
		}
		boolean b = super.saveOrUpdate(yearRating);
		return b;
	}


	@Override
	public OrganizationYearRating getOneYearRating(String yearRatingCode, String year) {

		QueryWrapper<OrganizationYearRating> qw = new QueryWrapper<>();
		if (StringUtils.isNotEmpty(yearRatingCode)) {
			qw.lambda().like(OrganizationYearRating::getYearRatingCode,yearRatingCode);
		}

		if (StringUtils.isNotEmpty(year)) {
			qw.lambda().eq(OrganizationYearRating::getYear,year);
		}

		OrganizationYearRating yearRating = baseMapper.selectOne(qw);

		return yearRating;
	}

	@Override
	public boolean saveyearRating(OrganizationYearRating yearRating) {

		StringBuilder sbcode = new StringBuilder("ndpj_").append(yearRating.getYear());
		if(null !=yearRating && null == yearRating.getId()){
			sbcode.append("_001");
			yearRating.setStage("流程未启动");
			yearRating.setYearRatingCode(sbcode.toString());
		}else {
			OrganizationYearRating one = getOneYearRating(yearRating.getYearRatingCode(),yearRating.getYear());
			if(null != one){
				throw new CustomException("已存在"+yearRating.getYear()+"年度评级");
			}
		}
		boolean b = this.saveOrUpdate(yearRating);

		return b;
	}
}
