package com.ims.eval.service.impl;

import com.alibaba.fastjson.JSON;
import com.alibaba.fastjson.JSONObject;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.ims.common.utils.MD5Utils;
import com.ims.common.utils.RSAUtils;
import com.ims.common.utils.StringUtils;
import com.ims.eval.config.CustomException;
import com.ims.eval.config.ImaConfig;
import com.ims.eval.dao.UserMapper;
import com.ims.eval.entity.Myuser;
import com.ims.eval.entity.dto.response.MyuserResDTO;
import com.ims.eval.feign.RemoteServiceBuilder;
import com.ims.eval.service.IUserService;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.http.HttpEntity;
import org.springframework.http.HttpMethod;
import org.springframework.http.ResponseEntity;
import org.springframework.stereotype.Service;
import org.springframework.util.LinkedMultiValueMap;
import org.springframework.util.MultiValueMap;
import org.springframework.web.client.RestTemplate;
import springfox.documentation.annotations.Cacheable;

import javax.servlet.http.HttpServletRequest;
import java.util.Base64;
import java.util.List;

/**
 * <p>
 * 服务实现类
 * </p>
 *
 * @author wang
 * @since 2023-03-03
 */
@Service
@Slf4j
public class UserServiceImpl extends ServiceImpl<UserMapper, Myuser> implements IUserService {


	@Autowired
	private RemoteServiceBuilder remoteServiceBuilder;

	@Autowired
	private RestTemplate restTemplate;

	@Autowired
	private ImaConfig imaConfig;

	@Override
	public List<Myuser> listAll(String id, String orgId, String unitId) {

		QueryWrapper<Myuser> qw = new QueryWrapper<>();

		if (StringUtils.isNotEmpty(id)) {
			qw.lambda().eq(Myuser::getId, id);
		}

//		if (StringUtils.isNotEmpty(orgId)) {
//			qw.lambda().like(Myuser::getOrgId, orgId);
//		}
//
//		if (StringUtils.isNotEmpty(unitId)) {
//			qw.lambda().like(Myuser::getUnitId, unitId);
//		}

		List<Myuser> list = baseMapper.selectList(qw);

		return list;
	}

	@Override
	public Myuser getUser(String username) {
		QueryWrapper<Myuser> qw = new QueryWrapper<>();
		qw.lambda().eq(Myuser::getLoginName, username);
		List<Myuser> list = baseMapper.selectList(qw);
		if (list.size() <= 0) {
			return null;
		}
		return list.get(0);
	}

	@Override
	public void updateUser(Myuser user) {
		super.saveOrUpdate(user);
	}

	@Cacheable(value = "user_code")
	@Override
	public MyuserResDTO getSysUser(String code) {
		try {
			Object json = remoteServiceBuilder.getGatewayUrl().getSysUser(code);
			MyuserResDTO user = JSON.parseObject(json.toString(), MyuserResDTO.class);
			return user;
		} catch (Exception e) {
			return null;
		}
	}

	@Override
	public MyuserResDTO getSysUser(HttpServletRequest request) {

		try {
			if (null == request.getHeader("code")) {
				throw new CustomException("请求鉴权信息为空");
			}
			String code = request.getHeader("code");
			JSONObject json = remoteServiceBuilder.getGatewayUrl().getSysUser(code);
			MyuserResDTO user = JSONObject.parseObject(json.getJSONObject("data").toJSONString(), MyuserResDTO.class);
			return user;
		} catch (Exception e) {
			e.printStackTrace();
			return null;
		}
	}

	@Override
	public JSONObject getCodeByToken(String token) {
		JSONObject json = remoteServiceBuilder.getGatewayUrl().getCodeByToken(token);
		return json;
	}


	@Override
	public JSONObject getbladeAuth(String tenantId, String username, String password, String grantType, String scope, String type) throws Exception {
		MultiValueMap<String, String> headers = new LinkedMultiValueMap<>();
		headers.add("Authorization", "Basic c2FiZXI6NWMwMmMzZDViNzYxNTNkZGM5ZTczYzc5YzMzNzYzODAxNmE5ZDM2Ng==");
		headers.add("Tenant-Id", "000000");


		// 使用后端私钥解密数据
		byte[] pwdData = RSAUtils.decrypt(Base64.getDecoder().decode(password.replaceAll(" ", "+")), RSAUtils.getKeyPair().getPrivate());

		String pwdstr = MD5Utils.md5Encrypt(new String(pwdData));
		// 数字签名
		byte[] signature = RSAUtils.sign(password.getBytes(), RSAUtils.getKeyPair().getPrivate());

		// 验证数字签名
		boolean isValid = RSAUtils.verify(password.getBytes(), signature, RSAUtils.getKeyPair().getPublic());
		if (!isValid) {
			throw new CustomException("登录密码验证错误");
		}

		HttpEntity<LinkedMultiValueMap<String, Object>> param = new HttpEntity<>(null, headers);

		ResponseEntity<String> responseEntity2 = restTemplate.postForEntity(imaConfig.getGatewayUrl() + "blade-auth/oauth/token?tenantId={1}&username={2}&password={3}&grant_type={4}&scope={5}&type={6}",
			param, String.class, tenantId, username, pwdstr, grantType, scope, type);
		log.info("\n code:{}\n header:{}\n body:{}\n", responseEntity2.getStatusCodeValue(), responseEntity2.getHeaders(), responseEntity2.getBody());
		if (200 == responseEntity2.getStatusCodeValue()) {
			return JSON.parseObject(responseEntity2.getBody());
		}
		return null;
	}

	@Override
	public JSONObject pageList(Integer current, Integer size, String orgId, String idCard, String mobile, String no, String name, String loginName, HttpServletRequest request) {

		MultiValueMap<String, String> headers = new LinkedMultiValueMap<>();
		headers.add("Blade-Auth", "bearer " + request.getHeader("Blade-Auth"));
		HttpEntity<LinkedMultiValueMap<String, Object>> param = new HttpEntity<>(null, headers);

		ResponseEntity<String> responseEntity2 = restTemplate.exchange(imaConfig.getGatewayUrl() + "ims-idms/sysuser/page?current={1}&size={2}&orgId={3}&idCard={4}&mobile={5}&no={6}&name={7}&loginName={8}",
			HttpMethod.GET, param, String.class, current, size, orgId, idCard, mobile, no, name, loginName);

		log.info("\n code:{}\n header:{}\n body:{}\n", responseEntity2.getStatusCodeValue(), responseEntity2.getHeaders(), responseEntity2.getBody());
		//
		if (200 == responseEntity2.getStatusCodeValue()) {

			return JSON.parseObject(responseEntity2.getBody());
		}
		return null;

	}


	@Override
	public JSONObject getbladeAuth(String username, String password) throws Exception {
		MultiValueMap<String, String> headers = new LinkedMultiValueMap<>();
		headers.add("Content-Type", "application/x-www-form-urlencoded");

		//固定字符串
		String auth = "c2FiZXI6NWMwMmMzZDViNzYxNTNkZGM5ZTczYzc5YzMzNzYzODAxNmE5ZDM2Ng==";
		// 使用后端私钥解密数据
		byte[] pwdData = RSAUtils.decrypt(Base64.getDecoder().decode(password.replaceAll(" ", "+")), RSAUtils.getKeyPair().getPrivate());

		String pwdstr = MD5Utils.md5Encrypt(new String(pwdData));
		// 数字签名
		byte[] signature = RSAUtils.sign(password.getBytes(), RSAUtils.getKeyPair().getPrivate());

		// 验证数字签名
		boolean isValid = RSAUtils.verify(password.getBytes(), signature, RSAUtils.getKeyPair().getPublic());
		if (!isValid) {
			throw new CustomException("登录密码验证错误");
		}

		HttpEntity<LinkedMultiValueMap<String, Object>> param = new HttpEntity<>(null, headers);

		ResponseEntity<String> responseEntity2 = restTemplate.postForEntity(imaConfig.getGatewayUrl() + "ims-idas/oauth/IToken?userInfo={1}&password={2}&auth={3}",
			param, String.class, username, pwdstr, auth);
		log.info("\n code:{}\n header:{}\n body:{}\n", responseEntity2.getStatusCodeValue(), responseEntity2.getHeaders(), responseEntity2.getBody());
		if (200 == responseEntity2.getStatusCodeValue()) {
			return JSON.parseObject(responseEntity2.getBody());
		}
		return null;
	}

	@Override
	public JSONObject getLoginNameByUserInfo(String info, HttpServletRequest request) {
		MultiValueMap<String, String> headers = new LinkedMultiValueMap<>();
		headers.add("Authorization", "Basic " + request.getHeader("Blade-Auth"));
		HttpEntity<LinkedMultiValueMap<String, Object>> param = new HttpEntity<>(null, headers);
		ResponseEntity<String> responseEntity2 = restTemplate.exchange(imaConfig.getGateway2Url() + "api/ims-idms/sysuser/getLoginNameByUserInfo?info={1}", HttpMethod.GET, param, String.class, info);
		log.info("\n code:{}\n header:{}\n body:{}\n", responseEntity2.getStatusCodeValue(), responseEntity2.getHeaders(), responseEntity2.getBody());
		if (200 == responseEntity2.getStatusCodeValue()) {
			return JSON.parseObject(responseEntity2.getBody());
		}
		return null;
	}
}
